# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module gtk.glade...'
	import gtk.glade

	print 'Importing module time...'
	import time

	print 'Importing module winki.lib...'
	import winki.lib

except:
	error_quit(0, 1)

class BaseAlert:
	def __init__(self, widget_name):
		self.xml = None
		self.state = 'unknown'
		self.__glade_file = winki.lib.build_path([winki.lib.winki_path, 'glade', 'winki_rip_gtk2.glade'])

		try:
			self.xml = gtk.glade.XML(self.__glade_file, widget_name)
		except:
			print 'BaseAlert: Cannot load %s from %s' % (widget_name, glade_file)
			raise SystemExit

		dic = self.get_dictionary()
		self.xml.signal_autoconnect(dic)

		winki.lib.check_widgets(self.xml, self.get_widgets())

		self.__dialog = self.xml.get_widget(widget_name)
		self.__dialog.connect('delete_event', self.no_quit)
		return

	def no_quit(self, button, event = None):
		"disallow close"
		return True

	def execute(self):
		self.__dialog.set_modal(True)
		self.__dialog.show()

		while self.state == 'unknown':
			time.sleep(0.001)
			while gtk.events_pending():
				gtk.main_iteration(False)
		return

	def hide(self):
		self.__dialog.hide()
		return True

	def reset(self):
		return

class AlertWarning(BaseAlert):
	def __init__(self):
		BaseAlert.__init__(self, 'alert_warning')
		return

	def get_dictionary(self):
		return {'on_button_alert_cancel_clicked' : self.on_cancel,
				'on_button_alert_affirmative_clicked' : self.on_ok}

	def get_widgets(self):
		return ['label_alert',
	            'button_alert_cancel',
				'button_alert_affirmative']

	def show(self, primary_text, secondary_text, button_text):
		self.state = 'unknown'

		text = ('<span weight="bold" size="larger">%s</span>\n\n%s' % (primary_text, secondary_text))

		label = self.xml.get_widget('label_alert')
		label.set_markup(text)

		button = self.xml.get_widget('button_alert_affirmative')
		winki.lib.set_button_label(button, button_text)

		self.execute()
		return True

	def on_ok(self, widget, event = None):
		self.state = 'ok'
		self.hide()
		return True

	def on_cancel(self, widget, event = None):
		self.state = 'cancel'
		self.hide()
		return True

class AlertError(BaseAlert):
	def __init__(self):
		BaseAlert.__init__(self, 'alert_error')
		return

	def get_dictionary(self):
		return {'on_button_ok_clicked' : self.on_ok}

	def get_widgets(self):
		return ['label_error',
				'button_ok']

	def show(self, primary_text, secondary_text):
		self.state = 'unknown'

		text = ('<span weight="bold" size="larger">%s</span>\n\n%s' % (primary_text, secondary_text))

		label = self.xml.get_widget('label_error')
		label.set_markup(text)

		self.execute()
		return True

	def on_ok(self, widget, event = None):
		self.state = 'ok'
		self.hide()
		return True

