# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module os...'
	import os

	print 'Importing module string...'
	import string

	print 'Importing module gtk...'
	import gtk

	print 'Importing module winki.lib...'
	import winki.lib

except:
	error_quit(0, 1)

class UserCommand:
	def __init__(self, line, type, pass_nr, fix_spaces = True):
		# fix white spaces
		if fix_spaces:
			parts = string.split(line)		
			self.line = string.join(parts)
		else:
			self.line = line

		self.type = type
		self.pass_nr = pass_nr
		self.chunk_nr = -1
		self.enabled = True
		return

	def get_line(self):
		return self.line

	def get_type(self):
		return self.type

	def get_pass_nr(self):
		return self.pass_nr

	def get_chunk_nr(self):
		return self.chunk_nr

	def get_enabled(self):
		return self.enabled

	def set_chunk_nr(self, chunk_nr):
		self.chunk_nr = chunk_nr
		return

	def set_pass_nr(self, pass_nr):
		self.pass_nr = pass_nr
		return

	def set_enabled(self, enabled):
		self.enabled = enabled
		return

class Builder:
	def __init__(self, project):
		self.project = project
		self.msg = {'Fbc' : 'Builder.build_commands()',
			    'Fbco' : 'Builder.build_copy()',
			    'Fbcp' : 'Builder.build_chapters()',
			    'Fbp' : 'Builder.build_pass()',
			    'F2p' : 'Builder.build_2pass()',
			    'F3p' : 'Builder.build_3pass()',
			    'Fmp' : 'Builder.build_multi_pass()',
			    'Ech' : 'start chapter > end chapter',
			    'Ect' : 'Project.current_title < 1',
			    'Efail' : 'command construction failed',
			    'Elen' : 'invalid length',
			    'Emeth' : 'unknown method',
			    'Enrch' : 'Project.number_chunks < 1',
			    'Eor' : 'index out of range',
			    'Esrc' : 'unknown source'
			    }
		return

	def build_crop_command(self):
		opts = '-wid %d -x %d -y %d -zoom -v -vop cropdetect -forcedsubsonly -noautosub -slave'
		
		program = self.get_prog_name('mplayer')
		src = self.get_source_name()
		chapters = self.build_chapters(0)

		alist = self.project.get_current_audio()
		aid = ''
		if len(alist):
			aindex = alist[0] # danger (todo)
			aid = self.build_aid(aindex)

		cmd = ''
		if self.project.get_source() == 'dvd':
			cmd = '%s %s %s %s %s' % (program, src, opts,  aid, chapters)

		elif self.project.get_source() == 'file':
			cmd = '%s %s %s %s' % (program, src, opts, aid)

		elif self.project.get_source() == 'vcd':
			cmd = '%s %s %s %s' % (program, src, opts, aid)

		else:
			# new source?
			assert(0)

		items = string.split(cmd)
		cmd = string.join(items)
		return cmd

	def build_test_command(self, aid_index, sid_index):
		slangs = self.project.get_current_subtitles()
		alangs = self.project.get_current_audio()
		if len(slangs) != 3:
			winki.lib.debug('Builder.build_test_command()', 'unexpected subitles length', winki.lib.error)
			return ''

		if len(alangs) != 3:
			winki.lib.debug('Builder.build_test_command()', 'unexpected audio length', winki.lib.error)
			return ''

		opts = ' -wid %d -x %d -y %d -zoom -slave'
		nosub = ' -forcedsubsonly -noautosub'

		program = self.get_prog_name('mplayer')
		src = self.get_source_name()
		cmd = '%s %s' % (program, src)

		source = self.project.get_source()
		if source == 'dvd':
			cmd += ' '
			cmd += opts
			cmd += ' '
			cmd += self.build_aid(alangs[aid_index])
			cmd += ' '
			cmd += self.build_chapters(0)

			if sid_index < len(slangs):
				sid = slangs[sid_index]
				if sid > 0:
					cmd += (' -sid %d' % (sid - 1))
				else:
					cmd += nosub
				
		elif source == 'file' or source == 'vcd':
			cmd += ' '
			cmd += self.build_aid(alangs[aid_index])
			cmd += ' '
			cmd += opts
			cmd += nosub

		else:
			# new source?
			assert(0)

		if self.project.has_crop():
			c = self.project.get_crop()
			cmd += ' -vf rectangle=%d:%d:%d:%d' % (c[0], c[1], c[2], c[3])

		items = string.split(cmd)
		cmd = string.join(items)
		return cmd

	def build_identify_command(self, chunk_nr = -1):
		program = self.get_prog_name('mplayer')
		src = self.get_source_name()
		chapters = ''
		if chunk_nr > -1:
			chapters = self.build_chapters(chunk_nr)
		options = '-identify -frames 0 -v -nocache'

		return ('%s %s %s %s' % (program, src, chapters, options))

	def build_scale_detect_command(self, scale_x):
		program = self.get_prog_name('mplayer')
		src = self.get_source_name()
		vf = '-vf scale=%d:-2' % scale_x
		options = '-identify -frames 1 -v -vo null -ao null -nocache'

		return ('%s %s %s %s' % (program, src, vf, options))

	def build_action_commands(self):
		encoding = self.project.get_encoding()
		if encoding == 'multi pass':
			return self.build_multi_pass()

		elif encoding == 'copy':
			return self.build_copy()

		elif encoding == 'vcd' or encoding == 'svcd':
			return self.build_vcd()

		else:
			print encoding
			# new encoding?
			assert(0)

		return []

	def build_copy(self):
		result = []

		if self.project.get_number_chunks() < 1:
			winki.lib.debug(self.msg['Fbco'], 'Project.number_chunks < 1', winki.lib.error)
			return result

		chunk_nr = 0
		while chunk_nr < self.project.get_number_chunks():
			commands = []
			success = self.build_copy_commands(chunk_nr, commands)

			if not success:
				winki.lib.debug('Builder.build_copy()', ('command construction for chunk %d failed' % chunk_nr), winki.lib.error)
				chunk_nr += 1
				for c in commands:
					print c
				continue

			for command in commands:
				command.set_chunk_nr(chunk_nr)
				result.append(command)

			chunk_nr += 1

		return result

	def build_copy_commands(self, chunk_nr, result):

		success = True

		success &= self.create_audio_cmds(chunk_nr, result)
		success &= self.create_sub_cmds(chunk_nr, result)
		success &= self.build_dump_video_command(chunk_nr, result)
		success &= self.build_fix_command(chunk_nr, result)
		success &= self.build_unlink_command(chunk_nr, result)

		if self.project.get_mix_streams():
			success &= self.create_merge_cmd(chunk_nr, result)
			success &= self.create_unlink_cmd(chunk_nr, result)

		return success


	def build_vcd(self):
		result = []

		if self.project.get_number_chunks() < 1:
			winki.lib.debug(self.msg['Fbco'], 'Project.number_chunks < 1', winki.lib.error)
			return result

		encoding = self.project.get_encoding()
		chunk_nr = 0
		while chunk_nr < self.project.get_number_chunks():
			commands = []
			success = False
			if encoding == 'vcd':
				success = self.build_vcd_commands(chunk_nr, commands)
			elif encoding == 'svcd':
				success = self.build_svcd_commands(chunk_nr, commands)
			else:
				assert(0)

			if not success:
				winki.lib.debug('Builder.build_vcd()', ('command construction for chunk %d failed' % chunk_nr), winki.lib.error)
				chunk_nr += 1
				for c in commands:
					print c
				continue

			for command in commands:
				command.set_chunk_nr(chunk_nr)
				result.append(command)

			chunk_nr += 1

		return result

	def build_vcd_commands(self, chunk_nr, result):
		success = True

		success &= self.create_video_cmds(chunk_nr, result)
		success &= self.create_mplex_cmds(chunk_nr, result)
		success &= self.build_image(chunk_nr, result)
		success &= self.create_unlink_cmd(chunk_nr, result)
		success &= self.create_unlink_aux_cmd(result)

		return success

	def build_svcd_commands(self, chunk_nr, result):
		success = True

		success &= self.create_vbitrate_cmd(chunk_nr, result)
		success &= self.create_video_cmds(chunk_nr, result)
		success &= self.create_mplex_cmds(chunk_nr, result)
		success &= self.build_image(chunk_nr, result)
		success &= self.create_unlink_cmd(chunk_nr, result)
		success &= self.create_unlink_aux_cmd(result)

		return success

	def build_image(self, chunk_nr, result):
		program = self.get_prog_name('vcdimager')
		src = self.build_dest_name(chunk_nr, 'mpeg')
		cue = self.build_dest_name(chunk_nr, 'cue')
		bin = self.build_dest_name(chunk_nr, 'bin')
		label = self.project.get_name(True)
		nr_chunks = self.project.get_number_chunks()

		kind = ''
		encoding = self.project.get_encoding()
		if encoding == 'vcd':
			kind = 'vcd2'
		elif encoding == 'svcd':
			kind = 'svcd'
		else:
			assert(0)

		cmd = '%s -t %s -c %s -b %s -l "%s" --volume-count %d --volume-number %d %s' % (program, kind, cue, bin, label, nr_chunks, (chunk_nr + 1), src)

		ucmd = UserCommand(cmd, 'build image', chunk_nr)
		result.append(ucmd)

		return True

	def build_dump_video_command(self, chunk_nr, result):
		program = self.get_prog_name('mencoder')
		src = self.get_source_name()
		dest = self.build_output(0, chunk_nr)
		chapters = self.build_chapters(chunk_nr)
		vf = self.build_video_filter(True)
		info = self.build_info('avi')

		opt = '-oac copy -ovc copy'
		usr = self.project.get_str_user_defined()

		if not program or not src:
			winki.lib.debug('Builder.build_dump_video_command()', 'command construction failed', winki.lib.error)
			return False

		cmd = '%s %s %s %s %s %s %s %s' % (program, src, chapters, opt, vf, usr, info, dest)

		ucmd = UserCommand(cmd, 'dump video', chunk_nr)
		result.append(ucmd)

		return True

	def build_fix_command(self, chunk_nr, result):
		program = self.get_prog_name('mencoder')
		src = self.build_dest_name(chunk_nr)
		src = winki.lib.append_to_filename(src, '_not_fixed')
		src = winki.lib.set_extension(src, 'avi')
		dest = self.build_output(1, chunk_nr)

		cp = '-forceidx -oac copy -ovc copy'

		if not program or not src:
			winki.lib.debug('Builder.build_fix_command()', 'command construction failed', winki.lib.error)
			return False

		cmd = '%s %s %s %s' % (program, src, cp, dest)

		ucmd = UserCommand(cmd, 'fix stream', chunk_nr)
		result.append(ucmd)

		return True

	def build_unlink_command(self, chunk_nr, result):
		fname = self.build_dest_name(chunk_nr)
		fname = winki.lib.append_to_filename(fname, '_not_fixed')
		fname = winki.lib.set_extension(fname, 'avi')

		ucmd = UserCommand([fname], 'unlink', chunk_nr, False)
		result.append(ucmd)

		return True

	def build_multi_pass(self):
		result = []
		if self.project.get_number_chunks() < 1:
			winki.lib.debug('Builder.build_multi_pass()', 'Project.number_chunks < 1', winki.lib.error)
			return result

		chunk_nr = 0
		while chunk_nr < self.project.get_number_chunks():
			commands = []
			success = self.create_multipass_commands(chunk_nr, commands)

			if not success:
				winki.lib.debug('Builder.build_multi_pass()', ('command construction for chunk %d failed' % chunk_nr), winki.lib.error)
				chunk_nr += 1
				for c in commands:
					print c
				continue

			for command in commands:
				command.set_chunk_nr(chunk_nr)
				result.append(command)

			chunk_nr += 1

		return result

	def create_multipass_commands(self, chunk_nr, result):
		success = True

		success &= self.create_audio_cmds(chunk_nr, result)
		success &= self.create_sub_cmds(chunk_nr, result)
		success &= self.create_vbitrate_cmd(chunk_nr, result)
		success &= self.create_video_cmds(chunk_nr, result)
		success &= self.create_merge_cmd(chunk_nr, result)
		success &= self.create_unlink_cmd(chunk_nr, result)
		success &= self.create_unlink_aux_cmd(result)

		return success

	def create_sub_cmds(self, chunk_nr, result):
		if self.project.get_source() == 'file' or self.project.get_mix_subtitles():
			return True

		langs = self.project.get_final_subtitles()

		count = 0
		for i in range(len(langs)):
			# index 0 means 'none'
			if langs[i] < 1:
				continue

			dump_cmd = self.build_dump_sub(chunk_nr, i, langs[i])

			if not dump_cmd: 
				winki.lib.debug('Builder.create_sub_cmds()', 'sub command build failed', winki.lib.error)
				return None

			ucmd = UserCommand(dump_cmd, 'dump subtitles', count)
			result.append(ucmd)

			count += 1
				
		return (count == self.project.get_sub_count())

	def has_subtitles(self):
		slangs = self.project.get_final_subtitles()
		mix = self.project.get_mix_subtitles()
		if len(slangs) > 0 and slangs[0] > 0 and not mix:
			return True
		return False

	def create_vbitrate_cmd(self, chunk_nr, result):
		audio = self.build_audio_stream_names(chunk_nr)

		idx = None
		sub = None
		source = self.project.get_source()
		encoding = self.project.get_encoding()
		has_subtitles = self.has_subtitles()

		if source == 'dvd':
			if encoding != 'svcd' and has_subtitles:
				idx = self.build_dest_name(chunk_nr, 'idx')
				sub = self.build_dest_name(chunk_nr, 'sub')

		elif source == 'file':
			idx, sub = self.build_ext_sub_name()

		elif source == 'vcd':
			# todo
			pass

		else:
			assert(0)

		streams = []
		for afile in audio:
			streams.append(afile)
		if idx and len(idx):
			streams.append(idx)
		if sub and len(sub):
			streams.append(sub)

		ucmd = UserCommand(streams, 'vbitrate', 0, False)
		result.append(ucmd)
		return True

	def build_ext_sub_name(self):
		idx = ''
		sub = ''

		if not self.project.get_consider_ext_sub():
			return [idx, sub]

		fname = self.project.get_external_subtitle()
		if not fname or not len(fname):
			return [idx, sub]

		fname = winki.lib.remove_quotes(fname)
		name, ext = os.path.splitext(fname)

		idx_name = winki.lib.set_extension(fname, 'idx')
		sub_name = winki.lib.set_extension(fname, 'sub')
		idx_name = winki.lib.set_quotes(idx_name)
		sub_name = winki.lib.set_quotes(sub_name)

		if winki.lib.is_file(idx_name) and winki.lib.is_readable(idx_name):
			idx = idx_name

		if winki.lib.is_file(sub_name) and winki.lib.is_readable(sub_name):
			sub = sub_name

		return [idx, sub]

	def create_video_cmds(self, chunk_nr, result):
		encoding = self.project.get_encoding()
		for pass_nr in range(2):
			program = self.get_prog_name('mencoder')
			src = self.get_source_name()
			chapters = self.build_chapters(chunk_nr)

			audio = ''
			if encoding == 'multi pass' or encoding == 'copy':
				audio = '-oac copy'
			elif encoding == 'vcd' or encoding == 'svcd':
				fps = self.project.get_vcd_fps()
				if fps < 0:
					assert(0)
				alangs = self.project.get_current_audio()
				if len(alangs) > 0:
					aid = self.build_aid(alangs[0]) # just one audio
					audio = '-ofps %.2f %s -oac lavc' % (fps, aid)
				else:
					audio = '-ofps %.2f -oac lavc' % fps
			else:
				assert(0)

			video = self.build_video(pass_nr)
			vf = self.build_video_filter(False)

			srate = ''
			if encoding == 'vcd' or encoding == 'svcd':
				srate = '-srate 44100'

			channels = ''
			if encoding == 'svcd':
				channels = '-channels 2'
				
			output = self.build_output(pass_nr, chunk_nr)

			info = ''
			if pass_nr == 1 and encoding == 'multi pass':
				info = self.build_info('avi')

			usr = ''
			if encoding == 'multi pass':
				usr = self.project.get_str_user_defined()

			cmd = ('%s %s %s %s %s %s %s %s %s %s %s' % (program, src, chapters, audio, video, vf, usr, info, srate, channels, output))

			vpass ='video pass %d' % (pass_nr + 1)

			ucmd = UserCommand(cmd, vpass, 0)
			result.append(ucmd)

		return True

	def create_mplex_cmds(self, chunk_nr, result):
		program = self.get_prog_name('Multiplexer')
		src = self.build_dest_name(chunk_nr, 'mpeg')
		dest = winki.lib.append_to_filename(src, '_mplexed')

		encoding = self.project.get_encoding()
		kind = ''
		if encoding == 'vcd':
			kind = 'VCD'
		elif encoding == 'svcd':
			kind = 'SVCD'
		else:
			assert(0)

		cmd = ('%s -t %s -o %s %s') % (program, kind, dest, src)

		ucmd = UserCommand(cmd, 'mplex', 0)
		result.append(ucmd)

		fnames = [dest, src]
		ucmd = UserCommand(fnames, 'rename', 0, False)
		result.append(ucmd)
		
		# todo: ugly, ugly, ugly
		#if fnames[0][ : 4] != 'tmp_':
		#	head, tail = os.path.split(fnames[0])
		#	glob = os.path.join(head, 'tmp_*')
		#	ucmd = UserCommand([glob], 'unlink', 0, False)
		#	result.append(ucmd)
			
		return True

	def create_merge_cmd(self, chunk_nr, result):
		program = self.project.get_stream_merger_name()

		merger = os.path.basename(program)
		xt = ''
		aspect_ratio = ''
		if merger == 'ogmmerge':
			xt = 'ogm'

		elif merger == 'mkvmerge':
			xt = 'mkv'
			ratio = self.project.get_aspect_ratio()
			if ratio:
				# the video track is the first track (TID 0)
				aspect_ratio = '--aspect-ratio 0:%s' % ratio

		else:
			print 'unknown stream merger %s' % merger
			assert(0)

		dest = self.build_dest_name(chunk_nr, xt)

		vstream = self.build_video_stream_name(chunk_nr)
		audio = self.build_audio_stream_names(chunk_nr)
		sub_idx = ''
		if self.has_subtitles():
			sub_idx, sub_sub = self.build_subtitle_stream_name(chunk_nr)
		info = self.build_info(xt)

		astreams = ''
		for afile in audio:
			astreams += (' %s' % afile)

		if merger == 'ogmmerge':
			command = ('%s -o %s -A %s %s %s' % (program, dest, vstream, info, astreams))

			ucmd = UserCommand(command, 'merge streams', 0)
			result.append(ucmd)

		elif merger == 'mkvmerge':
			command = ('%s -o %s %s -A %s %s %s %s' % (program, dest, aspect_ratio, vstream, astreams, sub_idx, info))

			ucmd = UserCommand(command, 'merge streams', 0)
			result.append(ucmd)

		else:
			print 'unknown stream merger %s' % merger
			assert(0)

		return True

	def create_unlink_cmd(self, chunk_nr, result):
		streams = []

		if self.project.get_encoding() != 'vcd':
			# temp hack: see changelog for version 0.3.7
			streams = self.build_audio_stream_names(chunk_nr)
			#streams = self.build_audio_stream_names(-1)
			# end temp hack

		merger_program = self.project.get_stream_merger_name()
		merger = os.path.basename(merger_program)
		if  merger == 'mkvmerge':
			sub_idx, sub_sub = self.build_subtitle_stream_name(chunk_nr)
			if sub_idx and len(sub_idx):
				streams.append(sub_idx)
			if sub_sub and len(sub_sub):
				streams.append(sub_sub)
			# unlink the attachment after the last chunk has been merged
			attachment = self.build_dest_name(0, 'txt')
			nr_chunks = self.project.get_number_chunks()
			if winki.lib.is_file(attachment) and chunk_nr == (nr_chunks - 1):
				streams.append(attachment)

		video = self.build_video_stream_name(chunk_nr)
		streams.append(video)

		ucmd = UserCommand(streams, 'unlink', chunk_nr, False)
		result.append(ucmd)

		return True

	def create_unlink_aux_cmd(self, result):
		ucmd = UserCommand('', 'unlink aux', -1, False)
		result.append(ucmd)
		return True

	def build_dest_name(self, chunk_nr, ext = None):
		fname = self.project.get_file_output_name(chunk_nr)
		fname = winki.lib.set_quotes(fname)
		return winki.lib.set_extension(fname, ext)

	def build_video_stream_name(self, chunk_nr):
		result = ''

		encoding = self.project.get_encoding()
		if encoding == 'vcd' or encoding == 'svcd':
			result = self.build_dest_name(chunk_nr, 'mpeg')
		else:
			result = self.build_dest_name(chunk_nr, 'avi')

		return result

	def build_audio_stream_names(self, chunk_nr):
		ext = ''
		encoding = self.project.get_encoding()
		if encoding != 'copy':
			ext = self.project.get_audio_compressor_ext()
		else:
			ext = 'wav'

		result = []
		internal = self.build_internal_audio_stream_names(chunk_nr, ext)
		for afile in internal:
			result.append(afile)

		external = self.build_external_audio_stream_names(ext, 'dest')
		for afile in external:
			result.append(afile)

		return result

	def build_subtitle_stream_name(self, chunk_nr):
		source = self.project.get_source()

		idx = ''
		sub = ''
		if source == 'dvd' and self.has_subtitles():
			idx = self.build_dest_name(chunk_nr, 'idx')
			sub = self.build_dest_name(chunk_nr, 'sub')
		else:
			pass

		return [idx, sub]

	def build_dump_sub(self, chunk_nr, out_id, sub_idx):
		program = self.get_prog_name('mencoder')
		src = self.get_source_name()
		chapters = self.build_chapters(chunk_nr)
		sid = ('-sid %d' % (sub_idx - 1))
		out = self.build_vobsubout(chunk_nr, out_id)
		av = ' -ovc copy -oac copy -o /dev/null'

		if not program or not src:
			winki.lib.debug('Builder.build_dump_sub()', 'command construction failed', winki.lib.error)
			return None

		return ('%s %s %s %s %s %s' % (program, src, chapters, sid, out, av))

	def build_vobsubout(self, chunk_nr, out_id):
		name = self.build_dest_name(chunk_nr)
		return '-vobsubout %s -vobsuboutindex %d' % (name, out_id)

	def get_prog_name(self, program):
		result = self.project.get_tool_locations([program])
		if not result or not len(result):
			assert(0)
			return ''

		return result[0]

	def get_source_name(self):
		source = self.project.get_source()

		if source == 'dvd':
			title = self.project.get_current_title()
			if title < 0:
				winki.lib.debug('Builder.get_source_name()', 'current title not set', winki.lib.error)
				return ''
			dvd_dev = self.project.get_dvd_device()
			return ('dvd://%d -dvd-device %s' % ((title + 1), dvd_dev))

		elif source == 'file':
			fname = winki.lib.remove_quotes(self.project.get_source_filename())
			if not fname:
				winki.lib.debug('Builder.get_source_name()', 'invalid file name', winki.lib.error)
				return ''
			return winki.lib.set_quotes(fname)

		elif source == 'vcd':
			title = self.project.get_current_title()
			if title < 0:
				winki.lib.debug('Builder.get_source_name()', 'current title not set', winki.lib.error)
				title = 1

			cd_dev = self.project.get_cd_device()
			return ('vcd://%d/%s' % ((title + 1), cd_dev))

		else:
			# new source?
			assert(0)
		return ''

	def build_video(self, pass_nr):
		video = ''

		encoding = self.project.get_encoding()
		if encoding == 'copy':
			video = ' -ovc copy'

		elif encoding == 'multi pass':
			video = ' -ovc '
			video += self.project.get_str_video_codec()
			video += ' '
			video += self.project.get_str_video_codec_options()
			if len(self.project.get_str_video_codec_options()):
				video += ':'
			else:
				return None
			video += ('vpass=%d' % (pass_nr + 1))
			video += ':vbitrate=%d'

		elif encoding == 'vcd':
			video = ' -of mpeg -ovc '
			video += self.project.get_str_vcd_codec()
			video += ' '
			video += self.project.get_str_vcd_codec_options()
			if len(self.project.get_str_vcd_codec_options()):
				video += ':'
			else:
				return None
			video += ('vpass=%d' % (pass_nr + 1))

		elif encoding == 'svcd':
			video = ' -of mpeg -ovc '
			video += self.project.get_str_svcd_codec()
			video += ' '
			opt = self.project.get_str_svcd_codec_options()
			pos = string.find(opt, 'vbitrate=value')
			if pos > -1:
				opt = string.replace(opt, 'vbitrate=value', 'vbitrate=%d')
			else:
				assert(0)
				return None

			video += opt
			if len(self.project.get_str_svcd_codec_options()):
				video += ':'
			else:
				return None
			video += ('vpass=%d' % (pass_nr + 1))

		else:
			# new encoding method?
			assert(0)

		return video

	def build_output(self, pass_nr, chunk_nr):
		output = ''
		encoding = self.project.get_encoding()
		if encoding == 'copy':
			# first pass: not fixed stream
			if pass_nr == 0:
				dest_name = self.build_dest_name(chunk_nr)
				dest_name = winki.lib.append_to_filename(dest_name, '_not_fixed')
				dest_name = winki.lib.set_extension(dest_name, 'avi')
				output = ('-o %s' % dest_name)
			# second pass: fixed stream
			else:
				dest_name = self.build_dest_name(chunk_nr, 'avi')
				output = ('-o %s' % dest_name)

		elif encoding == 'multi pass':
			if pass_nr == 0:
				output = '-o /dev/null'
			else:
				dest_name = self.build_dest_name(chunk_nr, 'avi')
				output =  ('-o %s' % dest_name)

		elif encoding == 'vcd' or encoding == 'svcd':
			if pass_nr == 0:
				output = '-o /dev/null'
			else:
				dest_name = self.build_dest_name(chunk_nr, 'mpeg')
				output =  ('-o %s' % dest_name)

		else:
			# new encoding method?
			assert(0)

		return output

	def build_chapters(self, chunk_idx):
		if self.project.get_source() != 'dvd':
			return ''

		if chunk_idx < 0 or chunk_idx > len(self.project.get_chunks()):
			winki.lib.debug(self.msg['Fbcp'], self.msg['Eor'], winki.lib.error)
			return ''

		start = self.project.get_chunks()[chunk_idx][0]
		end = self.project.get_chunks()[chunk_idx][1]

		if start > end:
			winki.lib.debug(self.msg['Fbcp'], self.msg['Ech'], winki.lib.error)
			return ''

		sub = ''
		if self.project.get_mix_subtitles():
			slangs = self.project.get_final_subtitles()
			if len(slangs) > 0 and slangs[0] > 0:
				sub = '-sid %d' % (slangs[0] - 1)

		# start and end are 0-based but mencoder expects 1-based chapter indices
		return '-chapter %d-%d %s' % (start + 1, end + 1, sub)


	def build_info(self, type):
		name_map = None

		if type == 'avi':
			name_map = {'name' : 'name="%s"',
				    'artist' : 'artist="%s"',
				    'genre' : 'genre="%s"',
				    'subject' : 'subject="%s"',
				    'copyright' : 'copyright="%s"',
				    'srcform' : 'srcform="%s"',
				    'comment' : 'comment="%s"'}

		elif type == 'ogm':
			name_map = {'name' : 'TITLE=%s',
				    'artist' : 'ARTIST=%s',
				    'genre' : 'GENRE=%s',
				    'subject' : 'SUBJECT=%s',
				    'copyright' : 'COPYRIGHT=%s',
				    'srcform' : 'SRCFORM=%s',
				    'comment' : 'COMMENT=%s'}

		elif type == 'mkv':
			name_map = {'name' : 'Title: %s\n',
				    'artist' : 'Artist: %s\n',
				    'genre' : 'Genre: %s\n',
				    'subject' : 'Subject: %s\n',
				    'copyright' : 'Copyright note: %s\n',
				    'srcform' : 'Source form: %s\n',
				    'comment' : 'Comment: %s\n'}

		else:
			print 'unknown build info type'
			assert(0)
			return ''

		items = ['', '', '', '', '', '', '']
		count = 0

		temp = self.project.get_name(True)
		if temp and len(temp):
			items[0]  = name_map['name'] % temp
			count += 1

		temp = self.project.get_artist(True)
		if temp and len(temp):
			items[1]  = name_map['artist'] % temp
			count += 1

		temp = self.project.get_genre(True)
		if temp and len(temp):
			items[2] = name_map['genre'] % temp
			count += 1

		temp = self.project.get_subject(True)
		if temp and len(temp):
			items[3] = name_map['subject'] % temp
			count += 1

		temp = self.project.get_copyright(True)
		if temp and len(temp):
			items[4] = name_map['copyright'] % temp
			count += 1

		temp = self.project.get_srcform(True)
		if temp and len(temp):
			items[5] = name_map['srcform'] % temp
			count += 1

		temp = self.project.get_comment(True)
		if temp and len(temp):
			items[6] = name_map['comment'] % temp
			count += 1

		if count == 0:
			return ''

		if type == 'avi':
			return self.build_avi_info(items)
		elif type == 'ogm':
			return self.build_ogm_info(items)
		elif type == 'mkv':
			return self.build_mkv_info(items)

		return ''

	def build_avi_info(self, items):
		result = '-info '

		for i in range(len(items)):
			if not items[i] or not len(items[i]):
				continue

			if i == 0:
				result = '%s %s' % (result, items[i])
			else:
				result = '%s:%s' % (result, items[i])

		return result

	def build_ogm_info(self, items):
		result = ''

		for item in items:
			if not item or not len(item):
				continue

			stripped = string.replace(item, '\'', '')
			result += (' -c \'%s\'' % stripped)

		return result

	def build_mkv_info(self, items):
		attachment = self.build_dest_name(0, 'txt')
		attachment = winki.lib.remove_quotes(attachment)
		try:
			fout = open(attachment, 'w')
			for item in items:
				if not item or not len(item):
					continue
				fout.write(item)
			fout.close()
		except:
			return ''

		attachment = winki.lib.set_quotes(attachment)
		return '--attachment-mime-type text/plain --attachment-description "Movie description" --attach-file %s' % attachment

	def build_video_filter(self, crop_only):
		result = ''
		vf = ''

		if crop_only:
			return result

		encoding = self.project.get_encoding()
		if encoding == 'multi pass' or encoding == 'copy':
			vf = self.project.get_str_video_filter()

		elif encoding == 'vcd':
			vf = self.project.get_str_vcd_filter()

		elif encoding == 'svcd':
			vf = self.project.get_str_svcd_filter()

		else:
			assert(0)

		crop = self.build_crop()
		scale, expand = self.build_scale_and_expand()

		pos = string.find(vf, 'crop=value')
		if pos > -1 and crop and len(crop):
			vf = string.replace(vf, 'crop=value', crop, 1)

		else:
			if pos > -1:
				vf = string.replace(vf, 'crop=value,', '', 1)

		pos = string.find(vf, 'scale=value')
		if pos > -1 and scale and len(scale):
			vf = string.replace(vf, 'scale=value', scale, 1)
		else:
			if pos > -1:
				vf = string.replace(vf, 'scale=value', '', 1)

		pos = string.find(vf, 'expand=value')
		if pos > -1 and expand and len(expand):
			vf = string.replace(vf, 'expand=value', expand, 1)
		else:
			if pos > -1:
				vf = string.replace(vf, 'expand=value', '', 1)

		if vf and len(vf):
			return ' -vf %s' % vf
		else:
			return ''

	def build_crop(self):
		crop = ''

		if not self.project.has_crop():
			return crop

		numbers = self.project.get_crop()

		crop = 'crop='
		for i in range(len(numbers)):
			if i == (len(numbers) - 1):
				crop += '%d' % numbers[i]
			else:
				crop += '%d:' % numbers[i]

		return crop

	def build_scale_and_expand(self):
		scale = ''
		expand = ''

		y = self.project.get_scale_y()
		if y < 0:
			return [scale, expand]

		width = -1
		height = -1
		encoding = self.project.get_encoding()
		if encoding == 'vcd' or encoding == 'svcd':
			width = self.project.get_vcd_width()
			height = self.project.get_vcd_height()

		if encoding == 'vcd' or encoding == 'svcd':
			if y > height:
				y = height

			scale = 'scale=%d:%d' % (width, y)
			expand = 'expand=%d:%d' % (width, height)

		return [scale, expand]

# --- audio stuff ---

	def create_internal_audio_cmds(self, chunk_nr, result):
		succ = True
		alangs = self.project.get_current_audio()

		vobsub_idx = 0
		for lang_idx in range(len(alangs)):
			# lang_idx == 0 means 'none'
			if alangs[lang_idx] < 1:
				continue

			succ &= self.build_dump_audio(chunk_nr, lang_idx, vobsub_idx, result)
			encoding = self.project.get_encoding()
			if encoding != 'copy':
				succ &= self.build_compr_audio(chunk_nr, lang_idx, vobsub_idx, result)
				succ &= self.build_unlink_audio(chunk_nr, lang_idx, vobsub_idx, result)
			if succ:
				vobsub_idx += 1

		return [succ, vobsub_idx]

	def create_external_audio_cmds(self, chunk_nr, vobsub_idx, result):
		succ = True

		external = self.project.get_external_audio()

		lang_idx = len(self.project.get_current_audio())
		for afile in external:
			if not afile or not len(afile):
				lang_idx += 1
				continue

			succ &= self.build_compr_audio(chunk_nr, lang_idx, vobsub_idx, result)
			succ &= self.build_unlink_audio(chunk_nr, lang_idx, vobsub_idx, result)

			if succ:
				vobsub_idx += 1

			lang_idx += 1

		return succ

	def create_audio_cmds(self, chunk_nr, result):

		succ, vobsub_idx = self.create_internal_audio_cmds(chunk_nr, result)
		succ &= self.create_external_audio_cmds(chunk_nr, vobsub_idx, result)
		return succ

	def build_internal_audio_stream_name(self, chunk_nr, stream_idx, ext):
		result = None

		# temp hack: see changelog for version 0.3.7
		if chunk_nr < 0:
			fname = self.build_internal_audio_name(0, stream_idx, ext)
			path, name, ext = winki.lib.split_path(fname)
			fname = winki.lib.build_path([path, 'audiodump.wav'])
		else:
			fname = self.build_internal_audio_name(chunk_nr, stream_idx, ext)
		# end temp hack

		#fname = self.build_internal_audio_name(chunk_nr, stream_idx, ext)

		if fname and len(fname):
			result = fname

		return result

	def build_internal_audio_stream_names(self, chunk_nr, ext):
		result = []

		alangs = self.project.get_current_audio()
		for stream_idx in range(len(alangs)):
			# index 0 means 'none'
			if alangs[stream_idx] < 1:
				continue

			# temp hack: see changelog for version 0.3.7
			if ext == 'wav':
				aofile = self.build_internal_audio_stream_name(-1, stream_idx, ext)
			else:
				aofile = self.build_internal_audio_stream_name(chunk_nr, stream_idx, ext)
			# end temp hack
			if aofile and len(aofile):
				result.append('%s' % aofile)

		return result


	def build_external_audio(self, offset, location, idx, ext):
		alangs = self.project.get_external_audio()
		name = alangs[idx - offset]
		if not name or not len(name):
			return ''

		if location == 'src':
			return name

		elif location == 'dest':
			d, f, x = winki.lib.split_path(name)
			outdir = self.project.get_outputdir()
			items = [outdir, f]
			result = winki.lib.build_path(items)
			result = winki.lib.set_extension(result, ext)
			result = winki.lib.set_quotes(result)

		else:
			winki.lib.debug('Builder.build_external_audio()', 'invalid location %s' % location, winki.lib.error)
			assert(0)

		return result

	def build_external_audio_stream_name(self, idx, ext, location):
		result = None

		ilen = len(self.project.get_current_audio())
		elen = len(self.project.get_external_audio())
		assert(idx >= ilen and idx < (ilen + elen))

		external = self.project.get_external_audio()
		full = external[idx - ilen]

		if location == 'src':
			result = full

		elif location == 'dest':
			dest = self.project.get_outputdir()
			d, f, x = winki.lib.split_path(full)
			items = [dest, f]
			fname = winki.lib.build_path(items)
			fname = winki.lib.set_extension(fname, ext)
			result = winki.lib.set_quotes(fname)

		else:
			print location
			assert(0)

		return result

	def build_external_audio_stream_names(self, ext, location):
		result = []

		internal = self.project.get_current_audio()
		external = self.project.get_external_audio()
		for i in range(len(external)):
			if not external[i] or not len(external[i]):
				continue

			name = self.build_external_audio_stream_name(len(internal) + i, ext, location)
			if name and len(name):
				result.append(name)

		return result

	def build_dump_audio(self, chunk_nr, lang_idx, vobsub_idx, result):
		program = self.get_prog_name('mplayer')
		src = self.get_source_name()
		# temp hack: see changelog for version 0.3.7
		dest = self.build_internal_audio_stream_name(-1, lang_idx, 'wav')
		#dest = self.build_internal_audio_stream_name(chunk_nr, lang_idx, 'wav')
		# end temp hack
		chapters = self.build_chapters(chunk_nr)

		alangs = self.project.get_current_audio()
		aid = self.build_aid(alangs[lang_idx])

		ao = ('-ao pcm:file=%s' % dest)
		vo = '-vo null -vc dummy'

		if not program or not src or not dest:
			winki.lib.debug('Builder.build_dump_audio()', 'command construction failed', winki.lib.error)
			return False

		cmd = '%s %s %s %s %s %s' % (program, src, chapters, aid, ao, vo)
		ucmd = UserCommand(cmd, 'dump audio', vobsub_idx)
		result.append(ucmd)
		return True

	def build_norm_audio(self, chunk_nr, audio_lang, lang_idx, result):
		program = self.get_prog_name('normalize')
		# temp hack: see changelog for version 0.3.7
		src = self.build_internal_audio_stream_name(-1, audio_lang, 'wav')
		#src = self.build_internal_audio_stream_name(chunk_nr, lang_idx, 'wav')
		# end temp hack

		if not program or not src:
			return False

		cmd = '%s %s' % (program, src)

		ucmd = UserCommand(cmd, 'normalize audio', lang_idx)
		result.append(ucmd)

		return True

	def build_compr_audio(self, chunk_nr, lang_idx, vobsub_idx, result):
		program = self.project.get_audio_compressor_name()
		options = self.project.get_audio_compressor_options()
		ext = self.project.get_audio_compressor_ext()

		ilen = len(self.project.get_current_audio())
		elen = len(self.project.get_external_audio())
		is_internal = (lang_idx < ilen)
		is_external = (lang_idx < (ilen + elen))

		if is_internal:
			# temp hack: see changelog for version 0.3.7
			src = self.build_internal_audio_stream_name(-1, lang_idx, 'wav')
			#src = self.build_internal_audio_stream_name(chunk_nr, lang_idx, 'wav')
			# end temp hack
		elif is_external:
			src = self.build_external_audio_stream_name(lang_idx, None, 'src')
		else:
			print 'build_compr_audio(): index %d' % lang_idx
			assert(0)

		if is_internal:
			dest = self.build_internal_audio_stream_name(chunk_nr, lang_idx, ext)
		elif is_external:
			dest = self.build_external_audio_stream_name(lang_idx, ext, 'dest')
		else:
			print 'build_compr_audio(): index %d' % lang_idx
			assert(0)

		if not program or not src or not dest:
			winki.lib.debug('Builder.build_compr_audio()', 'command construction failed', winki.lib.error)
			return False

		compressor = os.path.basename(program)
		if compressor == 'oggenc':
			cmd = '%s %s -o %s %s' % (program, options, dest, src)
			ucmd = UserCommand(cmd, 'compress audio', vobsub_idx)
			result.append(ucmd)
		elif compressor == 'lame':
			cmd = '%s %s %s %s' % (program, options, src, dest)
			ucmd = UserCommand(cmd, 'compress audio', vobsub_idx)
			result.append(ucmd)
		else:
			print 'unknow audio compressor %s' % compressor
			assert(0)

		return True

	def build_unlink_audio(self, chunk_nr, audio_lang, lang_idx, result):
		ilen = len(self.project.get_current_audio())
		elen = len(self.project.get_external_audio())
		is_internal = (lang_idx < ilen)
		is_external = (lang_idx < (ilen + elen))

		src = None
		if is_internal:
			# temp hack: see changelog for version 0.3.7
			src = self.build_internal_audio_stream_name(-1, audio_lang, 'wav')
			#src = self.build_internal_audio_stream_name(chunk_nr, audio_lang, 'wav')
			# end temp hack
		elif is_external:
			pass # don't unlink external files
		else:
			assert(0)

		if not src:
			return True

		ucmd = UserCommand([src], 'unlink', lang_idx, False)
		result.append(ucmd)

		return True

	def build_internal_audio_name(self, chunk_nr, idx, ext):

		name = self.build_dest_name(chunk_nr)
		name = winki.lib.append_to_filename(name, '-%d' % (idx + 1))
		name = winki.lib.set_extension(name, ext)

		return winki.lib.set_quotes(name)

	def build_aid(self, index):
		result = ''
		source = self.project.get_source()

		if index < 1:
			index = 1

		if source == 'dvd':
			result = ('-aid %d' % (index + 127))

		# todo:
		# mpeg:			0 - 31
		# avi/ogm:		1 - 99
		# asf/rm:		0 - 127
		# vob(ac3):		128 - 159
		# vob(lpcm):		160 - 191
		# mpeg-ts:		17 - 8190
		elif source == 'file' or source == 'vcd':
			result = ('-aid %d' % (index - 1))

		else:
			assert(0)

		return result

