# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module os...'
	import os

	print 'Importing module string...'
	import string

	#print 'Importing module select...'
	#import select

	print 'Importing module commands...'
	import commands

	print 'Importing module popen2...'
	import popen2

	print 'Importing module winki.lib...'
	import winki.lib

except:
	error_quit(0, 1)

class Command:
	def __init__(self, command, callback = None, finish = None):
		self.command = command
		self.callback = None
		self.finish = None
		if callback and callable(callback):
			self.callback = callback
		if finish and callable(finish):
			self.finish = finish
		self.bufsize = 64
		self.cmd_obj = None
		self.pid = -1
		self.output_line = ''
		self.result = -1
		self.pure_command = ''
		return

	def build_pure_name(self):
		if not self.command or not len(self.command):
			winki.lib.debug('Command.build_pure_name()', 'no self.command', winki.lib.error)
			return

		self.pure_command = string.replace(self.command, '"', '')
		return	

	def run(self):
		self.build_pure_name()

		self.cmd_obj = popen2.Popen4(self.command)
		self.pid = self.cmd_obj.pid

		text = None
		while self.cmd_obj and self.cmd_obj.poll() == -1:
			text = self.cmd_obj.fromchild.read(self.bufsize)
			line = self.build_line(text)

			if self.callback and line:
				self.callback(line)

			while gtk.events_pending():
				gtk.main_iteration(False)

		if self.cmd_obj:
			text = self.cmd_obj.fromchild.read()
			line = self.build_line(text)

		#if self.callback and len(self.output_line):
		#	self.callback(self.output_line)

		if self.callback and line:
			self.callback(line)

		winki.lib.debug('Command.run()', 'stopping command')
		self.stop()
		if self.finish:
			winki.lib.debug('Command.run()', 'finishing command')
			self.finish()
		return

	def build_line(self, text):
		result = None

		eol_pos = string.find(text, '\n')
		eol_pos1 = string.find(text, '\r')
		if eol_pos1 > -1 and eol_pos < 0:
			eol_pos = eol_pos1

		if eol_pos > -1:
			self.output_line += text[ : eol_pos]
			result = self.output_line
			self.output_line = text[eol_pos : ]
		else:
			self.output_line += text

		return result

	def to_stdin(self, cmd):
		if not self.cmd_obj:
			winki.lib.debug('Command.to_stdin()', 'in pipe not open', winki.lib.error)
			return

		self.cmd_obj.tochild.write(cmd)
		self.cmd_obj.tochild.flush()
		return

	def kill(self):
		if self.cmd_obj:
		    self.result = self.cmd_obj.poll()

		if self.pid > -1:
		    try:
				os.kill(self.pid, 9)
		    except:
				pass

		# crude but effective...
		pid = self.find_pid()
		while pid != -1:
			try:
				os.kill(pid, 9)
			except:
				winki.lib.debug('Command.kill()', 'error killing proccess', winki.lib.error)
			pid = self.find_pid()

		if self.cmd_obj:
		    self.cmd_obj.tochild.close()
		    self.cmd_obj.fromchild.close()
		return

	def find_pid(self):
		output = commands.getoutput('ps -Af')
		lines = string.split(output, '\n')

		for line in lines:
			parts = string.split(line)
			fixed_line = string.join(parts)

			pos = string.find(fixed_line, self.pure_command)
			if pos < 0:
				continue

			pid = winki.lib.to_int(parts[1])
			if pid != -1:
				return pid

		return -1



	def stop(self):
		self.kill()

		self.cmd_obj = None
		return

class LsDVD:
	def __init__(self, project):
		dvd_dev = project.get_dvd_device()

		self.titles_command = 'lsdvd %s' % dvd_dev
		self.chapters_command = 'lsdvd -c %s' % dvd_dev
		self.audio_command = 'lsdvd -a %s' % dvd_dev
		self.subtitles_command = 'lsdvd -s %s' % dvd_dev
		self.no_err_strings = ['libdvdread: Using libdvdcss version']
		self.titles = []
		self.chapters = []
		self.chapter_times = [] # in seconds
		self.audio = []
		self.subtitles = []
		self.errors = []
		self.longest_title = -1
		return

	def stop(self):
		return

	def read_titles(self):
		self.titles = []
		self.longest_title = -1

		self.titles = self.run_command(self.titles_command)

		for title in self.titles:
			if string.find(title, 'Disc Title') > -1:
				self.titles.remove(title)

		return len(self.titles)

	def get_longest_title(self, line):
		num = 1
		#str = line[15 : -1]
		number = line[15 : ]
		try:
			num = int(number)
		except:
			pass
		return num - 1

	def read_chapters(self):
		self.chapters = []
		chapters = []

		chapters = self.run_command(self.chapters_command)

		for chapter in chapters:
			if string.find(chapter, 'Disc Title') > -1:
				chapters.remove(chapter)

		self.split_items(chapters, self.chapters, self.strip_chapter)
		
		self.get_chapter_times()
		return

	def strip_chapter(self, chapter):
		entry = string.lstrip(chapter)
		pos = string.find(entry, ', Start Cell: ')
		if pos <= 9:
			return entry

		return entry[9 : pos]

	def read_audio(self):
		self.audio = []
		audio = []

		audio = self.run_command(self.audio_command)

		for aud in audio:
			if string.find(aud, 'Disc Title') > -1:
				audio.remove(aud)

		self.split_items(audio, self.audio, self.strip_audio, True)
		return

	def strip_audio(self, text):
		start0 = string.find(text, 'Language: ')
		if start0 < 0:
			return string.lstrip(text)

		start1 = string.find(text, ' - ', start0)
		if start1 < start0:
			return text[start0 : ]

		end = string.find(text, ',', start0)
		if end < start0:
			return text[start1 + 3: ]

		start2 = string.find(text, 'Content: ')
		if start2 < end:
			return text[start1 + 3 : ]

		entry = '%s, %s' % (text[start1 + 3 : end], text[start2 : ])
		return entry


	def read_subtitles(self):
		self.subtitles = []
		subtitles = []

		subtitles = self.run_command(self.subtitles_command)
		for subtitle in subtitles:
			if string.find(subtitle, 'Disc Title') > -1:
				subtitles.remove(subtitle)

		self.split_items(subtitles, self.subtitles, self.strip_subtitles, True)
		return

	def strip_subtitles(self, text):
		pos1 = string.find(text, 'Language: ')
		if pos1 < 0:
			return string.lstrip(text)

		pos2 = string.find(text, ' - ', pos1)
		if pos2 < pos1:
			return text[pos1 + 10 : ]

		return text[pos2 + 3 : ]

	def run_command(self, command):
		self.errors = []
		line_out = None
		line_err = None
		output = []

		pipe = os.popen3(command)
		while line_out != '' and line_err != '':
			#input_ready, output_ready, error_ready = select.select([], [pipe[1]], [pipe[2]], 0.01)
			#if output_ready and len(output_ready) > 0:
			#	line_out = output_ready[0].readline()
			#	print line_out
			#if error_ready and len(error_ready) > 0:
			#	print 'reading error'
			#	line_err = error_ready[0].readline()
			line_out = pipe[1].readline()

			if line_out and len(line_out) > 1 and line_out[ : 7] != 'Longest':
				output.append(line_out[ : -1])
			elif line_out and len(line_out) > 1 and line_out[ : 7] == 'Longest':
				if self.longest_title == -1:
					self.longest_title = self.get_longest_title(line_out)

			#if line_err and len(line_err) > 0 and self.is_error(line_err):
			#	self.errors.append(line_err)

			while gtk.events_pending():
				gtk.main_iteration(False)

		pipe[0].close()
		pipe[1].close()
		pipe[2].close()
		pipe = None

		return output

	def is_error(self, line):
		for no_err in self.no_err_strings:
			if string.find(line, no_err) > -1:
				return False

		return True

	def rename_duplicates(self, lst):
		if not lst or not len(lst):
			return

		count = 1
		for i in range(len(lst)):
			j = i
			while j < (len(lst)):
				if i != j and lst[i] == lst[j]:
					lst[j] = '%s (%d)' % (lst[i], count)
					count += 1
				j += 1
					
		return

	def split_items(self, src, target, split_func, append_none = False):
		i = 0
		j = 0
		item_list = []

		if append_none:
			item_list.append(_('none'))

		while i < len(self.titles):
			if j == len(src):
				break;
			if src[j] == self.titles[i]:
				if i < (len(self.titles) - 1):
					i += 1
				j += 1
				while j < len(src):
					if src[j] == self.titles[i]:
						break;
					item_list.append(split_func(src[j]))
					j += 1

				self.rename_duplicates(item_list)
				target.append(item_list)
				item_list = []
				if append_none:
					item_list.append(_('none'))
			else:
				print 'LsDVD.split_items(): %s' % src[j]
				assert(0);
		return

	def get_chapter_times(self):
		self.chapter_times = []
		self.errors = []
		times = []

		for chapters in self.chapters:
			for chapter in chapters:
				chapter_len = self.get_chapter_len(chapter)
				if chapter_len > -1:
					times.append(chapter_len)
				else:
					self.errors.append('oops...')
			self.chapter_times.append(times)
			times = []

		if len(self.chapters) != len(self.chapter_times):
			winki.lib.debug('LsDVD.get_chapter_times()', 'invalid number of titles', winki.lib.critical)

		for i in range(len(self.chapters)):
			if len(self.chapters[i]) != len(self.chapter_times[i]):
				winki.lib.debug('LsDVD.get_chapter_times()', 'invalid number of chapters', winki.lib.critical)
		return

	def get_chapter_len(self, chapter):
		pos = string.find(chapter, 'Length: ')
		if pos < 0:
			return -1

		raw = chapter[pos + len('Length: ') : ]
		h, m, s = string.split(raw, ':')

		if string.find(s, '.') > -1:
			s, mili = string.split(s, '.')

		if winki.lib.to_int(h) == -1 or winki.lib.to_int(m) == -1 or winki.lib.to_int(s) == -1:
			return -1

		return (winki.lib.to_int(h) * 3600 + winki.lib.to_int(m) * 60 + winki.lib.to_int(s))

