# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module ConfigParser...'
	import ConfigParser

	print 'Importing module string...'
	import string

	print 'Importing module os.path...'
	import os.path

	print 'Importing module winki.rc_version...'
	import winki.rc_version

	print 'Importing module winki.lib...'
	import winki.lib

except:
	error_quit(0, 1)

class Config:
	def __init__(self):
		self.parser = None
		self.config_files = ['/etc/winkirc', '~/.winkirc']

		self.versioner = winki.rc_version.RCVersion()
		self.defaults = self.versioner.get_current()
		return

	def load(self):
		if not self.parser:
			self.parser = ConfigParser.SafeConfigParser(self.defaults)

		cfg_files = []
		for cfg_file in self.config_files:
			fixed_name = winki.lib.normalize_path(cfg_file)
			if not fixed_name or not len(fixed_name):
				continue
			cfg_files.append(fixed_name)

		try:
			self.parser.read(cfg_files)
		except:
			pass
		return

	def get_default(self, key):
		value = ''
		try:
			value = self.defaults[key]
		except:
			winki.lib.debug('Config.get_default()', 'invalid key', winki.lib.error)
		return value

	def get_string_value(self, key, section):
		if not self.parser:
			winki.lib.debug('Config.get_string_value()', 'parser not initialized', winki.lib.error)
			return ''

		value = ''
		try:
			if not self.parser.has_section(section):
				winki.lib.debug('Config.get_string_value()', 'section not found; using default')
				return self.get_default(key)

			if not self.parser.has_option(section, key):
				winki.lib.debug('Config.get_string_value()', ('%s not found; using default' % key))
				return self.get_default(key)

			value = self.parser.get(section, key)
		except:
			return value
		return value

	def load_version(self, version, fname):
		parser = None
		dic = self.versioner.get_version(version)

		if not dic:
			winki.lib.debug('Config.load_version()', 'Cannot load %s (version %s)' % (fname, version), winki.lib.error)
			return [None, None]
			
		parser = ConfigParser.SafeConfigParser(dic)
		parser.read(fname)

		return [parser, dic]

	def get_output_dir(self):
		return self.get_string_value('Output folder', 'Settings')

	def get_source(self):
		source = self.get_string_value('Default source', 'Settings')
		if source != 'dvd' and source != 'file' and source != 'vcd':
			winki.lib.debug('Config.get_source()', 'invalid source "%s"' % source, winki.lib.error)
			return 'dvd'

		return source

	def get_encoding(self):
		encoding = self.get_string_value('Default encoding', 'Settings')
		if encoding != 'multi pass' and encoding != 'copy' and encoding != 'vcd' and encoding != 'svcd':
			winki.lib.debug('Config.get_encoding()', 'invalid encoding "%s"' % encoding, winki.lib.error)
			return 'multi pass'

		return encoding

	def get_size(self):
		size = self.get_string_value('Default size', 'Settings')
		intsize = winki.lib.to_int(size)
		if intsize < 0:
			winki.lib.debug('Config.get_size()', 'no numeric size specified %s' % size, winki.lib.error)
			return 700

		return winki.lib.to_int(size)

	def get_str_preferred_audio(self):
		return self.get_string_value('Preferred audio', 'Settings')

	def get_str_preferred_subtitles(self):
		return self.get_string_value('Preferred subtitles', 'Settings')

	def get_str_user_mode(self):
		mode = self.get_string_value('User mode','Settings')
		if mode != 'Default' and mode != 'Advanced':
			winki.lib.debug('Config.get_str_user_mode()', 'unknown user mode "%s"' % mode, winki.lib.error)
			mode = 'Default'
		return mode

	def get_vcd_type(self):
		kind = self.get_string_value('VCD type', 'Settings')
		if kind != 'PAL' and kind != 'NTSC':
			winki.lib.debug('Config.get_vcd_type()', 'unknown type "%s"' % kind, winki.lib.error)
			kind = 'PAL'
		return kind

	def get_dvd_device(self):
		return self.get_string_value('DVD device', 'Settings')

	def get_cd_device(self):
		return self.get_string_value('CD-ROM device', 'Settings')

	def get_str_video_codec(self):
		return self.get_string_value('Video codec', 'Mencoder')

	def get_str_video_codec_options(self):
		return self.get_string_value('Video codec options', 'Mencoder')

	def get_str_vcd_codec(self):
		return self.get_string_value('VCD codec', 'Mencoder')

	def get_str_vcd_codec_options(self):
		return self.get_string_value('VCD options', 'Mencoder')

	def get_str_vcd_filter(self):
		return self.get_string_value('VCD filter', 'Mencoder')

	def get_str_svcd_codec(self):
		return self.get_string_value('SVCD codec', 'Mencoder')

	def get_str_svcd_codec_options(self):
		return self.get_string_value('SVCD options', 'Mencoder')

	def get_str_svcd_filter(self):
		return self.get_string_value('SVCD filter', 'Mencoder')

	def get_str_video_filter(self):
		return self.get_string_value('Video filter', 'Mencoder')

	def get_str_user_defined(self):
		return self.get_string_value('Supplement', 'Mencoder')

	def get_str_comment(self):
		return self.get_string_value('Default comment', 'Mencoder')

	def get_nice_value(self):
		nice = self.get_string_value('Nice value', 'Misc')

		value = 0
		try:
			value = int(nice)
		except:
			winki.lib.debug('Config.get_nice_value()', 'not a number', winki.lib.error)
			return 19

		if value < -20 or value > 19:
			winki.lib.debug('Config.get_nice_value()', 'invalid value %d' % value, winki.lib.error)
			value = 19
		return value

	def get_lower_threshold(self):
		threshold = self.get_string_value('Lower threshold', 'Misc')

		value = 0
		try:
			value = int(threshold)
		except:
			winki.lib.debug('Config.get_lower_threshold()', 'not a number %s' % threshold, winki.lib.error)
			value = 800

		if value < 0:
			winki.lib.debug('Config.get_lower_threshold()', 'invalid value %s' % threshold, winki.lib.error)
			value = 800

		if value > 16000:
			value = 16000

		return value

	def get_upper_threshold(self):
		threshold = self.get_string_value('Upper threshold', 'Misc')

		value = 0
		try:
			value = int(threshold)
		except:
			winki.lib.debug('Config.get_upper_threshold()', 'not a number %s' % threshold, winki.lib.error)
			value = 2500

		if value < 0:
			winki.lib.debug('Config.get_upper_threshold()', 'invalid value %s' % threshold, winki.lib.error)
			value = 2500

		if value > 16000:
			value = 16000

		return value

	def get_str_mplayer_name(self):
		return self.get_string_value('mplayer', 'Program Locations')

	def get_str_mencoder_name(self):
		return self.get_string_value('mencoder', 'Program Locations')

	def get_str_normalize_name(self):
		return self.get_string_value('normalize', 'Program Locations')

	def get_str_lsdvd_name(self):
		return self.get_string_value('lsdvd', 'Program Locations')

	def get_str_oggenc_name(self):
		return self.get_string_value('oggenc', 'Program Locations')

	def get_str_lame_name(self):
		return self.get_string_value('lame', 'Program Locations')

	def get_str_ogmmerge_name(self):
		return self.get_string_value('ogmmerge', 'Program Locations')

	def get_str_mkvmerge_name(self):
		return self.get_string_value('mkvmerge', 'Program Locations')

	def get_str_vcdimager_name(self):
		return self.get_string_value('vcdimager', 'Program Locations')

	def get_str_mplexer_name(self):
		mplexer = self.get_string_value('Multiplexer', 'Multiplex')
		if not mplexer or not len(mplexer):
			return ''
		return self.get_string_value(mplexer, 'Program Locations')

	def get_str_audio_compressor(self):
		compressor = self.get_string_value('Audio compressor', 'Audio')
		if not compressor or not len(compressor):
			return ''
		return self.get_string_value(compressor, 'Program Locations')

	def get_str_auido_compressor_options(self):
		compressor_program = self.get_str_audio_compressor()
		compressor = os.path.basename(compressor_program)

		options = ''
		if compressor == 'oggenc':
			options = self.get_string_value('Oggenc options', 'Audio')
		elif compressor == 'lame':
			options = self.get_string_value('Lame options', 'Audio')
		else:
			winki.lib.debug('Config.get_str_audio_compressor_options()', 'unknown compressor %s' % compressor, winki.lib.error)

		return options


	def get_str_audio_compressor_ext(self):
		compressor_program = self.get_str_audio_compressor()
		compressor = os.path.basename(compressor_program)

		ext = ''
		if compressor == 'oggenc':
			ext = 'ogg'
		elif compressor == 'lame':
			ext = 'mp3'
		else:
			winki.lib.debug('Config.get_str_audio_compressor_ext()', 'unknown compressor %s' % compressor, winki.lib.error)
		
		return ext

	def get_str_stream_merger(self):
		merger = self.get_string_value('Stream merger', 'Stream merge')
		if not merger or not len(merger):
			return ''
		return self.get_string_value(merger, 'Program Locations')

	def get_use_warning(self, warning):
		if not warning or not len(warning):
			return 1

		warnings = self.get_string_value('Use warnings', 'Misc')
		if not warnings or not len(warnings):
			return 0

		items = string.split(warnings)
		if not len(items):
			return 0

		for item in items:
			if item == warning:
				return 1

		return 0


	def get_audio_bytes_per_second(self):
		compressor_program = self.get_str_audio_compressor()
		compressor = os.path.basename(compressor_program)

		if compressor == 'oggenc':
			return self.get_ogg_bytes_per_second()
		elif compressor == 'lame':
			return self.get_mp3_bytes_per_second()
		else:
			return self.get_mp2_bytes_per_second()

	def get_ogg_bytes_per_second(self):
		bps = self.get_string_value('Bytes per ogg second', 'Estimation')
		value = 0
		try:
			value = int(bps)
		except:
			winki.lib.debug('Config.get_ogg_bytes_per_second()', 'not a number %s' % bps, winki.lib.error)
			value = 14000

		return value

	def get_mp3_bytes_per_second(self):
		bps = self.get_string_value('Bytes per mp3 second', 'Estimation')
		value = 0
		try:
			value = int(bps)
		except:
			winki.lib.debug('Config.get_mp3_bytes_per_second()', 'not a number %s' % bps, winki.lib.error)
			value = 19500

		return value

	def get_mp2_bytes_per_second(self):
		bps = self.get_string_value('Bytes per mp2 second', 'Estimation')
		value = 0
		try:
			value = int(bps)
		except:
			winki.lib.debug('Config.get_mp2_bytes_per_second()', 'not a number %s' % bps, winki.lib.error)
			value = 14000

		return value

	def get_sub_bytes_per_second(self):
		bps = self.get_string_value('Bytes per subtitle second', 'Estimation')
		value = 0
		try:
			value = int(bps)
		except:
			winki.lib.debug('Config.get_sub_bytes_per_second()', 'not a number %s' % bps, winki.lib.error)
			value = 3500

		return value

	def get_preferred_bitrate(self):
		bps = self.get_string_value('Preferred bitrate', 'Estimation')
		value = 0
		try:
			value = int(bps)
		except:
			winki.lib.debug('Config.get_preferred_bitrate()', 'not a number %s' % bps, winki.lib.error)
			value = 850

		return value

	def get_minimum_bitrate(self):
		bps = self.get_string_value('Minimum bitrate', 'Estimation')
		value = 0
		try:
			value = int(bps)
		except:
			winki.lib.debug('Config.get_preferred_bitrate()', 'not a number %s' % bps, winki.lib.error)
			value = 800

		return value

	def get_container_penalty(self):
		merger_program = self.get_str_stream_merger()
		if not merger_program or not len(merger_program):
			winki.lib.debug('Config.get_container_penalty()', 'default merger not set', winki.lib.error)
			return ''
		merger = os.path.basename(merger_program)

		penalty = ''
		if merger == 'ogmmerge':
			penalty = self.get_string_value('OGM container penalty', 'Estimation')
		elif merger == 'mkvmerge':
			penalty = self.get_string_value('Matroska container penalty', 'Estimation')
		else:
			winki.lib.debug('Config.get_container_penalty()', 'unknown merger %s' % merger, winki.lib.error)
			assert(0)

		value = winki.lib.to_float(penalty)
		if value < 0.0:
			winki.lib.debug('Config.get_container_penalty()', 'invalid value %s' % penalty, winki.lib.error)
			value = 0.015

		return value

	def get_svcd_container_penalty(self):
		penalty = self.get_string_value('SVCD container penalty', 'Estimation')

		value = 0
		try:
			value = float(penalty)
		except:
			winki.lib.debug('Config.get_svcd_container_penalty()', 'not a number %s' % penalty, winki.lib.error)
			value = 0.14

		if value < 0.0:
			winki.lib.debug('Config.get_svcd_container_penalty()', 'invalid value %f' % value, winki.lib.error)
			value = 0.14

		return value

	def get_vcd_bytes_per_second(self):
		coef = self.get_string_value('Bytes per VCD second', 'Estimation')

		value = 0
		try:
			value = float(coef)
		except:
			winki.lib.debug('Config.get_vcd_bytes_per_second()', 'not a number %s' % coef, winki.lib.error)
			value = 144500

		if value < 0:
			winki.lib.debug('Config.get_vcd_bytes_per_second()', 'invalid value %f' % value, winki.lib.error)
			value = 144500

		return value

