# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module gtk.glade...'
	import gtk.glade

	print 'Importing module gnome.ui...'
	import gnome.ui

	print 'Importing module winki.lib...'
	import winki.lib

	print 'Importing module winki.page_source...'
	import winki.page_source

	print 'Importing module winki.page_encoding...'
	import winki.page_encoding

	print 'Importing module winki.page_info...'
	import winki.page_info

	print 'Importing module winki.page_size...'
	import winki.page_size

	print 'Importing module winki.page_title...'
	import winki.page_title

	print 'Importing module winki.page_audio...'
	import winki.page_audio

	print 'Importing module winki.page_subtitles...'
	import winki.page_subtitles

	print 'Importing module winki.page_rip...'
	import winki.page_rip

	print 'Importing module winki.page_crop...'
	import winki.page_crop

	print 'Importing module winki.page_test...'
	import winki.page_test

	print 'Importing module winki.page_edit...'
	import winki.page_edit

	print 'Importing module gettext...'
	import gettext

	print 'Importing module gnome...'
	import gnome

except:
	error_quit(0, 1)

class Druid:
	def __init__(self, app):
		self.app = app

		self.xml = None
		self.load_xml()

		self.connect_signals()

		self.pages = {}
		self.create_pages()

		self.check_widgets()

		self.load_icon()

		return

	def load_xml(self):
		appname = 'winki'
		i18ndir = winki.lib.build_path([winki.lib.winki_path, 'i18n'])
		gettext.bindtextdomain(appname, i18ndir)
		gettext.textdomain(appname)
		gettext.install(appname, i18ndir)

		try:
			gtk.glade.bindtextdomain(appname, i18ndir)
			gtk.glade.textdomain(appname)
			xml_path = winki.lib.build_path([winki.lib.winki_path, 'glade', 'winki_rip_gtk2.glade'])
			self.xml = gtk.glade.XML(xml_path, 'collect_window', gettext.textdomain())
		except:
			print 'Druid.load_xml(): Cannot load glade UI file'

		if not self.xml:
			raise SystemExit
		return

	def connect_signals(self):
		signals = { 'on_druid_cancel' : self.on_cancel,
			'on_druidpage_source_prepare' : self.on_prepare,
			'on_druidpage_encoding_prepare' : self.on_prepare,
			'on_druidpage_info_prepare' : self.on_prepare,
			'on_druidpage_title_prepare' : self.on_prepare,
			'on_druidpage_size_prepare' : self.on_prepare,
			'on_druidpage_audio_prepare' : self.on_prepare,
			'on_druidpage_subtitles_prepare' : self.on_prepare,
			'on_druidpage_crop_prepare' : self.on_prepare,
			'on_druidpage_test_prepare' : self.on_prepare,
			'on_druidpage_edit_prepare' : self.on_prepare,
			'on_druidpage_rip_prepare' : self.on_prepare,
			'on_druidpage_source_back' : self.on_prev, 
			'on_druidpage_encoding_back' : self.on_prev, 
			'on_druidpage_info_back' : self.on_prev, 
			'on_druidpage_size_back' : self.on_prev, 
			'on_druidpage_title_back' : self.on_prev, 
			'on_druidpage_audio_back' : self.on_prev, 
			'on_druidpage_subtitles_back' : self.on_prev, 
			'on_druidpage_crop_back' : self.on_prev, 
			'on_druidpage_test_back' : self.on_prev, 
			'on_druidpage_edit_back' : self.on_prev, 
			'on_druidpage_rip_back' : self.on_prev, 
			'on_druidpagestart_next' : self.on_next, 
			'on_druidpage_source_next' : self.on_next,
			'on_druidpage_encoding_next' : self.on_next, 
			'on_druidpage_info_next' : self.on_next, 
			'on_druidpage_size_next' : self.on_next, 
			'on_druidpage_title_next' : self.on_next, 
			'on_druidpage_audio_next' : self.on_next, 
			'on_druidpage_subtitles_next' : self.on_next, 
			'on_druidpage_crop_next' : self.on_next, 
			'on_druidpage_test_next' : self.on_next, 
			'on_druidpage_edit_next' : self.on_next, 
			'on_druidpage_rip_next' : self.on_next, 
			'on_druidpagefinish_finish' : self.on_finish,
			'on_radio_dvd_toggled' : self.on_source_toggled,
			'on_radio_file_toggled' : self.on_source_toggled,
			'on_radio_vcd_toggled' : self.on_source_toggled,
			'on_button_input_file_clicked' : self.on_select_source,
			'on_radiobutton_multiple_pass_toggled' : self.on_encoding_toggled,
			'on_radiobutton_copy_stream_toggled' : self.on_encoding_toggled,
			'on_radiobutton_vcd_toggled' : self.on_encoding_toggled,
			'on_radiobutton_svcd_toggled' : self.on_encoding_toggled,
			'on_checkbutton_stream_mix_toggled' : self.on_stream_mix_toggled,
			'on_entry_name_changed' : self.on_change_name,
			'on_entry_output_file_changed' : self.on_change_outputfile,
			'on_button_output_file_clicked' : self.on_select_dest,
			'on_button_cancel_scan_clicked' : self.on_cancel_scan,
			'on_combo-entry_dvd_title_changed' : self.on_dvd_title_changed,
			'on_entry2_changed' : self.on_audio_entry_changed,
			'on_entry_audio1_changed' : self.on_audio_entry_changed,
			'on_button_audio1_clicked' : self.on_audio_clicked,
			'on_button_audio2_clicked' : self.on_audio_clicked,
			'on_button_audio3_clicked' : self.on_audio_clicked,
			'on_checkbutton_mix_subitles_toggled' : self.on_mix_subtitles_toggled,
			'on_checkbutton_consider_external_toggled' : self.on_consider_external_toggled,
			'on_button_sub1_clicked' : self.on_sub_clicked,
			'on_button_sub2_clicked' : self.on_sub_clicked,
			'on_button_sub3_clicked' : self.on_sub_clicked,
			'on_combo_sub1_changed' : self.on_combo_sub1_changed,
			'on_entry_sub1_changed' : self.on_entry_sub1_changed,
			'on_entry_combo_chunk_size_changed' : self.on_chunk_size_changed,
			'on_entry_chunk_size_changed' : self.on_chunk_size_changed,
			'on_spinbutton_nr_chunks_changed' : self.on_nr_chunks_changed,
			'on_combo_entry_start1_changed' : self.on_chapter_changed,
			'on_combo_entry_start2_changed' : self.on_chapter_changed,
			'on_combo_entry_start3_changed' : self.on_chapter_changed,
			'on_combo_entry_start4_changed' : self.on_chapter_changed,
			'on_combo_entry_start5_changed' : self.on_chapter_changed,
			'on_combo_entry_end1_changed' : self.on_chapter_changed,
			'on_combo_entry_end2_changed' : self.on_chapter_changed,
			'on_combo_entry_end3_changed' : self.on_chapter_changed,
			'on_combo_entry_end4_changed' : self.on_chapter_changed,
			'on_combo_entry_end5_changed' : self.on_chapter_changed,
			'on_button_crop_detect_clicked' : self.on_crop_detect,
			'on_button_1_forward_clicked' : self.on_forward_1,
			'on_button_10_forward_clicked' : self.on_forward_10,
			'on_button_30_forward_clicked' : self.on_forward_30,
			'on_button_1_backward_clicked' : self.on_backward_1,
			'on_button_10_backward_clicked' : self.on_backward_10,
			'on_button_30_backward_clicked' : self.on_backward_30,
			'on_button_1_forward_test_clicked' : self.on_forward_1,
			'on_button_10_forward_test_clicked' : self.on_forward_10,
			'on_button_30_forward_test_clicked' : self.on_forward_30,
			'on_button_1_backward_test_clicked' : self.on_backward_1,
			'on_button_10_backward_test_clicked' : self.on_backward_10,
			'on_button_30_backward_test_clicked' : self.on_backward_30,
			'on_button_rip_clicked' : self.on_toggle_rip,
			'on_button_test_settings_clicked' : self.on_test_settings,
			'on_druid_show_help' : self.on_show_help,
			'on_druidpage_source_show_help' : self.on_show_help,
			'on_druidpage_encoding_show_help' : self.on_show_help,
			'on_druidpage_info_show_help' : self.on_show_help,
			'on_druidpage_title_show_help' : self.on_show_help,
			'on_druidpage_size_show_help' : self.on_show_help,
			'on_druidpage_audio_show_help' : self.on_show_help,
			'on_druidpage_subtitles_show_help' : self.on_show_help,
			'on_druidpage_crop_show_help' : self.on_show_help,
			'on_druidpage_test_show_help' : self.on_show_help,
			'on_druidpage_edit_show_help' : self.on_show_help,
			'on_druidpage_rip_show_help' : self.on_show_help
			}

		self.xml.signal_autoconnect(signals)

		winki.lib.check_widgets(self.xml, ['collect_window'])
		window = self.xml.get_widget('collect_window')
		window.connect('destroy', self.on_cancel)
		return
		
	def create_pages(self):
		obj = None
		prev = None
		next = 'druidpage_source'
		self.pages['druidpagestart'] = [obj, prev, next]

		obj = winki.page_source.PageSource(self.xml, self.app)
		prev = 'druidpagestart'
		next = 'druidpage_encoding'
		self.pages['druidpage_source'] = [obj, prev, next]

		obj = winki.page_encoding.PageEncoding(self.xml, self.app)
		prev = 'druidpage_source'
		next = 'druidpage_info'
		self.pages['druidpage_encoding'] = [obj, prev, next]

		obj = winki.page_info.PageInfo(self.xml, self.app)
		prev = 'druidpage_encoding'
		next = 'druidpage_title'
		self.pages['druidpage_info'] = [obj, prev, next]

		obj = winki.page_title.PageTitle(self.xml, self.app)
		prev = 'druidpage_info'
		next = 'druidpage_audio'
		self.pages['druidpage_title'] = [obj, prev, next]

		obj = winki.page_audio.PageAudio(self.xml, self.app)
		prev = 'druidpage_title'
		next = 'druidpage_subtitles'
		self.pages['druidpage_audio'] = [obj, prev, next]

		obj = winki.page_subtitles.PageSubtitles(self.xml, self.app)
		prev = 'druidpage_audio'
		next = 'druidpage_size'
		self.pages['druidpage_subtitles'] = [obj, prev, next]

		obj = winki.page_size.PageSize(self.xml, self.app)
		prev = 'druidpage_subtitles'
		next = 'druidpage_crop'
		self.pages['druidpage_size'] = [obj, prev, next]

		obj = winki.page_crop.PageCrop(self.xml, self.app)
		prev = 'druidpage_size'
		next = 'druidpage_test'
		self.pages['druidpage_crop'] = [obj, prev, next]

		obj = winki.page_test.PageTest(self.xml, self.app)
		prev = 'druidpage_crop'
		next = 'druidpage_edit'
		self.pages['druidpage_test'] = [obj, prev, next]

		obj = winki.page_edit.PageEdit(self.xml, self.app)
		prev = 'druidpage_test'
		next = 'druidpage_rip'
		self.pages['druidpage_edit'] = [obj, prev, next]

		obj = winki.page_rip.PageRip(self.xml, self.app)
		prev = 'druidpage_edit'
		next = 'druidpagefinish'
		self.pages['druidpage_rip'] = [obj, prev, next]
	
		obj = None
		prev = 'druidpage_rip'
		next = None
		self.pages['druidpagefinish'] = [obj, next, prev]

		self.current_page = 'druidpagestart'
		return

	def check_widgets(self):
		widgets = []
		for page_name in self.pages:
			widgets.append(page_name)

		winki.lib.check_widgets(self.xml, widgets)

		for page_name in self.pages:
			obj, prev, next = self.pages[page_name]
			if not obj:
				continue
			obj.check_widgets()

		return

	def load_icon(self):
		window = self.xml.get_widget('collect_window')
		try:
			icon_path = winki.lib.build_path([winki.lib.winki_path, 'glade', 'winki.png'])
			pixbuf = gtk.gdk.pixbuf_new_from_file(icon_path)
			window.set_icon(pixbuf)
		except:
			print 'failed to load winki\'s face. huh...'
		return

	def get_window(self):
		""" used by the application object """

		return self.xml.get_widget('collect_window')

	def on_cancel(self, widget, event = None):
		can_exit = True

		obj, prev, next = self.pages[self.current_page]
		if obj:
			can_exit = obj.can_exit()

		if not can_exit:
			return False

		self.app.cleanup()
		return True

	def on_next(self, widget, event = None):
		cur_name = widget.get_name()
		obj, prev, next = self.pages[cur_name]
		if obj and not obj.on_next():
			return True

		name = self.get_next_page(cur_name)
		if not name:
			return True

		page = self.xml.get_widget(name)

		druid = self.xml.get_widget('druid')
		druid.set_page(page)

		self.current_page = name
		return True

	def on_prev(self, widget, event = None):
		cur_name = widget.get_name()
		obj, prev, next = self.pages[cur_name]
		if obj and not obj.on_prev():
			return True

		name = self.get_prev_page(cur_name)
		if not name:
			return True

		page = self.xml.get_widget(name)

		druid = self.xml.get_widget('druid')
		druid.set_page(page)

		self.current_page = name
		return True

	def get_next_page(self, name):
		source = self.app.project.get_source()
		encoding = self.app.project.get_encoding()
		mode = self.app.project.get_user_mode()

		if source == 'file':
			if name == 'druidpage_info':
				self.app.project.set_number_chunks(1)
				return 'druidpage_audio'

			if name == 'druidpage_subtitles' and encoding == 'copy':
				self.app.project.set_number_chunks(1)
				return 'druidpage_crop'

		if source == 'vcd':
			if name == 'druidpage_info':
				self.app.project.set_number_chunks(1)
				return 'druidpage_audio'

			if name == 'druidpage_audio' and encoding == 'copy':
				self.app.project.set_number_chunks(1)
				return 'druidpage_crop'

			if name == 'druidpage_audio':
				self.app.project.set_number_chunks(1)
				return 'druidpage_size'

		if encoding == 'vcd' or encoding == 'svcd':
			if name == 'druidpage_audio':
				return 'druidpage_size'

		if mode != 'Advanced':
			if name == 'druidpage_test':
				return 'druidpage_rip'

		obj, prev, next = self.pages[name]
		return next

	def get_prev_page(self, name):
		source = self.app.project.get_source()
		encoding = self.app.project.get_encoding()
		mode = self.app.project.get_user_mode()

		if source == 'file':
			if encoding == 'copy' and name == 'druidpage_crop':
				return 'druidpage_subtitles'

			if name == 'druidpage_audio':
				return 'druidpage_info'

		if source == 'vcd':
			if encoding == 'copy' and name == 'druidpage_crop':
				return 'druidpage_audio'

			if name == 'druidpage_audio':
				return 'druidpage_info'

			if name == 'druidpage_size':
				return 'druidpage_audio'

		if mode != 'Advanced':
			if name == 'druidpage_rip':
				return 'druidpage_test'

		if encoding == 'vcd' or encoding == 'svcd':
			if name == 'druidpage_size':
				return 'druidpage_audio'

		obj, prev, next = self.pages[name]
		return prev

	def on_prepare(self, widget, event = None):
		obj, prev, next = self.pages[widget.get_name()]
		if not obj:
			return True

		return obj.on_prepare()

	def on_select_source(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_source']
		fname = obj.get_file_name()
		self.app.select_source(fname)
		return True

	def source_selected(self, file_name):
		obj, prev, next = self.pages['druidpage_source']
		obj.set_file_name(file_name)
		return

	def on_encoding_toggled(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_encoding']
		obj.encoding_toggled()
		return True

	def on_stream_mix_toggled(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_encoding']
		obj.stream_mix_toggled()
		return True

	def on_change_name(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_info']
		obj.name_changed()
		return True

	def on_change_outputfile(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_info']
		obj.toggle_generated()
		return True

	def cancel_file_selection(self, action):
		if action == 'select dest':
			obj, prev, next = self.pages['druidpage_info']
			obj.is_generated = True
		return True

	def on_select_dest(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_info']
		fname = obj.get_file_name()
		self.app.select_dest(fname)
		return True

	def dest_selected(self, file_name):
		obj, prev, next = self.pages['druidpage_info']
		obj.set_file_name(file_name)
		return

	def on_cancel_scan(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_info']
		obj.on_cancel()
		return True

	def on_audio_entry_changed(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_audio']
		obj.on_toggle_ui()
		return True

	def on_audio_clicked(self, widget, event = None):
		if not widget:
			return False

		index = -1
		if widget.get_name() == 'button_audio1':
			index = 1
		elif widget.get_name() == 'button_audio2':
			index = 2
		elif widget.get_name() == 'button_audio3':
			index = 3
		else:
			return False

		obj, prev, next = self.pages['druidpage_audio']
		fname = obj.get_audio_filename(index)
		self.app.select_external_audio(fname, widget.get_name())
		return True

	def external_audio_selected(self, fname, number):
		obj, prev, next = self.pages['druidpage_audio']
		obj.set_audio_filename(fname, number)
		return

	def on_sub_clicked(self, widget, event = None):
		if not widget:
			return False

		index = -1
		if widget.get_name() == 'button_sub1':
			index = 1
		elif widget.get_name() == 'button_sub2':
			index = 2
		elif widget.get_name() == 'button_sub3':
			index = 3
		else:
			return False

		obj, prev, next = self.pages['druidpage_subtitles']
		path = obj.get_subtitles_filename(index)
		self.app.select_external_subtitles(path, widget.get_name())
		return True

	def external_subtitles_selected(self, fname, number):
		obj, prev, next = self.pages['druidpage_subtitles']
		obj.set_subtitles_filename(fname, number)
		return

	def on_combo_sub1_changed(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_subtitles']
		obj.on_enable_mix_ui()
		return True

	def on_entry_sub1_changed(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_subtitles']
		obj.on_enable_mix_ui()
		return True

	def on_mix_subtitles_toggled(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_subtitles']
		obj.on_mix_subtitles_toggled()
		return True

	def on_consider_external_toggled(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_subtitles']
		obj.on_enable_mix_ui()
		return True

	def on_source_toggled(self, widget, event = None):
		src = ''
		wname = widget.get_name()
		if wname == 'radio_dvd':
			src = 'dvd'
		elif wname == 'radio_file':
			src = 'file'
		elif wname == 'radio_vcd':
			src = 'vcd'
		else:
			print wname
			assert(0)

		self.app.project.set_source(src)
		obj, prev, next = self.pages['druidpage_source']
		obj.enable_ui()
		return True

	def on_dvd_title_changed(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_size']
		obj.initialized = False

		obj, prev, next = self.pages['druidpage_title']
		return obj.title_changed()

	def on_chunk_size_changed(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_size']
		obj.on_chunk_size_changed()
		return True

	def on_nr_chunks_changed(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_size']
		obj.nr_chunks_changed()
		return True

	def on_chapter_changed(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_size']
		obj.on_chapter_changed()
		return True

	def on_crop_detect(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_crop']
		obj.on_crop()
		return True

	def on_forward_1(self, widget, event = None):
		obj, prev, next = self.pages[self.current_page]
		obj.on_forward_1()
		return True

	def on_forward_10(self, widget, event = None):
		obj, prev, next = self.pages[self.current_page]
		obj.on_forward_10()
		return True

	def on_forward_30(self, widget, event = None):
		obj, prev, next = self.pages[self.current_page]
		obj.on_forward_30()
		return True

	def on_backward_1(self, widget, event = None):
		obj, prev, next = self.pages[self.current_page]
		obj.on_backward_1()
		return True

	def on_backward_10(self, widget, event = None):
		obj, prev, next = self.pages[self.current_page]
		obj.on_backward_10()
		return True

	def on_backward_30(self, widget, event = None):
		obj, prev, next = self.pages[self.current_page]
		obj.on_backward_30()
		return True

	def on_toggle_rip(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_rip']
		obj.on_toggle_rip()
		return True

	def on_test_settings(self, widget, event = None):
		obj, prev, next = self.pages['druidpage_test']
		obj.on_test_settings()
		return True

	def show_help(self):
		obj, prev, next = self.pages[self.current_page]
		if not obj:
			obj, prev, next = self.pages['druidpage_source']

		return obj.show_help()

	def on_show_help(self, widget, event = None):
		return self.show_help()

	def on_finish(self, widget, event = None):
		self.app.cleanup()
		return True

