# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module gtk.glade...'
	import gtk.glade

	print 'Importing module winki.lib...'
	import winki.lib

except:
	error_quit(0, 1)

class FileSelection:
	def __init__(self, app, parent_window):
		self.app = app
		self.parent = parent_window
		self.xml = None

		try:
			glade_path = winki.lib.build_path([winki.lib.winki_path, 'glade', 'winki_rip_gtk2.glade'])
			self.xml = gtk.glade.XML(glade_path, 'fileselection')
		except:
			print 'FileSelection: Cannot load glade UI file'
			raise SystemExit

		dic = { 'on_ok_button_fileselection_clicked' : self.on_ok,
			'on_cancel_button_fileselection_clicked' : self.on_cancel}
		self.xml.signal_autoconnect(dic)

		winki.lib.check_widgets(self.xml, ['fileselection'])
		self.dialog = self.xml.get_widget('fileselection')
		self.dialog.connect('delete_event', self.no_quit)
		
		self.path_should_exist = False
		self.file_name = None
		self.caption = None
		return

	def check_widgets(self):
		widgets = ['ok_button_fileselection',
			   'cancel_button_fileselection']

		if not winki.lib.check_widgets(self.xml, widgets):
			return False

		return True

	def show(self):
		if self.caption:
			self.dialog.set_title(self.caption)

		if self.file_name:
			self.dialog.set_filename(self.file_name)

		self.dialog.show()
		self.dialog.set_modal(True)

		self.parent.set_sensitive(False)
		return True

	def hide(self):
		self.dialog.hide()
		self.parent.set_sensitive(True)
		return True

	def no_quit(self, button, event = None):
		"disallow close"
		return True

	def reset(self):
		self.path_should_exist = False
		self.file_name = None
		self.caption = None
		return

	def set_filename(self, fname):
		self.dialog.set_filename(fname)
		return

	def on_ok(self, widget, event = None):
		path = self.dialog.get_filename()
		if self.path_should_exist:
			if not winki.lib.is_file(path) or not winki.lib.is_readable(path):
				return True
		self.file_name = path
		self.hide()
		self.app.file_selected(path)
		return True

	def on_cancel(self, widget, event = None):
		self.hide()
		self.app.cancel_file_selection()
		return True

