# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module string...'
	import string

	print 'Importing module os...'
	import os

	print 'Importing module winki.alert...'
	import winki.alert
except:
	error_quit(0, 1)

# current program version
winki_version = '0.3.8'

# program installation path (winki_path is set here _and_ in ../setup.py; i18n stuff in install)
winki_path = os.path.join('/', 'usr', 'share', 'winki')

# enumerations for the priority of the debug messages
info = 1
error = 2
critical = 4

# variable which controls the debug output
#show = info | error | critical
show = error

def debug(module, message, priority = info):
	if show & info and priority & info:
		print 'info: %s - %s' % (module, message)

	if show & error and priority & error:
		print 'error: %s - %s' % (module, message)

	if show & critical and priority & critical:
		print 'critical: %s - %s' % (module, message)
	return

def to_int(str):
	num = -1

	try:
		num = int(str)
	except:
		pass

	return num

def to_float(str):
	num = -1.0

	try:
		num = float(str)
	except:
		pass

	return num

def is_allowed_in_filename(ch):
	# i don't know if this is worth since string.letters etc. are modifiyable...
	if string.find(string.letters, ch) > -1:
		return True
	elif string.find(string.digits, ch) > -1:
		return True
	elif string.find(string.whitespace, ch) > -1:
		return True
	elif string.find('/', ch) > -1:
		return True

	return False

def fix_name(name):
	# see ord(): file:///usr/share/doc/python2.1/html/lib/built-in-funcs.html
	if not name or not len(name):
		return ''

	name = filter(is_allowed_in_filename, name)
	name = string.replace(name, ' ', '_')
	name = string.replace(name, '\t', '_')

	return string.lower(name)

def escape_name(name):
	sym = ['"', '|', '$']

	if not name:
		return name

	table = string.maketrans('!:', '  ')
	fname = string.translate(name, table)

	result = [] 
	for i in range(len(fname)):
		for j in range(len(sym)):
			if sym[j] == name[i]:
				result.append('\\')
				break

		result.append(name[i])

	return string.join(result, '')

def fill_combo(combo, strings, default = 0, empty = None):
	if combo == None:
		return

	default_empty = [_('none')]
	if not strings and (not empty or len(empty) == 0):
		combo.set_popdown_strings(default_empty)
		combo.entry.set_text(default_empty[0])
		combo.set_sensitive(False)
		return

	if not strings or len(strings) == 0:
		combo.set_popdown_strings(empty)
		combo.entry.set_text(empty[0])
		combo.set_sensitive(False)
		return

	combo.set_sensitive(True)
	combo.set_popdown_strings(strings)

	if default < 0 or default >= len(strings):
		combo.entry.set_text(strings[0])		
	else:
		combo.entry.set_text(strings[default])
	return

def list_from_string(str):
	lst = []

	pos1 = string.find(str, '\'')
	while pos1 > -1:
		pos2 = string.find(str, '\'', pos1 + 1)
		if pos2 < 0:
			debug('list_from_string()', 'invalid string passed', error)
			continue

		lst.append(str[pos1 + 1 : pos2])
		pos1 = string.find(str, '\'', pos2 + 1)
	return lst

def check_widgets(xml, widget_names):
	for widget_name in widget_names:
		widget = xml.get_widget(widget_name)
		if not widget:
			msg = '"%s" cannot be loaded' % widget_name
			print 'lib.check_widgets(): %s' % msg
			print 'Please check your glade file'
			raise SystemExit

	return

def is_quoted(name):
	if not name or not len(name):
		return False

	return (string.find(name, '"') > -1)

def remove_quotes(name):
	if not name or not len(name):
		return name

	return string.replace(name, '"', '')

def set_quotes(name, use_flag = True):
	if not name or not len(name) or not use_flag:
		return name

	if is_quoted(name):
		name = remove_quotes(name)

	return '"%s"' % name

def split_path(full_name):
	path = None
	fname = None
	ext = None
	if not full_name or not len(full_name):
		return [path, fname, ext]

	quoted = is_quoted(full_name)
	if quoted:
		full_name = remove_quotes(full_name)

	if full_name == '.' or full_name == '..' or full_name == '~':
		path = set_quotes(full_name, quoted)
		return [path, fname, ext]

	path, name = os.path.split(full_name)
	if not name or not len(name):
		return [set_quotes(path, quoted), fname, ext]

	fname, ext = os.path.splitext(name)
	return [set_quotes(path, quoted), set_quotes(fname, quoted), set_quotes(ext, quoted)]

def normalize_path(path):
	if not path or not len(path):
		return path

	quoted = is_quoted(path)
	if quoted:
		path = remove_quotes(path)

	if string.find(path, '~') > -1:
		path = os.path.expanduser(path)
	
	for ch in path:
		if is_allowed_in_filename(ch):
			break
		else:
			debug('lib.normalize_path()', 'please don\'t use %c in file names' % ch, error)
			return None

	if string.find(path, '~') > -1:
		msg_info = 'mailformed path %s' % path
		debug('lib.normalize_path()', msg_info, error)
		return None

	path =  os.path.normpath(path)
	return set_quotes(path, quoted)
		
def build_path(items):
	path = ''
	if not items or not len(items):
		return None

	quoted = False
	for item in items:
		quoted |= is_quoted(item)

	for item in items:
		if not item or not len(item):
			continue

		if quoted:
			item = remove_quotes(item)

		path = os.path.join(path, item)

	return normalize_path(set_quotes(path, quoted))

def set_extension(fullpath, extension):
	if not extension or not len(extension):
		return fullpath

	quoted = is_quoted(fullpath)
	quoted |= is_quoted(extension)

	if quoted:
		fullpath = remove_quotes(fullpath)
		extension = remove_quotes(extension)
		
	if extension[0] == '.':
		extension = extension[1: ]

	fname, ext = os.path.splitext(fullpath)
	result = '%s.%s' % (fname, extension)

	return set_quotes(result, quoted)

def append_to_filename(fullpath, appendix):
	if not appendix or not len(appendix):
		return fullpath

	quoted = is_quoted(fullpath)
	quoted |= is_quoted(appendix)

	if quoted:
		fullpath = remove_quotes(fullpath)
		appendix = remove_quotes(appendix)
		
	path, name, ext = split_path(fullpath)
	if name:
		name = '%s%s' % (name, appendix)

	fname = set_extension(name, ext)
	return build_path([set_quotes(path, quoted), set_quotes(fname, quoted)])

def does_path_exist(path):
	if not path or not len(path):
		return False

	if is_quoted(path):
		path = remove_quotes(path)

	return os.path.exists(path)

def is_file(path):
	if not does_path_exist(path):
		return False

	if is_quoted(path):
		path = remove_quotes(path)

	if os.path.isfile(path) == 0:
		return False;
		
	return True

def is_dir(path):
	if not does_path_exist(path):
		return False

	if is_quoted(path):
		path = remove_quotes(path)

	if os.path.isdir(path) == 0:
		return False;
		
	return True

def is_readable(path):
	if not does_path_exist(path):
		return False

	if is_quoted(path):
		path = remove_quotes(path)

	if os.access(path, os.R_OK)  == 0:
		return False

	return True

def is_writable(path):
	if not does_path_exist(path):
		return False

	if is_quoted(path):
		path = remove_quotes(path)

	if os.access(path, os.W_OK)  == 0:
		return False

	return True

def is_executable(path):
	if not does_path_exist(path):
		return False

	if is_quoted(path):
		path = remove_quotes(path)

	if os.access(path, os.X_OK)  == 0:
		return False

	return True

def is_dir_empty(path):
	if not does_path_exist(path):
		return False

	if is_quoted(path):
		path = remove_quotes(path)

	if not is_dir(path):
		return False

	if not is_readable(path):
		return False

	file_list = os.listdir(path)
	return (len(file_list) == 0)


def file_size(fname):
	if not fname or not len(fname):
		return -1

	if is_quoted(fname):
		fname = remove_quotes(fname)

	size = -1
	try:
		f = os.open(fname, os.O_RDONLY)
		size = os.lseek(f, 0, 2)
		os.close(f)
	except:
		pass

	return size

def mkdir(path):
	if is_quoted(path):
		path = remove_quotes(path)
	
	path = normalize_path(path)
	if does_path_exist(path):
		if is_dir(path):
			return True
		else:
			return False

	try:
		os.makedirs(path)
	except:
		debug('lib.mkdir', 'cannot create %s' % path, error)
		return False

	return True

def rename(src, dest):
	if is_quoted(src):
		src = remove_quotes(src)
	if is_quoted(dest):
		dest = remove_quotes(dest)
	
	try:
		os.rename(src, dest)
	except:
		debug('lib.rename', 'cannot rename %s into %s' % (src, dest), error)
		return False

	return True

def read_lines(fname):
	if is_quoted(fname):
		fname = remove_quotes(fname)
	
	lines = None
	try:
		f = open(fname, 'r')
		raw = f.read()
		f.close()

		lines = string.split(raw, '\n')
	except:
		debug('read_lines()', 'failed to read in "%s"' % fname, error)

	return lines

def write_lines(fname, lines):
	if not lines:
		debug('write_lines()', 'no lines passed', error)
		return False

	if is_quoted(fname):
		fname = remove_quotes(fname)
	
	try:
		f = open(fname, 'w')
		for line in lines:
			f.write('%s\n' % line)
		f.close()
	except:
		debug('write_lines()', 'failed to write out "%s"' % fname, error)
		return False

	return True

def get_index(list, entry):
	if not list:
		return -1

	for item in range(len(list)):
		if list[item] == entry:
			return item
	return -1

def format_time(times, start, end):
	if start < 0 or start > len(times):
		return ''

	if end < 0 or end > len(times):
		return ''

	seconds = 0
	for i in range(len(times)):
		if i < start:
			continue
		if i > end:
			break
		seconds += times[i]

	h = seconds / 3600
	seconds -= (h * 3600)
	m = seconds / 60
	seconds -= (m * 60)

	hours = ''
	minutes = ''
	sec = ''

	if h < 10: hours = '0%d' % h
	else: hours = '%d' % h
	
	if m < 10: minutes = '0%d' % m
	else: minutes = '%d' % m
	
	if seconds < 10: sec = '0%d' % seconds
	else: sec = '%d' % seconds
	
	time = '%s:%s:%s' % (hours, minutes, sec)
	return time

def format_size(times, start, end, bytes_per_second):
	if start < 0 or start > len(times):
		return ''

	if end < 0 or end > len(times):
		return ''

	if bytes_per_second < 0:
		return ''

	seconds = 0
	for i in range(len(times)):
		if i < start:
			continue
		if i > end:
			break
		seconds += times[i]

	return _('~%dMB') % ((seconds * bytes_per_second) / 1024 / 1024)

def is_radio_active(radio):
	if not radio:
		return False

	return radio.get_active() != 0


def entry_has_focus(entry):
	if not entry:
		return False

	return entry.flags() & gtk.HAS_FOCUS

def is_sensitive(widget):
	if not widget:
		return False

	return (widget.get_property('sensitive') != 0)

def update_progress(bar, text, number):
	if not bar:
		return False

	bar.set_text(text)
	bar.set_fraction(float(number))
	return True

def set_button_label(button, text):
	if not button:
		return False

	button.set_label(text)
	return True

def alert(primary, secondary, kind, affirmative = None):
	state = 'ok'

	if kind == 'error':
		err = winki.alert.AlertError()
		err.show(primary, secondary)

	elif kind == 'warning':
		if not affirmative:
			assert(0)

		warning = winki.alert.AlertWarning()
		warning.show(primary, secondary, affirmative)
		state = warning.state

	else:
		assert(0)

	return state
