# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module winki.lib...'
	import winki.lib

	print 'Importing module winki.base_page...'
	import winki.base_page

except:
	error_quit(0, 1)

class PageAudio(winki.base_page.BasePage):
	def __init__(self, xml, app):
		winki.base_page.BasePage.__init__(self, xml, app)
		self.no_audio = [_('No audio in this title')]
		self.initialized = False
		self.error = None
		return
		
	def check_widgets(self):
		widgets = ['button_audio1', 'button_audio2',
		'button_audio3', 'combo_audio1', 'combo_audio2',
		'combo_audio3', 'entry_audio1', 'entry_audio2',
		'entry_audio3', 'label_audio1', 'label_audio2',
		'label_audio3', 'label_external_audio']

		winki.lib.check_widgets(self.xml, widgets)
		return

	def on_prepare(self):
		self.init_layout()
		self.on_toggle_ui()
		return True

	def show_help(self):
		return winki.base_page.BasePage.show_help(self, 'audio.html')

	def init_layout(self):
		combo_audio1 = self.xml.get_widget('combo_audio1')
		combo_audio2 = self.xml.get_widget('combo_audio2')
		combo_audio3 = self.xml.get_widget('combo_audio3')

		current = self.app.project.get_current_title()
		audio_langs = self.app.project.get_audio(current)

		default_audio = []
		if not self.initialized:
			default_audio = self.app.project.get_preferred_audio()
			self.initialized = True
		else:
			default_audio = self.app.project.get_current_audio()

		audio1 = 0
		if default_audio and len(default_audio) > 0:
			audio1 = default_audio[0]
		audio2 = 0
		if default_audio and len(default_audio) > 1:
			audio2 = default_audio[1]
		audio3 = 0
		if default_audio and len(default_audio) > 2:
			audio3 = default_audio[2]

		winki.lib.fill_combo(combo_audio1, audio_langs, audio1, self.no_audio)
		winki.lib.fill_combo(combo_audio2, audio_langs, audio2, self.no_audio)
		winki.lib.fill_combo(combo_audio3, audio_langs, audio3, self.no_audio)
		return

	def on_toggle_ui(self):
		v = (self.app.project.get_source() == 'dvd')
		c = (self.app.project.get_encoding() == 'copy')
		a = (self.app.project.get_user_mode() == 'Advanced')

		self.toggle_visibility()
		self.toggle_sensitivity()

		if a and c and v:
			self.toggle_concurrent()
		return

	def toggle_concurrent(self):
		cur_title = self.app.project.get_current_title()
		if cur_title < 0:
			return

		combo_audio1 = self.xml.get_widget('combo_audio1')
		label_audio1 = self.xml.get_widget('label_audio1')
		entry_audio1 = self.xml.get_widget('entry_audio1')
		button_audio1 = self.xml.get_widget('button_audio1')

		cur_audio = self.app.project.get_audio(cur_title)
		text = combo_audio1.entry.get_text()
		i = winki.lib.get_index(cur_audio, text)

		fname = entry_audio1.get_text()

		# i == 0 means no DVD audio
		if i > 0:
			combo_audio1.set_sensitive(True)
			label_audio1.set_sensitive(True)
			entry_audio1.set_sensitive(False)
			button_audio1.set_sensitive(False)

		# external audio file
		elif fname and len(fname):
			combo_audio1.set_sensitive(False)
			label_audio1.set_sensitive(False)
			entry_audio1.set_sensitive(True)
			button_audio1.set_sensitive(True)

		# nothing is set at all
		else:
			combo_audio1.set_sensitive(True)
			label_audio1.set_sensitive(True)
			entry_audio1.set_sensitive(True)
			button_audio1.set_sensitive(True)
		return

# visibility scheme:
#    	c1	c2	c3	e1	e2	e3
# dfm	on	on	on
# dfc	on	on	on
# dfV	on
# dvm	on	on	on	
# dvc	on	on	on	
# dvV	on
# afm	on  	on  	on  	on	on	on
# afc	on  	on  	on  	on	on	on
# afV	on  	on  	on  	on	on	on
# avm	on	on	on	on	on	on
# avc	on	on	on	on	on	on
# avV	on
#
# where c1 - c3 mean combo1 - combo3 and e1 - e3 mean entry1 - entry3
# f - source file
# v - source dvd
# m - encoding multi pass
# c - encoding copy
# V - encoding VCD/SVCD
# d - user mode default
# a - user mode advanced
	def toggle_visibility(self):
		a = (self.app.project.get_user_mode() == 'Advanced')

		combo_audio1 = self.xml.get_widget('combo_audio1')
		combo_audio2 = self.xml.get_widget('combo_audio2')
		combo_audio3 = self.xml.get_widget('combo_audio3')
		label_audio1 = self.xml.get_widget('label_audio1')
		label_audio2 = self.xml.get_widget('label_audio2')
		label_audio3 = self.xml.get_widget('label_audio3')
		button_audio1 = self.xml.get_widget('button_audio1')
		button_audio2 = self.xml.get_widget('button_audio2')
		button_audio3 = self.xml.get_widget('button_audio3')
		entry_audio1 = self.xml.get_widget('entry_audio1')
		entry_audio2 = self.xml.get_widget('entry_audio2')
		entry_audio3 = self.xml.get_widget('entry_audio3')
		label_external = self.xml.get_widget('label_external_audio')

		combo_audio1.show()
		combo_audio2.show()
		combo_audio3.show()
		label_audio1.show()
		label_audio2.show()
		label_audio3.show()

		if a:
			entry_audio1.show()
			entry_audio2.show()
			entry_audio3.show()
			button_audio1.show()
			button_audio2.show()
			button_audio3.show()
			label_external.show()
		else:
			entry_audio1.hide()
			entry_audio2.hide()
			entry_audio3.hide()
			button_audio1.hide()
			button_audio2.hide()
			button_audio3.hide()
			label_external.hide()
		return

	def toggle_sensitivity(self):
		enc = self.app.project.get_encoding()

		for i in range(3):
			combo_name = 'combo_audio%d' % (i + 1)
			label_name = 'label_audio%d' % (i + 1)
			entry_name = 'entry_audio%d' % (i + 1)
			button_name = 'button_audio%d' % (i + 1)

			combo = self.xml.get_widget(combo_name)
			label = self.xml.get_widget(label_name)
			entry = self.xml.get_widget(entry_name)
			button = self.xml.get_widget(button_name)

			if enc == 'vcd' or enc == 'svcd':
				if i != 0:
					combo.set_sensitive(False)
					label.set_sensitive(False)
				entry.set_sensitive(False)
				button.set_sensitive(False)
			else:
				combo.set_sensitive(True)
				label.set_sensitive(True)
				entry.set_sensitive(True)
				button.set_sensitive(True)
		return

	def on_next(self):
		return self.update_project()

	def on_prev(self):
		return self.update_project()

	def can_exit(self):
		return True

	def get_audio_filename(self, index):
		if index < 0 or index > 3:
			winki.lib.debug('PageAudio.get_audio_filename()', 'invalid index', winki.lib.error)
			return None

		entry_name = ('entry_audio%d' % index)
		entry = self.xml.get_widget(entry_name)
		return entry.get_text()

	def set_audio_filename(self, file_name, index):
		if index < 0 or index > 3:
			winki.lib.debug('PageAudio.set_audio_filename()', 'invalid index', winki.lib.error)
			return

		entry_name = ('entry_audio%d' % index)
		entry = self.xml.get_widget(entry_name)
		entry.set_text(file_name)
		return

	def update_project(self):
		result = True
		self.error = None

		result &= self.submit_stream_audio()
		result &= self.submit_audio_files()

		if not result and self.error:
			winki.lib.alert(_('Could not use file'), self.error, 'error')

		return result

	def submit_stream_audio(self):
		combo_audio1 = self.xml.get_widget('combo_audio1')
		combo_audio2 = self.xml.get_widget('combo_audio2')
		combo_audio3 = self.xml.get_widget('combo_audio3')

		result = [_('none'), _('none'), _('none')]
		if winki.lib.is_sensitive(combo_audio1):
			result[0] = combo_audio1.entry.get_text()

		if winki.lib.is_sensitive(combo_audio2):
			result[1] = combo_audio2.entry.get_text()

		if winki.lib.is_sensitive(combo_audio3):
			result[2] = combo_audio3.entry.get_text()

		return self.app.project.set_current_audio(result)

	def submit_audio_files(self):
		result = True

		for i in range(3):
			entry_name = ('entry_audio%d' % (i + 1))
			entry = self.xml.get_widget(entry_name)
			fname = entry.get_text()
			if fname and len(fname):
				if winki.lib.is_file(fname):
					if winki.lib.is_readable(fname):
						self.app.project.set_external_audio(i, fname)
					else:
						self.error = (_('%s should be readable by you') % fname)
						result = False
				else:
					self.error = (_('%s should be a regular file') % fname)
					result = False

		return result

