# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module string...'
	import string

	print 'Importing module winki.lib...'
	import winki.lib

	print 'Importing module winki.build...'
	import winki.build

	print 'Importing module winki.base_page...'
	import winki.base_page

	print 'Importing module winki.base_player_page...'
	import winki.base_player_page

except:
	error_quit(0, 1)

class PageCrop(winki.base_page.BasePage, winki.base_player_page.BasePlayerPage):
	def __init__(self, xml, app):
		winki.base_page.BasePage.__init__(self, xml, app)
		winki.base_player_page.BasePlayerPage.__init__(self)
		return
		
	def check_widgets(self):
		widgets = [ 'button_10_backward', 'button_10_forward',
		'button_1_backward', 'button_1_forward',
		'button_30_backward', 'button_30_forward',
		'button_crop_detect', 'drawingarea_crop', 'spinbutton_h',
		'spinbutton_w', 'spinbutton_x', 'spinbutton_y']

		winki.lib.check_widgets(self.xml, widgets)
		return


	def on_prepare(self):
		values = self.app.project.get_crop()
		w = self.xml.get_widget('spinbutton_w')
		h = self.xml.get_widget('spinbutton_h')
		x = self.xml.get_widget('spinbutton_x')
		y = self.xml.get_widget('spinbutton_y')

		w.set_text('%d' % (values[0]))
		h.set_text('%d' % (values[1]))
		x.set_text('%d' % (values[2]))
		y.set_text('%d' % (values[3]))

		self.enable_ui(True)
		return True

	def on_next(self):
		if not winki.base_player_page.BasePlayerPage.on_next(self):
			return False

		return self.update_project()

	def on_prev(self):
		if not winki.base_player_page.BasePlayerPage.on_prev(self):
			return False

		return self.update_project()

	def can_exit(self):
		return not winki.base_player_page.BasePlayerPage.has_player(self)

	def update_project(self):
		spin_w = self.xml.get_widget('spinbutton_w')
		spin_h = self.xml.get_widget('spinbutton_h')
		spin_x = self.xml.get_widget('spinbutton_x')
		spin_y = self.xml.get_widget('spinbutton_y')

		w = winki.lib.to_int(spin_w.get_text())
		h = winki.lib.to_int(spin_h.get_text())
		x = winki.lib.to_int(spin_x.get_text())
		y = winki.lib.to_int(spin_y.get_text())

		if w < 0 or h < 0 or x < 0 or y < 0:
			winki.lib.debug('PageCrop.update_project()', 'negative values for cropping specified - resetted')
			w = 0
			h = 0
			x = 0
			y = 0

		return self.app.project.set_crop([w, h, x, y])

	def show_help(self):
		return winki.base_page.BasePage.show_help(self, 'crop.html')

	def on_crop(self):
		if winki.base_player_page.BasePlayerPage.has_player(self):
			self.on_grab_now()
		else:
			self.start_crop_detect()
		return

	def on_grab_now(self):
		self.stop_crop_detect()
		self.update_project()
		return

	def start_crop_detect(self):
		if self.app.project.get_source() == 'dvd' and self.app.project.get_current_title() < 0:
			winki.lib.debug('PageCrop.start_crop_detect()', 'no current title set', winki.lib.error)
			return

		self.enable_ui(False)

		builder = winki.build.Builder(self.app.project)
		cmd = builder.build_crop_command()

		area = self.xml.get_widget('drawingarea_crop')

		winki.base_player_page.BasePlayerPage.start_player(self, area, cmd, self.parse_crop)
		return

	def stop_crop_detect(self):
		winki.base_player_page.BasePlayerPage.stop_player(self)

		self.enable_ui(True)
		return

	def enable_ui(self, enable):
		button_detect = self.xml.get_widget('button_crop_detect')
		area = self.xml.get_widget('drawingarea_crop')
		forward1 = self.xml.get_widget('button_1_forward')
		forward10 = self.xml.get_widget('button_10_forward')
		forward30 = self.xml.get_widget('button_30_forward')
		backward1 = self.xml.get_widget('button_1_backward')
		backward10 = self.xml.get_widget('button_10_backward')
		backward30 = self.xml.get_widget('button_30_backward')

		if not enable:
			winki.lib.set_button_label(button_detect, _('_Grab now!'))

		else:
			winki.lib.set_button_label(button_detect, _('Start _detection'))

		area.set_sensitive(enable)
		forward1.set_sensitive(not enable)
		forward10.set_sensitive(not enable)
		forward30.set_sensitive(not enable)
		backward1.set_sensitive(not enable)
		backward10.set_sensitive(not enable)
		backward30.set_sensitive(not enable)
		return

	def parse_crop(self, line):
		pos_start = string.find(line, '-vf crop=')
		if pos_start < 0:
			winki.lib.debug('PageCrop.parse_crop()', '-vf crop= not found')
			return

		pos_end = string.find(line, ')', pos_start + 1)
		if pos_end < 0 or pos_end <= pos_start:
			winki.lib.debug('PageCrop.parse_crop()', ') not found')
			return

		chunks = string.split(line[pos_start + len('-vf crop=') : pos_end], ':')
		if len(chunks) != 4:
			winki.lib.debug('PageCrop.parse_crop()', 'crop values != 4')
			return

		spin_w = self.xml.get_widget('spinbutton_w')
		spin_h = self.xml.get_widget('spinbutton_h')
		spin_x = self.xml.get_widget('spinbutton_x')
		spin_y = self.xml.get_widget('spinbutton_y')

		spin_w.set_text(chunks[0])
		spin_h.set_text(chunks[1])
		spin_x.set_text(chunks[2])
		spin_y.set_text(chunks[3])
		return

