# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module gobject...'
	import gobject

	print 'Importing module winki.lib...'
	import winki.lib

	print 'Importing module winki.build...'
	import winki.build

	print 'Importing module winki.base_page...'
	import winki.base_page

except:
	error_quit(0, 1)

COLUMN_ENABLED = 0
COLUMN_TYPE = 1
COLUMN_CHUNKNR = 2
COLUMN_COMMAND = 3
COLUMN_PASSNR = 4

def on_toggle_enabled(cell, path, model):
	it = model.get_iter((int(path),))
	enabled = model.get_value(it, COLUMN_ENABLED)
	
	enabled = not enabled
	
	model.set(it, COLUMN_ENABLED, enabled)
	return True

def on_edit_command(cell, path, text, model):
	it = model.get_iter((int(path),))
	model.set(it, COLUMN_COMMAND, text)
	return True

class PageEdit(winki.base_page.BasePage):
	def __init__(self, xml, app):
		winki.base_page.BasePage.__init__(self, xml, app)
		self.model = None
		return
		
	def check_widgets(self):
		widgets = ['treeview1']

		winki.lib.check_widgets(self.xml, widgets)
		return

	def on_prepare(self):
		cmds = self.app.project.get_commands()

		self.create_model(cmds)
		self.add_columns()
		return True

	def on_next(self):
		return self.update_project()

	def on_prev(self):
		self.app.project.reset_commands()
		return True

	def can_exit(self):
		return True

	def update_project(self):
		commands = []
		for row in self.model:
			cmd = row[COLUMN_COMMAND]
			kind = row[COLUMN_TYPE]
			chunk_nr = int(row[COLUMN_CHUNKNR]) - 1
			pass_nr = int(row[COLUMN_PASSNR]) - 1
			enabled = row[COLUMN_ENABLED]

			command = None
			if kind == 'vbitrate' or kind == 'unlink' or kind == 'unlink aux' or kind == 'rename':
				cmd = winki.lib.list_from_string(cmd)
				command = winki.build.UserCommand(cmd, kind, pass_nr, False)
			else:
				command = winki.build.UserCommand(cmd, kind, pass_nr)

			command.set_enabled(enabled)
			command.set_chunk_nr(chunk_nr)
			commands.append(command)

		self.app.project.set_commands(commands)
		return True

	def show_help(self):
		return winki.base_page.BasePage.show_help(self, 'edit.html')

	def create_model(self, cmds):
		#del self.model
		self.model = gtk.ListStore(gobject.TYPE_BOOLEAN,
				      gobject.TYPE_STRING,
				      gobject.TYPE_STRING,
				      gobject.TYPE_STRING,
				      gobject.TYPE_STRING)

		for cmd in cmds:
			it = self.model.append()
			self.model.set(it, COLUMN_ENABLED, cmd.get_enabled(),
				  COLUMN_TYPE, cmd.get_type(),
				  COLUMN_CHUNKNR, '%d' % (cmd.get_chunk_nr() + 1),
				  COLUMN_COMMAND, cmd.get_line(),
				  COLUMN_PASSNR, '%d' % (cmd.get_pass_nr() + 1))
		return

	def add_columns(self):
	    treeview = self.xml.get_widget('treeview1')
	    treeview.set_model(self.model)

	    renderer = gtk.CellRendererToggle()
	    renderer.connect('toggled', on_toggle_enabled, self.model)
	    
	    column = gtk.TreeViewColumn(_('Enabled'), renderer, active=COLUMN_ENABLED)
	    treeview.append_column(column)

	    column = gtk.TreeViewColumn(_('Type'), gtk.CellRendererText(),
					text=COLUMN_TYPE)
	    treeview.append_column(column)

	    treeview.insert_column_with_attributes(-1, _('Chunk'), gtk.CellRendererText(), text=COLUMN_CHUNKNR)

	    renderer = gtk.CellRendererText()
	    renderer.set_property('editable', True)
	    renderer.connect('edited', on_edit_command, self.model)
	    treeview.insert_column_with_attributes(-1, _('Command'), renderer, text=COLUMN_COMMAND)

	    treeview.insert_column_with_attributes(-1, _('Pass'), gtk.CellRendererText(), text=COLUMN_PASSNR)

	    return
	    
