# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module winki.lib...'
	import winki.lib

	print 'Importing module winki.base_page...'
	import winki.base_page

except:
	error_quit(0, 1)

class PageEncoding(winki.base_page.BasePage):
	def __init__(self, xml, app):
		winki.base_page.BasePage.__init__(self, xml, app)
		return
		
	def check_widgets(self):
		widgets = ['label_multi_pass', 'label_stream_copy', 'label_create_vcd',
		'label_create_svcd', 'radiobutton_copy_stream',
		'radiobutton_multiple_pass', 'radiobutton_vcd', 'radiobutton_svcd',
		'checkbutton_stream_mix']

		winki.lib.check_widgets(self.xml, widgets)
		return

	def enable_ui(self):
		has_mplayer = self.app.has_program('mplayer')
		has_mencoder = self.app.has_program('mencoder')
		has_stream_merger = self.app.has_program('Stream merger')
		has_audio_compressor = self.app.has_program('Audio compressor')
		has_vcdimager = self.app.has_program('vcdimager')
		has_mplexer = self.app.has_program('Multiplexer')

		copy = (has_mplayer and has_mencoder)
		multi_pass = (copy and has_stream_merger and has_audio_compressor)
		vcd = (copy and has_vcdimager and has_mplexer)

		radio_multi = self.xml.get_widget('radiobutton_multiple_pass')
		radio_copy = self.xml.get_widget('radiobutton_copy_stream')
		radio_vcd = self.xml.get_widget('radiobutton_vcd')
		radio_svcd = self.xml.get_widget('radiobutton_svcd')

		radio_copy.set_sensitive(copy)
		radio_multi.set_sensitive(multi_pass)
		radio_vcd.set_sensitive(vcd)
		radio_svcd.set_sensitive(vcd)
		return

	def on_prepare(self):
		radio_multi = self.xml.get_widget('radiobutton_multiple_pass')
		radio_copy = self.xml.get_widget('radiobutton_copy_stream')
		radio_vcd = self.xml.get_widget('radiobutton_vcd')
		radio_svcd = self.xml.get_widget('radiobutton_svcd')

		self.enable_ui()

		if not self.app.project.get_encoding():
			enc = self.app.project.get_default_encoding()
			if enc and len(enc):
				if enc == 'multi pass':
					radio_multi.set_active(True)
					radio_multi.grab_focus()
				elif enc == 'copy':
					radio_copy.set_active(True)
					radio_copy.grab_focus()
				elif enc == 'vcd':
					radio_vcd.set_active(True)
					radio_vcd.grab_focus()
				elif enc == 'svcd':
					radio_svcd.set_active(True)
					radio_svcd.grab_focus()
				else:
					# new encoding?
					assert(0)
			else:
				radio_multi.set_active(True)
				radio_multi.grab_focus()
		else:
			self.focus_active()

		return True

	def on_next(self):
		return self.update_project()

	def on_prev(self):
		return self.update_project()

	def can_exit(self):
		return True

	def show_help(self):
		return winki.base_page.BasePage.show_help(self, 'encoding.html')

	def focus_active(self):
		radio_multi = self.xml.get_widget('radiobutton_multiple_pass')
		radio_copy = self.xml.get_widget('radiobutton_copy_stream')
		radio_vcd = self.xml.get_widget('radiobutton_vcd')
		radio_svcd = self.xml.get_widget('radiobutton_svcd')

		copy_set = (self.app.project.get_encoding() == 'copy')
		multi_set = (self.app.project.get_encoding() == 'multi pass')
		vcd_set = (self.app.project.get_encoding() == 'vcd')
		svcd_set = (self.app.project.get_encoding() == 'svcd')

		winki.lib.debug('PageEncoding.focus_active()', 'method %s' % self.app.project.get_encoding())
		if copy_set:
			radio_copy.grab_focus()
		elif multi_set:
			radio_multi.grab_focus()
		elif vcd_set:
			radio_vcd.grab_focus()
		elif svcd_set:
			radio_svcd.grab_focus()
		else:
			winki.lib.debug('PageEncoding.focus_active()', 'unknown method requested', winki.lib.error)
			assert(False)
		return

	def encoding_toggled(self):
		radio_multi = self.xml.get_widget('radiobutton_multiple_pass')
		radio_copy = self.xml.get_widget('radiobutton_copy_stream')
		radio_vcd = self.xml.get_widget('radiobutton_vcd')
		radio_svcd = self.xml.get_widget('radiobutton_svcd')
		check_merge = self.xml.get_widget('checkbutton_stream_mix')

		if radio_multi.get_active():
			check_merge.set_sensitive(False)
		elif radio_copy.get_active():
			check_merge.set_sensitive(True)
		elif radio_vcd.get_active():
			check_merge.set_sensitive(False)
		elif radio_svcd.get_active():
			check_merge.set_sensitive(False)
		else:
			# new radio button?
			assert(0)
		return

	def stream_mix_toggled(self):
		check_merge = self.xml.get_widget('checkbutton_stream_mix')
		label_copy = self.xml.get_widget('label_stream_copy')

		disabled = _('Dump the selected streams to files')
		enabled = _('Dump the selected streams to one file')

		if check_merge.get_active():
			label_copy.set_text(enabled)
		else:
			label_copy.set_text(disabled)
		return

	def update_project(self):
		radio_multi = self.xml.get_widget('radiobutton_multiple_pass')
		radio_copy = self.xml.get_widget('radiobutton_copy_stream')
		radio_vcd = self.xml.get_widget('radiobutton_vcd')
		radio_svcd = self.xml.get_widget('radiobutton_svcd')
		check_merge = self.xml.get_widget('checkbutton_stream_mix')

		if winki.lib.is_radio_active(radio_multi) and radio_multi.get_property('sensitive'):
			self.app.project.set_encoding('multi pass')

		elif winki.lib.is_radio_active(radio_copy) and radio_copy.get_property('sensitive'):
			self.app.project.set_encoding('copy')
			self.app.project.set_mix_streams(check_merge.get_active())

		elif winki.lib.is_radio_active(radio_vcd) and radio_vcd.get_property('sensitive'):
			self.app.project.set_encoding('vcd')

		elif winki.lib.is_radio_active(radio_svcd) and radio_svcd.get_property('sensitive'):
			self.app.project.set_encoding('svcd')

		else:
			winki.lib.debug('PageEncoding.update_project()', 'unknown method requested', winki.lib.error)
			return False

		return True

