# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module string...'
	import string

	print 'Importing module winki.lib...'
	import winki.lib

	print 'Importing module winki.command...'
	import winki.command

	print 'Importing module winki.build...'
	import winki.build

	print 'Importing module winki.base_page...'
	import winki.base_page

except:
	error_quit(0, 1)

class PageInfo(winki.base_page.BasePage):
	def __init__(self, xml, app):
		winki.base_page.BasePage.__init__(self, xml, app)
		self.is_generated = True
		self.show_error = True
		self.command = None

		self.file_audio = 0
		self.file_video = 0
		self.file_subtitles = 0
		self.len_in_seconds = 0
		self.current_iteration = 0
		self.refresh_ratio = 5
		self.kind = 'unknown'
		return
		
	def check_widgets(self):
		widgets = ['button_output_file', 'entry_artist',
		'entry_comment', 'entry_copyright', 'entry_genre',
		'entry_name', 'entry_output_file', 'entry_srcform',
		'entry_subject', 'label_artist', 'label_comment',
		'label_copyng', 'label_genre', 'label_name',
		'label_output_file', 'label_srcform', 'label_subject',
		'label_scan', 'progressbar_scan', 'button_cancel_scan']

		winki.lib.check_widgets(self.xml, widgets)
		return

	def on_prepare(self):
		entry_name = self.xml.get_widget('entry_name')
		entry_artist = self.xml.get_widget('entry_artist')
		entry_genre = self.xml.get_widget('entry_genre')
		entry_subject = self.xml.get_widget('entry_subject')
		entry_copyright = self.xml.get_widget('entry_copyright')
		entry_srcform = self.xml.get_widget('entry_srcform')
		entry_comment = self.xml.get_widget('entry_comment')

		if self.app.project.get_name() != None:
			entry_name.set_text(self.app.project.get_name())

		if self.app.project.get_artist() != None:
			entry_artist.set_text(self.app.project.get_artist())

		if self.app.project.get_genre() != None:
			entry_genre.set_text(self.app.project.get_genre())

		if self.app.project.get_subject() != None:
			entry_subject.set_text(self.app.project.get_subject())

		if self.app.project.get_copyright() != None:
			entry_copyright.set_text(self.app.project.get_copyright())

		if self.app.project.get_srcform() != None:
			entry_srcform.set_text(self.app.project.get_srcform())
		else:
			src = self.app.project.get_source()
			if src and src == 'dvd':
				entry_srcform.set_text(_('Digital Versatile Disk'))
			elif src and src == 'vcd':
				entry_srcform.set_text(_('Video Compact Disc'))

		if self.app.project.get_comment() != None:
			entry_comment.set_text(self.app.project.get_comment())
		else:
			entry_comment.set_text(self.app.project.get_str_comment())

		self.name_changed()

		self.enable_ui(True)
		entry_name.grab_focus()

		return True

	def set_main_window_title(self):
		wnd = self.xml.get_widget('collect_window')

		name = self.app.project.get_name()
		src_name = self.app.project.get_source_filename(True)
		dest_name = self.app.project.get_outputdir(True)

		src = self.app.project.get_source()
		if src == 'file':
			# if the user supplied a name - use it
			if name and name != '':
				wnd.set_title(name)

			# or use the source file name
			elif src_name and src_name != '':
				wnd.set_title(src_name)

			# or use the output file name
			elif dest_name and dest_name != '':
				wnd.set_title(dest_name)
			else:
				# no input and output file names???
				assert(0)

		elif src == 'dvd' or src == 'vcd':
			# if the user supplied a name - use it
			if name and name != '':
				wnd.set_title(name)

			# or use the output file name
			elif dest_name and dest_name != '':
				wnd.set_title(dest_name)

			else:
				# no output file name???
				assert(0)
		else:
			# new source?
			assert(0)
		return

	def on_next(self):
		if not self.can_exit():
			return False

		source = self.app.project.get_source()

		if source == 'dvd' and not self.scan_dvd():
			winki.lib.debug('PageInfo.on_next()', 'failed to scan the dvd', winki.lib.error)
			if self.show_error:
				dvd_dev = self.app.project.get_dvd_device()
				txt = _('Please make sure you inserted the DVD into the drive %s.') %dvd_dev
				winki.lib.alert(_('Could not access DVD'), txt, 'error')
			return False

		if source == 'file' and not self.scan_file():
			winki.lib.debug('PageInfo.on_next()', 'failed to scan the file', winki.lib.error)
			if self.show_error:
				fname = self.app.project.get_source_filename()
				txt = _('Failed to scan the movie file %s') % fname
				winki.lib.alert(_('File scan failed'), txt, 'error')
			return False

		if source == 'vcd' and not self.scan_vcd():
			winki.lib.debug('PageInfo.on_next()', 'failed to scan the vcd', winki.lib.error)
			if self.show_error:
				vcd_dev = self.app.project.get_cd_device()
				txt = _('Please make sure you inserted the VCD into the drive %s.') % vcd_dev
				winki.lib.alert(_('Could not access VCD'), txt, 'error')
			return False

		if source != 'dvd' and source != 'file' and source != 'vcd':
			assert(0)

		return self.update_project()


	def on_prev(self):
		return True

	def can_exit(self):
		return (not self.command)

	def update_project(self):
		entry_name = self.xml.get_widget('entry_name')
		entry_artist = self.xml.get_widget('entry_artist')
		entry_genre = self.xml.get_widget('entry_genre')
		entry_subject = self.xml.get_widget('entry_subject')
		entry_copyright = self.xml.get_widget('entry_copyright')
		entry_srcform = self.xml.get_widget('entry_srcform')
		entry_comment = self.xml.get_widget('entry_comment')
		entry_output_file = self.xml.get_widget('entry_output_file')

		self.app.project.set_name(entry_name.get_text())
		self.app.project.set_artist(entry_artist.get_text())
		self.app.project.set_genre(entry_genre.get_text())
		self.app.project.set_subject(entry_subject.get_text())
		self.app.project.set_copyright(entry_copyright.get_text())
		self.app.project.set_srcform(entry_srcform.get_text())
		self.app.project.set_comment(entry_comment.get_text())

		outfile = entry_output_file.get_text()
		if not outfile or not len(outfile):
			winki.lib.debug('PageInfo.update_project()', 'no output file name selected', winki.lib.error)
			primary = _('No output folder selected')
			secondary = _('You have to select an output folder')
			winki.lib.alert(primary, secondary, 'error')
			entry_output_file.grab_focus()
			return False

		fname = winki.lib.normalize_path(outfile)
		warn = self.app.project.get_use_warning('dest_dir')
		if warn and winki.lib.does_path_exist(fname) and not winki.lib.is_dir_empty(fname):
			primary = _('Folder %s already exist and is not empty. Are you sure you want to use it?') % fname
			secondary = _('Winki the Ripper will create temporary files in this folder. If the folder is not empty you may loose data or the program may not work as expected')
			state = winki.lib.alert(primary, secondary, 'warning', _('Use anyway'))
			if state != 'ok':
				return False

		if not self.submit_file_info():
			return False

		self.app.project.set_outputdir(fname)
		self.app.project.build_directories()

		self.set_main_window_title()
		return True

	def show_help(self):
		return winki.base_page.BasePage.show_help(self, 'info.html')

	def get_file_name(self):
		entry_output_file = self.xml.get_widget('entry_output_file')
		fname = entry_output_file.get_text()

		self.is_generated = False

		if not winki.lib.is_dir(fname):
			winki.lib.debug('PageInfo.get_file_name()', 'should be folder', winki.lib.error)
			return ''

		if not fname or not len(fname):
			winki.lib.debug('PageInfo.get_file_name()', 'error building path', winki.lib.error)
			return ''

		return fname

	def set_file_name(self, fname):
		entry_output_file = self.xml.get_widget('entry_output_file')
		entry_output_file.set_text(fname)
		return

	def name_changed(self):
		entry_name = self.xml.get_widget('entry_name')
		entry_output_file = self.xml.get_widget('entry_output_file')

		name = entry_name.get_text()
		if not self.is_generated:
			return

		output = winki.lib.fix_name(winki.lib.build_path([self.app.project.get_default_output_dir(), name]))
		if not output or not len(output):
			entry_output_file.set_text('')
			return
			
		entry_output_file.set_text(output)
		return

	def toggle_generated(self):
		entry_output_file = self.xml.get_widget('entry_output_file')

		if winki.lib.entry_has_focus(entry_output_file):
			self.is_generated = False
			return

		return

	def on_cancel(self):
		winki.lib.debug('PageInfo.on_cancel()', 'lsdvd canceled')
		self.show_error = False
		self.command.stop()
		self.command = None
		return

	def scan_dvd(self):
		self.enable_ui(False)
		self.show_error = True

		label_scan = self.xml.get_widget('label_scan')
		progress = self.xml.get_widget('progressbar_scan')

		steps = 5.0
		encoding = self.app.project.get_encoding()

		label_scan.set_text(_('Searching for DVD titles'))
		self.command = winki.command.LsDVD(self.app.project)
		winki.lib.update_progress(progress, '', 1.0 / steps)
		self.command.read_titles()
		if not self.command:
			self.enable_ui(True)
			winki.lib.debug('PageInfo.scan_dvd()', 'no command')
			return False

		if self.command.errors and len(self.command.errors) > 0:
			for err in self.command.errors:
				winki.lib.debug('PageInfo.scan_dvd()', err, winki.lib.error)
			self.enable_ui(True)
			self.command.stop()
			self.command = None
			winki.lib.debug('PageInfo.scan_dvd()', 'errors')
			return False

		if not self.command.titles or not(len(self.command.titles)):
			self.enable_ui(True)
			self.command.stop()
			self.command = None
			winki.lib.debug('PageInfo.scan_dvd()', 'no titles')
			return False

		self.app.project.set_titles(self.command.titles)
		self.app.project.set_current_title(self.command.longest_title)

		winki.lib.update_progress(progress, '', 2.0 / steps)
		label_scan.set_text(_('Searching for chapters'))
		self.command.read_chapters()
		if not self.command:
			self.app.project.reset_dvd()
			self.enable_ui(True)
			winki.lib.debug('PageInfo.scan_dvd()', 'no chapters')
			return False

		self.app.project.set_chapters(self.command.chapters)
		self.app.project.set_chapter_times(self.command.chapter_times)

		winki.lib.update_progress(progress, '', 3.0 / steps)
		label_scan.set_text(_('Searching for audio streams'))
		self.command.read_audio()
		if not self.command:
			self.app.project.reset_dvd()
			self.enable_ui(True)
			winki.lib.debug('PageInfo.scan_dvd()', 'no audio')
			return False

		self.app.project.set_audio(self.command.audio)

		label_scan.set_text(_('Searching for subtitles'))
		winki.lib.update_progress(progress, '', 4.0 / steps)
		self.command.read_subtitles()
		if not self.command:
			self.app.project.reset_dvd()
			self.enable_ui(True)
			winki.lib.debug('PageInfo.scan_dvd()', 'no subtitles')
			return False

		self.app.project.set_subtitles(self.command.subtitles)

		# DVD info
		label_scan.set_text(_('Getting scaling information'))
		winki.lib.update_progress(progress, '', 5.0 / steps)

		width = self.app.project.get_vcd_width()
		aspect = self.app.project.get_vcd_aspect()

		builder = winki.build.Builder(self.app.project)
		cmd = builder.build_scale_detect_command(width * aspect)
		self.command = winki.command.Command(cmd, self.get_scale)
		self.command.bufsize = 20
		self.command.run()

		winki.lib.update_progress(progress, '', 0.0)
		label_scan.set_text('')
		self.enable_ui(True)
		self.command = None
		return True

	def scan_file(self):
		self.file_audio = 0
		self.file_video = 0
		self.len_in_seconds = 0

		self.enable_ui(False)

		label_scan = self.xml.get_widget('label_scan')
		label_scan.set_text(_('Scanning file'))
		self.current_iteration = 0

		builder = winki.build.Builder(self.app.project)
		cmd = builder.build_identify_command()
		self.command = winki.command.Command(cmd, self.extract_file_info)
		self.command.bufsize = 10

		self.command.run()
		self.command.stop()
		self.command = None

		encoding = self.app.project.get_encoding()
		if encoding == 'vcd' or encoding == 'svcd':
			label_scan.set_text(_('Getting scaling information'))

			width = self.app.project.get_vcd_width()
			aspect = self.app.project.get_vcd_aspect()

			builder = winki.build.Builder(self.app.project)
			cmd = builder.build_scale_detect_command(width * aspect)
			self.command = winki.command.Command(cmd, self.get_scale)
			self.command.bufsize = 20
			self.command.run()

		label_scan.set_text('')
		self.enable_ui(True)
		return True

	def scan_vcd(self):
		self.enable_ui(False)
		self.show_error = True

		label_scan = self.xml.get_widget('label_scan')
		progress = self.xml.get_widget('progressbar_scan')

		steps = 1.0
		encoding = self.app.project.get_encoding()
		if encoding == 'vcd':
			steps = 2.0

		label_scan.set_text(_('Searching for VCD titles'))
		cmd = 'mplayer vcd://2 -v -identify -frames 0 -nocache'
		self.command = winki.command.Command(cmd, self.extract_file_info)
		self.command.bufsize = 20
		self.command.run()

		if encoding == 'vcd' or encoding == 'svcd':
			label_scan.set_text(_('Getting scaling information'))
			winki.lib.update_progress(progress, '', 2.0 / steps)

			width = self.app.project.get_vcd_width()
			aspect = self.app.project.get_vcd_aspect()

			builder = winki.build.Builder(self.app.project)
			cmd = builder.build_scale_detect_command(width * aspect)
			self.command = winki.command.Command(cmd, self.get_scale)
			self.command.bufsize = 20
			self.command.run()

		winki.lib.update_progress(progress, '', 0.0)
		label_scan.set_text('')
		self.enable_ui(True)
		self.command = None
		return True

	def extract_file_info(self, line):
		if (self.current_iteration % self.refresh_ratio) == 0:
			progress = self.xml.get_widget('progressbar_scan')
			progress.pulse()
		self.current_iteration += 1

		if string.find(line, '[mkv] Found the head...') > -1:
			self.kind = 'mkv'

		if string.find(line, '==> Found audio stream:') > -1 and self.kind != 'mkv':
			self.file_audio += 1

		if string.find(line, '[mkv] Track ID') > -1 and string.find(line, '-aid') > -1 and string.find(line, '-alang') > -1:
			self.file_audio += 1

		if string.find(line, '[mkv] Track ID') > -1 and string.find(line, '-sid') > -1 and string.find(line, '-slang') > -1:
			self.file_subtitles += 1

		if string.find(line, '==> Found video stream:') > -1:
			self.file_video += 1

		pos = string.find(line, 'ID_LENGTH=')
		if pos > -1:
			pos2 = len(line)
			pos1 = pos + len('ID_LENGTH=')
			if pos2 > pos1:
				self.len_in_seconds = winki.lib.to_int(line[pos1 : pos2]) 
			else:
				winki.lib.debug('PageInfo.extract_file_info()', 'invalid length string', winki.lib.error)
		return 

	def submit_file_info(self):
		src = self.app.project.get_source()
		if src != 'file' and src != 'vcd':
			return True

		if self.file_video < 1 or self.len_in_seconds < 1:
			text = _('%s does not appear to be a valid movie file. It contains %d audio and %d video streams with length %d seconds') % (self.app.project.get_source_filename(), self.file_audio, self.file_video, self.len_in_seconds)
			winki.lib.alert(_('Could not use file'), text, 'error')
			return False

		audio_langs = [_('none')]
		for i in range(self.file_audio):
			item = (_('Audio stream %d') % (i + 1))
			audio_langs.append(item)
		self.app.project.set_audio(audio_langs)

		#subtitle_langs = [_('none')]
		#for i in range(self.file_subtitles):
		#	item = (_('Subtitle stream %d') % (i + 1))
		#	subtitle_langs.append(item)
		#self.app.project.set_subtitles([subtitle_langs])

		self.app.project.set_file_time(self.len_in_seconds)
		return True

	def enable_ui(self, enable):
		button_output = self.xml.get_widget('button_output_file')
		entry_artist = self.xml.get_widget('entry_artist')
		entry_comment = self.xml.get_widget('entry_comment')
		entry_copyright = self.xml.get_widget('entry_copyright')
		entry_genre = self.xml.get_widget('entry_genre')
		entry_name = self.xml.get_widget('entry_name')
		entry_output = self.xml.get_widget('entry_output_file')
		entry_srcform = self.xml.get_widget('entry_srcform')
		entry_subject = self.xml.get_widget('entry_subject')
		label_artist = self.xml.get_widget('label_artist')
		label_comment = self.xml.get_widget('label_comment')
		label_copyng = self.xml.get_widget('label_copyng')
		label_genre = self.xml.get_widget('label_genre')
		label_name = self.xml.get_widget('label_name')
		label_output = self.xml.get_widget('label_output_file')
		label_srcform = self.xml.get_widget('label_srcform')
		label_subject = self.xml.get_widget('label_subject')
		label_scan = self.xml.get_widget('label_scan')
		progress = self.xml.get_widget('progressbar_scan')
		button_cancel = self.xml.get_widget('button_cancel_scan')

		encoding = self.app.project.get_encoding()
		vcd = (encoding == 'vcd' or encoding == 'svcd')

		button_output.set_sensitive(enable)
		entry_artist.set_sensitive(enable and not vcd)
		entry_comment.set_sensitive(enable and not vcd)
		entry_copyright.set_sensitive(enable and not vcd)
		entry_genre.set_sensitive(enable and not vcd)
		entry_name.set_sensitive(enable)
		entry_output.set_sensitive(enable)
		entry_srcform.set_sensitive(enable and not vcd)
		entry_subject.set_sensitive(enable and not vcd)
		label_comment.set_sensitive(enable and not vcd)
		label_artist.set_sensitive(enable and not vcd)
		label_copyng.set_sensitive(enable and not vcd)
		label_genre.set_sensitive(enable and not vcd)
		label_name.set_sensitive(enable)
		label_output.set_sensitive(enable)
		label_srcform.set_sensitive(enable and not vcd)
		label_subject.set_sensitive(enable and not vcd)

		label_scan.set_sensitive(not enable)
		progress.set_sensitive(not enable)
		button_cancel.set_sensitive(not enable)

		if enable:
			progress.set_fraction(0.0)	

		return

	def get_scale(self, text):
		if string.find(text, 'VO: [null] ') > -1:
			try:
				start = len('VO: [null] ')
				items = string.split(text[start : ])
				x, y = string.split(items[0], 'x')
				self.app.project.set_scale_detected(x, y)
			except:
				pass

		if string.find(text, 'ID_VIDEO_ASPECT=') > -1:
			pos = string.find(text, '=')
			self.app.project.set_aspect_ratio(text[pos + 1 : ])

		return
