# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module string...'
	import string

	print 'Importing module time...'
	import time

	print 'Importing module threading...'
	import threading

	print 'Importing module os.path...'
	import os.path

	print 'Importing module winki.lib...'
	import winki.lib

	print 'Importing module winki.command...'
	import winki.command

	print 'Importing module winki.alert...'
	import winki.alert

	print 'Importing module winki.base_page...'
	import winki.base_page

except:
	error_quit(0, 1)

class PageRip(winki.base_page.BasePage):
	def __init__(self, xml, app):
		winki.base_page.BasePage.__init__(self, xml, app)
		self.program = None
		self.progress = None
		self.timer = None
		self.line = ''
		self.canceled = True
		self.vbitrate = -1
		self.len_seconds = []
		self.refresh_ratio = 1
		self.current_iteration = 0
		self.chunk_nr = -1
		self.start_time = 0

		self.labels = None
		return
		
	def check_widgets(self):
		widgets = ['button_rip', 'label_current',
		'progress_current', 'label_misc1', 'label_misc2',
		'label_misc3', 'label_misc4', 'label_misc5', 'label_misc6',
		'label_misc7', 'label_misc8', 'label_misc9', 'label_misc10',
		'label_estimated_time', 'label_elapsed_time', 'label_chunk',
		'label_elapsed_time_label']

		winki.lib.check_widgets(self.xml, widgets)
		return

	def on_prepare(self):
		self.progress = self.xml.get_widget('progress_current')

		encoding = self.app.project.get_encoding()
		if encoding == 'multi pass':
			self.on_prepare_multi_pass()

		elif encoding == 'copy':
			self.on_prepare_copy()

		elif encoding == 'vcd' or encoding == 'svcd':
			self.labels = { _('Creating video index') : 3,
					_('Encoding') : 4,
					_('Remultipexing streams') : 5,
					_('Creating image') : 6}

		else:
			assert(0)

		self.init_images()
		self.on_cancel_rip()
		return True

	def on_prepare_multi_pass(self):
		self.labels = { _('Dumping audio') : 3,
				_('Compressing audio') : 4,
				_('Dumping subtitles') : 5,
				_('Creating video index') : 6,
				_('Encoding video') : 7,
				_('Merging streams') : 8}

		audio_count = self.app.project.get_audio_count()
		if  audio_count < 1:
			del self.labels[_('Dumping audio')]
			del self.labels[_('Compressing audio')]
			self.labels[_('Dumping subtitles')] = 3
			self.labels[_('Creating video index')] = 4
			self.labels[_('Encoding video')] = 5
			self.labels[_('Merging streams')] = 6

		if self.app.project.get_sub_count() < 1 or self.app.project.get_mix_subtitles():
			del self.labels[_('Dumping subtitles')]
			if audio_count < 1:
				self.labels[_('Creating video index')] = 3
				self.labels[_('Encoding video')] = 4
				self.labels[_('Merging streams')] = 5
			else:
				self.labels[_('Creating video index')] = 5
				self.labels[_('Encoding video')] = 6
				self.labels[_('Merging streams')] = 7
		return

	def on_prepare_copy(self):
		self.labels = { _('Dumping audio') : 3,
				_('Dumping subtitles') : 4,
				_('Dumping video') : 5,
				_('Fixing video index') : 6,
				_('Merging streams') : 7}

		audio_count = self.app.project.get_audio_count()
		mix = self.app.project.get_mix_streams()

		if audio_count < 1:
			del self.labels[_('Dumping audio')]
			self.labels[_('Dumping subtitles')] = 3
			self.labels[_('Dumping video')] = 4
			self.labels[_('Fixing video index')] = 5
			if mix:
				self.labels[_('Merging streams')] = 6

		if self.app.project.get_sub_count() < 1 or self.app.project.get_mix_subtitles():
			del self.labels[_('Dumping subtitles')]
			if audio_count < 1:
				self.labels[_('Dumping video')] = 3
				self.labels[_('Fixing video index')] = 4
				if mix:
					self.labels[_('Merging streams')] = 5
			else:
				self.labels[_('Dumping video')] = 4
				self.labels[_('Fixing video index')] = 5
				if mix:
					self.labels[_('Merging streams')] = 6

		if not mix:
			del self.labels[_('Merging streams')]

		return

	def on_next(self):
		return (not self.program)

	def on_prev(self):
		if self.program:
			return False

		if self.app.project.get_user_mode() != 'Advanced':
			self.app.project.reset_commands()

		return True

	def can_exit(self):
		if self.program:
			return False

		return True

	def show_help(self):
		return winki.base_page.BasePage.show_help(self, 'encode.html')

	def cleanup(self):
		if self.program:
			self.program.stop()
		self.program = None
		self.chunk_nr = -1
		self.start_time = 0

		self.app.project.unlink_aux()
		return

	def on_toggle_rip(self):
		if not self.program:
			self.on_start_rip()
		else:
			if self.app.project.get_use_warning('abort'):
				info = winki.alert.AlertWarning()
				primary = _('Do you really want to abort the current process?')
				secondary = _('Aborting now will leave your work in inconsistent state.')
				info.show(primary, secondary, _('Abort anyway'))
				if info.state != 'ok':
					return
			self.on_cancel_rip()
		return

	def log_intended(self, cmds):
		fname = self.app.project.get_outputdir()
		if not fname or not len(fname):
			winki.lib.debug('PageRip.log_intended()', 'no dest dir', winki.lib.error)
			return

		fname = os.path.join(fname, 'winki_intended.log')
		try:
			fout = open(fname, 'w')
			for command in cmds:
				if not self.check_command(command):
					continue

				cmd = command.get_line()
				kind = command.get_type()
				chunk = command.get_chunk_nr()
				ps = command.get_pass_nr()

				fout.write('type: %s\n' % kind)
				fout.write('pass: %s\n' % ps)
				fout.write('chunk: %s\n' % chunk)
				fout.write('command: %s\n' % cmd)
				fout.write('--------------------------\n')
			fout.close()
		except IOError:
			winki.lib.debug('PageRip.log_intended()', 'cannot write to %s' % fname, winki.lib.error)
		return
		
	def get_time_stamp(self):
		elapsed = int(time.time() - self.start_time)
		return 'elapsed time: %s' % self.format_time(elapsed)

	def log_command(self, command):
		fname = self.app.project.get_outputdir()
		if not fname or not len(fname):
			winki.lib.debug('PageRip.log_command()', 'no dest dir', winki.lib.error)
			return

		fname = os.path.join(fname, 'winki_commands.log')
		try:
			fout = open(fname, 'a')
			fout.write('# %s\n' % self.get_time_stamp())
			fout.write('%s\n' % command)
			fout.close()
		except IOError:
			winki.lib.debug('PageRip.log_command()', 'cannot write to %s' % fname, winki.lib.error)
		return
		
	def remove_log(self):
		fname = self.app.project.get_outputdir()
		if not fname or not len(fname):
			winki.lib.debug('PageRip.remove_log()', 'no dest dir', winki.lib.error)
			return

		command_name = os.path.join(fname, 'winki_commands.log')
		intended_name = os.path.join(fname, 'winki_intended.log')

		self.app.project.unlink_file(command_name)
		self.app.project.unlink_file(intended_name)
		return

	def check_command(self, cmd):
		if not cmd.get_enabled():
			return False

		return True

	def on_start_rip(self):
		if not self.app.project.cd_wd():
			return
			
		cmds = self.app.project.get_commands()

		if not len(cmds):
			winki.lib.debug('PageRip.on_start_rip()', 'no commands', winki.lib.error)
			return

		self.remove_log()
		self.log_intended(cmds)

		self.canceled = False

		button_rip = self.xml.get_widget('button_rip')
		winki.lib.set_button_label(button_rip, _('_Abort'))
		self.init_timer()

		for cmd in cmds:
			if not self.check_command(cmd):
				continue

			if self.canceled:
				winki.lib.debug('PageRip.on_start_rip()', 'canceled')
				return

			self.update_chunk_label()

			winki.lib.update_progress(self.progress, '0%', 0.0)

			if not self.run_command(cmd):
				self.on_cancel_rip()
				return

		#self.app.success_rip()
		self.canceled = True
		self.cleanup()
		button_rip = self.xml.get_widget('button_rip')
		winki.lib.set_button_label(button_rip, _('_Start now!'))
		winki.lib.update_progress(self.progress, '0%', 0.0)
		label_elaps = self.xml.get_widget('label_elapsed_time_label')
		label_elaps.set_text(_('Total encoding time'))
		self.start_time = 0
		self.set_all_visible_labels('ready')
		return

	def on_cancel_rip(self):
		self.canceled = True
		self.cleanup()

		button_rip = self.xml.get_widget('button_rip')
		winki.lib.set_button_label(button_rip, _('_Start now!'))
		winki.lib.update_progress(self.progress, '0%', 0.0)

		self.set_all_visible_labels('frame')
		return

	def run_command(self, cmd):
		if self.program:
			winki.lib.debug('PageRip.run_command()', 'program instance already exist', winki.lib.error)
			return False

		cmdline = cmd.get_line()
		kind = cmd.get_type()
		stream_number = cmd.get_pass_nr() # means: pass over the data (eg. stream count)
		self.chunk_nr = cmd.get_chunk_nr() # means: number chunk (eg dvd splitted in 2 chunks)

		self.current_iteration = 0

		self.update_ui()

		if kind == 'vbitrate':
			result = self.on_vbitrate(cmdline)
			return result

		callback = None
		bufsize = -1
		if kind == 'video pass 1':
			cmdline, callback, bufsize = self.on_create_index(cmdline, stream_number)
			if not cmdline:
				return False

		if kind == 'video pass 2':
			cmdline, callback, bufsize = self.on_encode_video(cmdline, stream_number)
			if not cmdline:
				return False

		if kind == 'dump audio':
			callback, bufsize = self.on_dump_audio(stream_number)

		if kind == 'normalize audio':
			callback, bufsize = self.on_normalize_audio(stream_number)

		if kind == 'compress audio':
			callback, bufsize = self.on_compress_audio(stream_number)

		if kind == 'dump subtitles':
			callback, bufsize = self.on_dump_subtitles(stream_number)

		if kind == 'dump video':
			callback, bufsize = self.on_dump_video(stream_number)

		if kind == 'merge streams':
			callback, bufsize = self.on_merge_streams(stream_number)

		if kind == 'copy stream':
			callback, bufsize = self.on_copy_stream(stream_number)

		if kind == 'fix stream':
			callback, bufsize = self.on_fix_stream(stream_number)

		if kind == 'mplex':
			callback, bufsize = self.on_mplex_stream(stream_number)

		if kind == 'unlink':
			for fname in cmdline:
				self.app.project.unlink_file(fname)
				self.log_command('unlinking %s' % fname)
			return True

		if kind == 'unlink aux':
			self.app.project.unlink_aux()
			return True

		if kind == 'rename':
			self.log_command('renaming %s to %s' % (cmdline[0], cmdline[1]))
			return winki.lib.rename(cmdline[0], cmdline[1])

		if kind == 'build image':
			callback, bufsize = self.on_build_image(stream_number)

		if callback == None or bufsize < 0:
			winki.lib.debug('PageRip.run_command()', 'internal error', winki.lib.error)
			return False

		self.log_command(cmdline)
		self.program = winki.command.Command(cmdline, callback)
		self.program.bufsize = bufsize

		self.program.run()

		if self.timer:
			self.timer.cancel()
			self.timer = None

		if not self.canceled:
			self.program.stop()
			self.program = None
		return True

	def on_vbitrate(self, list_fnames):
		self.len_seconds = self.app.project.get_chunk_lenghts()
		if self.chunk_nr >= len(self.len_seconds):
			winki.lib.debug('PageRip().on_vbitrate()', 'nr_chunks >= len(seconds)', winki.lib.error)
			return False

		len_in_seconds = self.len_seconds[self.chunk_nr]
		info_list = self.app.project.compute_bitrate(list_fnames, len_in_seconds)
		
		vbitrate, target, penalty, occupied = info_list
		text = 'vbitrate: target=%d, penalty=%d, occupied=%d, length=%d, vbitrate=%d' % (target, penalty, occupied, len_in_seconds, int(vbitrate))
		self.log_command(text)

		if vbitrate < 0:
			self.vbitrate = -1
			text = _('The computed video bitrate is negative. Please select bigger chunk size or smaller movie chunk to be encoded.')
			error = winki.alert.AlertError()
			error.show(_('Negative video bitrate'), text)
			if self.program:
				self.program.stop()
			self.program = None
			return False

		warn_low = self.app.project.get_use_warning('bitrate_low')
		warn_high = self.app.project.get_use_warning('bitrate_high')
		self.vbitrate = int(vbitrate)
		lower_threshold = self.app.project.get_lower_threshold()
		upper_threshold = self.app.project.get_upper_threshold()
		if lower_threshold > 0 and vbitrate < lower_threshold and warn_low:
			info = winki.alert.AlertWarning()
			primary = _('Movie quality will be unsatisfactory. Do you want to continue?')
			secondary = _('The computed video bitrate is too low (%d). Most probably you will be disappointed with the video quality. You can go back and select bigger chunk size or smaller movie chunk to be encoded.') % vbitrate
			info.show(primary, secondary, _('Continue anyway'))
			if info.state != 'ok':
				if self.program:
					self.program.stop()
				self.program = None
				self.vbitrate = -1
				return False

		elif upper_threshold > 0 and vbitrate > upper_threshold and warn_high:
			info = winki.alert.AlertWarning()
			primary = _('Video bitrate too high. Do you want to continue?')
			secondary = _('The computed video bitrate is too high (%d). Most probably you will not be able to notice the quality improvement of this. You can go back and select smaller chunk size or bigger movie chunk to be encoded. This is NOT an error. Just a warning for possible disk space wasting.') % vbitrate
			info.show(primary, secondary, _('Continue anyway'))
			if info.state != 'ok':
				if self.program:
					self.program.stop()
				self.program = None
				self.vbitrate = -1
				return False

		if self.vbitrate > 16000:
			self.vbitrate = 16000

		return True

	def set_all_visible_labels(self, img_descr):
		temp = self.chunk_nr
		for chunk_nr in range(self.app.project.get_number_chunks()):
			self.chunk_nr = chunk_nr
			for label in self.labels:
				self.set_image(label, img_descr)
		self.chunk_nr = temp

		label_estim = self.xml.get_widget('label_estimated_time')
		label_estim.set_text('')

		self.update_ui()
		return

	def init_images(self):
		fname = self.get_image_file_name('empty')
		for i in range(50):
			widget_name = 'image%d' % (i + 1)
			widget = self.xml.get_widget(widget_name)
			widget.set_from_file(fname)

		return


	def set_image(self, active_label, img_descr):
		row = -1

		for label_id in self.labels:
			if label_id == active_label:
				row = self.labels[label_id]
				break
		if row < 0:
			print 'label not found'
			return

		fname = self.get_image_file_name(img_descr)
		if not fname:
			print 'image file for %s not found' % img_descr
			return

		img_nr = self.chunk_nr * 10 + row 
		if img_nr < 1 or img_nr > 50:
			return

		widget_name = 'image%d' % (img_nr)
		widget = self.xml.get_widget(widget_name)
		widget.set_from_file(fname)
		return

	def get_image_file_name(self, img_descr):
		result = None

		img_1of3 = winki.lib.build_path([winki.lib.winki_path, 'glade', '1of3.png'])
		img_2of3 = winki.lib.build_path([winki.lib.winki_path, 'glade', '2of3.png'])
		img_3of3 = winki.lib.build_path([winki.lib.winki_path, 'glade', '3of3.png'])
		img_1of2 = winki.lib.build_path([winki.lib.winki_path, 'glade', '1of2.png'])
		img_2of2 = winki.lib.build_path([winki.lib.winki_path, 'glade', '2of2.png'])
		img_1of1 = winki.lib.build_path([winki.lib.winki_path, 'glade', '1of1.png'])
		img_empty = winki.lib.build_path([winki.lib.winki_path, 'glade', 'empty.png'])
		img_ready = winki.lib.build_path([winki.lib.winki_path, 'glade', 'ready.png'])
		img_frame = winki.lib.build_path([winki.lib.winki_path, 'glade', 'frame.png'])
		mymap = {'1of3' : img_1of3,
			'2of3' : img_2of3,
			'3of3' : img_3of3,
			'1of2' : img_1of2,
			'2of2' : img_2of2,
			'1of1' : img_1of1,
			'empty' : img_empty,
			'ready' : img_ready,
			'frame' : img_frame}

		for item in mymap:
			if item == img_descr:
				result = mymap[item]
				break

		return result

	def update_ui(self):
		for number in range(10):
			label_name = 'label_misc%d' % (number + 1)
			label = self.xml.get_widget(label_name)
			label.set_text('')

		for label_id in self.labels:
			label_name = 'label_misc%d' % self.labels[label_id]
			label = self.xml.get_widget(label_name)
			ftext = '<span weight="light">%s</span>' % label_id
			label.set_markup(ftext)
		return

	def enable_ui(self, text, active_label, stream_idx, nr_streams):
		label_current = self.xml.get_widget('label_current')
		label_current.set_text(text)
		self.update_chunk_label()

		for label_id in self.labels:
			label_name = 'label_misc%d' % self.labels[label_id]
			label = self.xml.get_widget(label_name)

			is_current = (label_id == active_label)
			text = label.get_text()
			if is_current:
				ftext = ('<span weight="bold">%s</span>' % text)
			else:
				ftext = ('<span weight="light">%s</span>' % text)
			label.set_markup(ftext)

		if stream_idx < 0 or stream_idx > 2:
			print 'stream_idx %d?' % stream_idx
			return

		if nr_streams < 1 or nr_streams > 3:
			print 'nr_streams %d?' % nr_streams
			return

		if stream_idx + 1 > nr_streams:
			print 'requesting file: "%dof%d.png" (%s)' % (stream_idx + 1, nr_streams, text)
			return

		descr = '%dof%d' % (stream_idx + 1, nr_streams)
		self.set_image(active_label, descr)
		return

	def on_dump_audio(self, stream_number):
		label_estim = self.xml.get_widget('label_estimated_time')
		label_estim.set_text(_('no estimation possible'))

		nr_streams = self.app.project.get_audio_count()
		text = _('Dumping audio stream %d from %d') % (stream_number + 1, nr_streams)
		self.enable_ui(text, _('Dumping audio'), stream_number, nr_streams)

		self.refresh_ratio = 10

		self.len_seconds = self.app.project.get_chunk_lenghts()

		return [self.dump_audio_callback, 20]

	def on_normalize_audio(self, stream_number):
		text = _('Normalizing audio stream %d from %d') % (stream_number + 1, self.app.project.get_audio_count())
		nr_streams = self.app.project.get_audio_count()
		self.enable_ui(text, _('Normalizing audio'), stream_number, nr_streams)

		self.refresh_ratio = 10

		return [self.normalize_callback, 20]

	def on_compress_audio(self, stream_number):
		label_estim = self.xml.get_widget('label_estimated_time')
		label_estim.set_text('')

		nr_streams = self.app.project.get_audio_count()
		text = _('Compressing audio stream %d from %d') % (stream_number + 1, nr_streams)
		self.enable_ui(text, _('Compressing audio'), stream_number, nr_streams)

		self.refresh_ratio = 10

		compress_program = self.app.project.get_audio_compressor_name()
		compressor = os.path.basename(compress_program)
		callback = None
		buf_len = 0

		if compressor == 'oggenc':
			callback = self.oggenc_compress_callback
			buf_len = 20
		elif compressor == 'lame':
			callback = self.lame_compress_callback
			buf_len = 5
		else:
			winki.lib.debug(PageRip.on_compress_audio(), 'unknown compressor %s' % compressor, winki.lib.error)
			assert(0)

		return [callback, buf_len]

	def on_dump_subtitles(self, stream_number):
		nr_streams = self.app.project.get_sub_count()
		text = _('Dumping subtitle %d from %d') % (stream_number + 1, nr_streams)
		self.enable_ui(text, _('Dumping subtitles'), stream_number, nr_streams)

		self.refresh_ratio = 10

		return [self.mencoder_callback, 64]

	def on_dump_video(self, stream_number):
		text = _('Dumping video stream')
		self.enable_ui(text, _('Dumping video'), stream_number, 1)

		self.refresh_ratio = 10

		return [self.mencoder_callback, 64]

	def on_create_index(self, cmdline, stream_number):
		result = [None, None, None]

		encoding = self.app.project.get_encoding()
		if self.vbitrate > -1 and encoding == 'multi pass':
			cmdline = (cmdline % self.vbitrate)

		elif self.vbitrate > -1 and encoding == 'svcd':
			cmdline = (cmdline % self.vbitrate)

		elif encoding == 'vcd':
			pass

		else:
			winki.lib.debug('PageRip.on_create_index()', 'no vbitrate', winki.lib.error)
			return result

		text = _('Creating video index')
		self.enable_ui(text, _('Creating video index'), stream_number, 1)

		self.refresh_ratio = 30

		return [cmdline, self.mencoder_callback, 64]

	def on_encode_video(self, cmdline, stream_number):
		result = [None, None, None]

		text = ''
		encoding = self.app.project.get_encoding()

		if self.vbitrate > -1 and encoding == 'multi pass':
			cmdline = (cmdline % self.vbitrate)
			text = _('Encoding video stream')
			self.enable_ui(text, _('Encoding video'), stream_number, 1)

		elif self.vbitrate > -1 and encoding == 'svcd':
			cmdline = (cmdline % self.vbitrate)
			text = _('Encoding audio and video streams')
			self.enable_ui(text, _('Encoding'), stream_number, 1)

		elif encoding == 'vcd':
			text = _('Encoding audio and video streams')
			self.enable_ui(text, _('Encoding'), stream_number, 1)

		else:
			winki.lib.debug('PageRip.on_encode_video()', 'no vbitrate', winki.lib.error)
			return result

		self.refresh_ratio = 30

		return [cmdline, self.mencoder_callback, 64]
	
	def on_merge_streams(self, stream_number):
		label_estim = self.xml.get_widget('label_estimated_time')
		label_estim.set_text(_('no estimation possible'))

		text = _('Merging audio and video streams')
		self.enable_ui(text, _('Merging streams'), stream_number, 1)

		self.refresh_ratio = 10

		return [self.merge_callback, 20]

	def on_copy_stream(self, stream_number):
		text = _('Copying audio and video streams')
		self.enable_ui(text, _('Copying streams'), stream_number, 1)

		self.refresh_ratio = 30

		return [self.mencoder_callback, 64]

	def on_fix_stream(self, stream_number):
		text = _('Fixing video index')
		self.enable_ui(text, _('Fixing video index'), stream_number, 1)

		self.refresh_ratio = 30

		return [self.mencoder_callback, 64]

	def on_mplex_stream(self, stream_number):
		text = _('Remultipexing streams')
		self.enable_ui(text, _('Remultipexing streams'), stream_number, 1)

		self.refresh_ratio = 30

		return [self.mplex_callback, 64]


	def on_build_image(self, stream_number):
		self.no_estimation_possible()

		text = _('Building VCD image')
		self.enable_ui(text, _('Creating image'), stream_number, 1)

		self.refresh_ratio = 30

		return [self.dummy_callback, 64]


	def step_ready(self):
		self.program = None
		return

	def mencoder_callback(self, text):
		self.current_iteration += 1
		if (self.current_iteration % self.refresh_ratio) != 0:
			return

		self.line = text
		chunks = string.split(text)
		len_chunks = len(chunks)

		perc_idx = -1
		time_idx = -1
		for chunk_nr in range(len_chunks):
			if string.find(chunks[chunk_nr], '%') > -1:
				perc_idx = chunk_nr
				continue

			if chunks[chunk_nr] == 'Trem:':
				time_idx = chunk_nr + 1
				break

		if perc_idx < 0:
			return

		if perc_idx >= len_chunks or time_idx >= len_chunks:
			return

		self.update_status(chunks[perc_idx], chunks[time_idx])
		self.update_elapsed_time()
		return

	def mplex_callback(self, text):
		percent = self.mplex_parser(text, 'Demuxing')
		if percent > 0:
			final = (100.0 * percent) / 400.0
			self.update_progress(final)
			self.update_elapsed_time()

			text = _('Demultiplexing streams')
			self.enable_ui(text, _('Remultipexing streams'), 0, 1)
			return

		percent = self.mplex_parser(text, 'Scanning video stream:')
		if percent > 0:
			final = (100.0 * percent) / 400.0 + 25.0
			self.update_progress(final)

			text = _('Scanning video stream')
			self.enable_ui(text, _('Remultipexing streams'), 0, 1)
			self.update_elapsed_time()
			return

		percent = self.mplex_parser(text, 'Scanning audio stream:')
		if percent > 0:
			final = (100.0 * percent) / 400.0 + 50.0
			self.update_progress(final)
			self.update_elapsed_time()

			text = _('Scanning audio stream')
			self.enable_ui(text, _('Remultipexing streams'), 0, 1)
			return

		percent = self.mplex_parser(text, 'Multiplexing:')
		if percent > 0:
			final = (100.0 * percent) / 400.0 + 75.0
			self.update_progress(final)
			self.update_elapsed_time()

			text = _('Multiplexing streams')
			self.enable_ui(text, _('Remultipexing streams'), 0, 1)
			return

		return

	def mplex_parser(self, line, keyword):
		pos1 = string.find(line, keyword)
		if pos1 > -1:
			pos2 = string.find(line, '%')
			if pos2 < pos1 or pos2 < 0:
				print 'unexpected pos2'
				return

			percent = line[pos1 + len(keyword) : pos2]
			number = winki.lib.to_int(percent)
			if number < 0:
				print 'not a number'
				return

			return number

		return -1

	def dump_audio_callback(self, text):
		self.current_iteration += 1
		if (self.current_iteration % self.refresh_ratio) != 0:
			return

		items = string.split(text)
		if not items or len(items) < 4:
			return

		if string.find(items[2], '%') < 0 and string.find(items[3], '%') < 0:
			return

		seconds = self.to_seconds(items[1])
		if seconds < 0 or seconds > self.len_seconds[self.chunk_nr]:
			txt = '%d of %d seconds requested' % (seconds, self.len_seconds[self.chunk_nr])
			winki.lib.debug('PageRip.dump_audio_callback()', txt, winki.lib.error)
			return

		percent = (seconds * 100) / self.len_seconds[self.chunk_nr]
		self.update_progress(percent)
		self.update_elapsed_time()
		return

	def normalize_callback(self, text):
		self.current_iteration += 1
		if (self.current_iteration % self.refresh_ratio) != 0:
			return

		items = string.split(text)
		if len(items) != 10:
			return

		percent_string = items[1]
		len_percent = len(percent_string)

		if percent_string[len_percent - 1] != '%':
			return

		percent = winki.lib.to_int(percent_string[ : len_percent - 1])
		if percent < 0:
			return

		self.update_progress(percent)
		self.update_elapsed_time()
		return

	def oggenc_compress_callback(self, text):
		self.current_iteration += 1
		if (self.current_iteration % self.refresh_ratio) != 0:
			return

		items = string.split(text)
		if len(items) != 6:
			return

		percent_string = items[1]
		len_percent = len(percent_string)

		if percent_string[len_percent - 2 : len_percent - 1] != '%':
			return

		percent = winki.lib.to_float(percent_string[ : len_percent - 2])
		if percent < 0.0:
			return

		self.update_progress(percent)

		label_estim = self.xml.get_widget('label_estimated_time')
		label_estim.set_text(items[3])

		self.update_elapsed_time()
		return

	def lame_compress_callback(self, text):
		# lame tends to print a lot of garbage on the screen so we have to
		# parse every single line in order to be able to update safely

		#self.current_iteration += 1
		#if (self.current_iteration % self.refresh_ratio) != 0:
		#	return

		items = string.split(text, '|')
		if len(items) != 5:
			return

		first = string.split(items[0])
		len_first = len(first)
		if len_first != 2 and len_first != 3:
			return

		percent_string = first[len_first - 1]
		len_percent = len(percent_string)

		if percent_string[len_percent - 2 : len_percent - 1] != '%':
			return

		start = 0
		if percent_string[0] == '(':
			start = 1

		percent = winki.lib.to_float(percent_string[start : len_percent - 2])
		if percent < 0.0:
			return

		self.update_progress(percent)

		label_estim = self.xml.get_widget('label_estimated_time')
		label_estim.set_text(string.strip(items[4]))

		self.update_elapsed_time()
		return

	def merge_callback(self, text):
		self.current_iteration += 1
		if (self.current_iteration % self.refresh_ratio) != 0:
			return

		items = string.split(text)
		if len(items) != 4:
			return

		percent_string = items[3]
		len_percent = len(percent_string)

		if percent_string[len_percent - 2 : len_percent - 1] != '%':
			return

		if percent_string[0] != '(':
			return

		percent = winki.lib.to_float(percent_string[1 : len_percent - 2])
		if percent < 0.0:
			return

		self.update_progress(percent)
		self.update_elapsed_time()
		return

	def dummy_callback(self, text):
		return

	def to_seconds(self, text):
		dot_pos = string.find(text, '.')
		if dot_pos < 0:
			return -1

		text = text[ : dot_pos]
		items = string.split(text, ':')

		if len(items) == 1:
			return winki.lib.to_int(items[0])

		if len(items) == 2:
			return winki.lib.to_int(items[0]) * 60 + winki.lib.to_int(items[1])
		elif len(items) == 3:
			return winki.lib.to_int(items[0]) * 3600 + winki.lib.to_int(items[1]) * 60 + winki.lib.to_int(items[2])
			
		return -1

	def update_progress(self, percent):
		text = '%d%%' % percent
		winki.lib.update_progress(self.progress, text, percent / 100.0)
		return

	def update_chunk_label(self):
		label_chunk = self.xml.get_widget('label_chunk')
		text = _('Chunk %d from %d') % ((self.chunk_nr + 1), self.app.project.get_number_chunks())
		if self.chunk_nr < 0:
			label_chunk.set_text('')
		else:
			label_chunk.set_text(text)

		return

	def update_status(self, percent, estimated_time):
		prog_text = self.get_progress(percent)
		if prog_text and winki.lib.to_int(prog_text) > -1 and self.progress:
			text = '%s%%' % prog_text
			number = float((winki.lib.to_int(prog_text) / 100.0))
			winki.lib.update_progress(self.progress, text, number)

		label_estim = self.xml.get_widget('label_estimated_time')
		label_estim.set_text(estimated_time)
		return

	def get_progress(self, percent):
		end = string.find(percent, '%')
		if end < 0:
			return None

		start = string.find(percent, '(')
		if start > -1 and (start + 1) >= end:
			return None

		return percent[start + 1 : end]

	def init_timer(self):
		self.start_time = time.time()
		return

	def format_time(self, seconds):
		hr = int(seconds / 3600)
		m = int((seconds - hr * 3600) / 60)
		s = int(seconds - hr * 3600 - m * 60)

		formatted_time = ''
		if hr > 9:
			formatted_time += ('%d:' % hr)
		else:
			formatted_time += ('0%d:' % hr)
		if m > 9:
			formatted_time += ('%d:' % m)
		else:
			formatted_time += ('0%d:' % m)
		if s > 9:
			formatted_time += ('%d' % s)
		else:
			formatted_time += ('0%d' % s)

		return formatted_time

	def update_elapsed_time(self):
		elapsed = int(time.time() - self.start_time)
		text = self.format_time(elapsed)

		label_elaps = self.xml.get_widget('label_elapsed_time')
		label_elaps.set_text(text)
		return

	def no_estimation_possible(self):
		self.timer = threading.Timer(1.0, self.progress_pulse)
		self.timer.start()
		return
		
	def progress_pulse(self):
		if not self.progress:
			print 'no self.progress'
			return

		self.progress.pulse()
		
		if not self.timer:
			print 'no self.timer'
			return

		del self.timer
		self.timer = threading.Timer(1.0, self.progress_pulse)
		self.timer.start()
		return
