# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module winki.lib...'
	import winki.lib

	print 'Importing module winki.base_page...'
	import winki.base_page

except:
	error_quit(0, 1)

class PageSize(winki.base_page.BasePage):
	def __init__(self, xml, app):
		winki.base_page.BasePage.__init__(self, xml, app)
		self.lock_update = False
		self.initialized = False
		return
		
	def check_widgets(self):
		widgets = ['combo_chunk_size', 'combo_end1', 'combo_end2',
		'combo_end3', 'combo_end4', 'combo_end5', 'combo_start1',
		'combo_start2', 'combo_start3', 'combo_start4', 'combo_start5',
		'label_chunk_size', 'label_end1', 'label_end2', 'label_end3',
		'label_end4', 'label_end5', 'label_nr_chunks', 'label_start1',
		'label_start2', 'label_start3', 'label_start4', 'label_start5',
		'label_time1', 'label_time2', 'label_time3', 'label_time4',
		'label_time5', 'spinbutton_nr_chunks', 'label_mb',
		'label_est1', 'label_est2', 'label_est3', 'label_est4',
		'label_est5']

		winki.lib.check_widgets(self.xml, widgets)
		return


	def on_prepare(self):
		source = self.app.project.get_source()

		if source == 'file' or source == 'vcd':
			self.fill_combo_size()
			self.layout_file()

		elif source == 'dvd':
			if not self.initialized:
				current_title = self.app.project.get_current_title()
				chunks = self.app.project.estimate_nr_chunks(current_title)
				self.app.project.set_number_chunks(chunks)
				spin_chunks = self.xml.get_widget('spinbutton_nr_chunks')
				spin_chunks.set_text('%d' % chunks)
				self.layout_dvd()

		else:
			assert(0)

		return True

	def on_next(self):
		return self.update_project()

	def on_prev(self):
		return self.update_project()

	def can_exit(self):
		return True

	def show_help(self):
		return winki.base_page.BasePage.show_help(self, 'size.html')

	def nr_chunks_changed(self):
		if self.lock_update:
			return

		if self.app.project.get_source() == 'file':
			return

		spin_chunks = self.xml.get_widget('spinbutton_nr_chunks')
		number = winki.lib.to_int(spin_chunks.get_text())
		if number == -1:
			return

		# this is an idiot hack: the initial value of the spinbutton
		# will be set after on_prepare() has been called, so the
		# value which is set in on_prepare() will be overwritten with
		# the default value of this control: "1" (set trough glade)
		if not self.initialized and number == 1:
			self.initialized = True
			spin_chunks.set_text(str(self.app.project.get_number_chunks()))
			return

		self.app.project.set_number_chunks(number)

		update_size = False
		if self.app.project.get_encoding() == 'vcd':
			update_size = True
		self.update_from_project(update_size)

		return

	def on_chapter_changed(self):
		spin_chunks = self.xml.get_widget('spinbutton_nr_chunks')
		number = winki.lib.to_int(spin_chunks.get_text())
		if number > 0: 
			self.fill_times_from_combo(number)
		return

	def layout_file(self):
		label_chunks = self.xml.get_widget('label_nr_chunks')
		spin_chunks = self.xml.get_widget('spinbutton_nr_chunks')
		combo_size = self.xml.get_widget('combo_chunk_size')
		label_chunk_size = self.xml.get_widget('label_chunk_size')

		label_chunks.set_sensitive(False)
		spin_chunks.set_sensitive(False)

		label_chunk_size.set_text(_('Desired size'))
		combo_size.set_sensitive(True)
		combo_size.entry.grab_focus()

		self.update_from_project()
		self.chunk_layout(-1)

		return


	def layout_dvd(self):
		label_chunks = self.xml.get_widget('label_nr_chunks')
		spin_chunks = self.xml.get_widget('spinbutton_nr_chunks')
		combo_size = self.xml.get_widget('combo_chunk_size')
		label_chunk_size = self.xml.get_widget('label_chunk_size')

		label_chunks.show()
		spin_chunks.show()

		# todo: calc right table
		if self.app.project.get_encoding() == 'copy':
			combo_size.set_sensitive(False)
			label_chunk_size.set_sensitive(False)
		else:
			combo_size.set_sensitive(True)
			label_chunk_size.set_sensitive(True)

		label_chunks.set_sensitive(True)
		spin_chunks.set_sensitive(True)

		label_chunk_size.set_text(_('Chunk size'))

		self.update_from_project()

		spin_chunks.grab_focus()
		return

	def on_chunk_size_changed(self):
		source = self.app.project.get_source()
		if self.lock_update or source == 'file':
			return

		widget = self.xml.get_widget('combo_chunk_size')
		text = widget.entry.get_text()
		self.app.project.set_chunk_size(text)

		current_title = self.app.project.get_current_title()
		nr_chunks = self.app.project.estimate_nr_chunks(current_title)
		self.app.project.set_number_chunks(nr_chunks)

		self.update_from_project(False)
		return

	def update_from_project(self, size = True):
		self.lock_update = True

		spin_chunks = self.xml.get_widget('spinbutton_nr_chunks')

		nr_chunks = self.app.project.get_number_chunks()
		nr_chunks = self.max_number_rows(nr_chunks)

		parts = self.app.project.split_chapters(nr_chunks)
		if not parts:
			self.lock_update = False
			return

		spin_chunks.set_text('%d' % nr_chunks)

		self.chunk_layout(nr_chunks)
		self.fill_chapters(nr_chunks)
		self.fill_times_from_project(nr_chunks)
		if size:
			self.fill_combo_size()

		self.lock_update = False
		return

	def fill_combo_size(self):
		if self.app.project.get_encoding() == 'copy':
			return
			
		strings = self.app.project.get_str_chunk_sizes()

		combo_size = self.xml.get_widget('combo_chunk_size')
		text = combo_size.entry.get_text()
		default = -1
		sz = self.app.project.get_default_size()
		if not text or not len(text):
			if sz > -1:
				if sz == 650:
					default = 0
				elif sz == 700:
					default = 1
				elif sz == 800:
					default = 2
				else:
					default = 3
			else:
				default = 3
		else:
			index = winki.lib.get_index(strings, text)
			if index > -1:
				default = index
			else:
				default = 3

		winki.lib.fill_combo(combo_size, strings, default)
		if default == 3:
			combo_size.entry.set_text('%d' % sz)
		return

	def fill_chapters(self, rows):
		current_title = self.app.project.get_current_title()
		strings = self.app.project.get_chapters(current_title)
		if not strings:
			rows = 0
		elif strings and len(strings) < rows:
			rows = len(strings)

		parts = self.app.project.split_chapters(rows)

		self.chunk_layout(len(parts))
		for i in range(5):
			name_start = 'combo_start%d' % (i + 1)
			combo_start = self.xml.get_widget(name_start)

			name_end = 'combo_end%d' % (i + 1)
			combo_end = self.xml.get_widget(name_end)
			if i < len(parts):
				winki.lib.fill_combo(combo_start, strings, parts[i][0])
				winki.lib.fill_combo(combo_end, strings, parts[i][1])
			else:
				winki.lib.fill_combo(combo_start, None)
				winki.lib.fill_combo(combo_end, None)

		return

	def fill_times_from_project(self, rows):
		current_title = self.app.project.get_current_title()
		times = self.app.project.get_chapter_times(current_title)
		parts = self.app.project.split_chapters(rows)

		for i in range(5):
			name = 'label_time%d' % (i + 1)
			label = self.xml.get_widget(name)
			if i < len(parts):
				text = winki.lib.format_time(times, parts[i][0], parts[i][1])
				label.set_text(text)
			else:
				label.set_text('')

			name = 'label_est%d' % (i + 1)
			label = self.xml.get_widget(name)
			label.set_text('')

		encoding = self.app.project.get_encoding()
		if  encoding != 'vcd':
			return

		bps = self.app.project.get_vcd_bytes_per_second()
		for i in range(5):
			name = 'label_est%d' % (i + 1)
			label = self.xml.get_widget(name)
			if i < len(parts):
				text = winki.lib.format_size(times, parts[i][0], parts[i][1], bps)
				label.set_text(text)
			else:
				label.set_text('')

		return

	def fill_times_from_combo(self, rows):
		current_title = self.app.project.get_current_title()
		chapter_list = self.app.project.get_chapters(current_title)
		times = self.app.project.get_chapter_times(current_title)
		encoding = self.app.project.get_encoding

		bps = -1
		if encoding == 'vcd':
		    bps = self.app.project.get_vcd_bytes_per_second()

		for i in range(5):
			label_time_name = 'label_time%d' % (i + 1)
			label_est_name = 'label_est%d' % (i + 1)
			label_time = self.xml.get_widget(label_time_name)			
			label_est = self.xml.get_widget(label_est_name)

			if i < rows:
				name_start = 'combo_start%d' % (i + 1)
				name_end = 'combo_end%d' % (i + 1)
				combo_start = self.xml.get_widget(name_start)
				combo_end = self.xml.get_widget(name_end)

				start_chapter = combo_start.entry.get_text()
				end_chapter = combo_end.entry.get_text()

				if not len(start_chapter) or not len(end_chapter):
					continue

				start = winki.lib.get_index(chapter_list, start_chapter)
				end = winki.lib.get_index(chapter_list, end_chapter)
				if start < 0 or end < 0:
					winki.lib.debug('PageSize.fill_times_from_combo()', 'internal error')
					label_time.set_text('')
					label_est.set_text('')
					continue

				time = winki.lib.format_time(times, start, end)
				label_time.set_text(time)

				est = winki.lib.format_size(times, start, end, bps)
				label_est.set_text(est)
			else:
				label_time.set_text('')
				label_est.set_text('')

		return

	def chunk_layout(self, number):
		enable = False
		for i in range(5):
			if i < number:
				enable = True
			else:
				enable = False
			self.enable_row(i + 1, enable)

		return

	def enable_row(self, row_nr, enable):
		if row_nr < 1 or row_nr > 5:
			return

		label_start_name = 'label_start%d' % row_nr
		label_end_name = 'label_end%d' % row_nr
		label_time_name = 'label_time%d' % row_nr
		label_est_name = 'label_est%d' % row_nr
		combo_start_name = 'combo_start%d' % row_nr
		combo_end_name = 'combo_end%d' % row_nr

		label_start = self.xml.get_widget(label_start_name)
		label_end = self.xml.get_widget(label_end_name)
		label_time = self.xml.get_widget(label_time_name)
		label_est = self.xml.get_widget(label_est_name)
		combo_start = self.xml.get_widget(combo_start_name)
		combo_end = self.xml.get_widget(combo_end_name)

		label_start.set_sensitive(enable)
		label_end.set_sensitive(enable)
		label_time.set_sensitive(enable)
		if not enable:
			label_time.set_text('')
			label_est.set_text('')
		combo_start.set_sensitive(enable)
		combo_end.set_sensitive(enable)
		return

	def max_number_rows(self, rows):
		current_title = self.app.project.get_current_title()
		if current_title == -1:
			return 0

		strings = self.app.project.get_chapters(current_title)
		if not strings or len(strings) == 0:
			return 0

		if rows > len(strings):
			rows = len(strings)

		return rows
		
	def update_project(self):
		widget = self.xml.get_widget('combo_chunk_size')
		text = widget.entry.get_text()

		result = self.submit_nr_chunks()
		result &= self.app.project.set_chunk_size(text)
		if not result:
			winki.lib.alert(_('Wrong chunk size'), _('Please enter the chunk size in MB'), 'error')
			combo_size = self.xml.get_widget('combo_chunk_size')
			combo_size.grab_focus()
			return False

		if self.app.project.get_source() == 'dvd':
			result &= self.submit_chapters()

		return result

	def submit_nr_chunks(self):
		if self.app.project.get_source() == 'dvd':
			spin = self.xml.get_widget('spinbutton_nr_chunks')
			nr_chunks = winki.lib.to_int(spin.get_text())
			if nr_chunks < 0:
				return False

			self.app.project.set_number_chunks(nr_chunks)

		return True

	def submit_chapters(self):
		nr_chunks = self.app.project.get_number_chunks()
		current_title = self.app.project.get_current_title()
		strings = self.app.project.get_chapters(current_title)
		chapters = []

		for i in range(nr_chunks):
			name_start = 'combo_start%d' % (i + 1)
			name_end = 'combo_end%d' % (i + 1)

			combo_start = self.xml.get_widget(name_start)
			combo_end = self.xml.get_widget(name_end)

			start_text = combo_start.entry.get_text()
			end_text = combo_end.entry.get_text()

			start = winki.lib.get_index(strings, start_text)
			end = winki.lib.get_index(strings, end_text)

			if start < 0 or end < 0:
				winki.lib.debug('PageSize.submit_chapters()', 'invalid chapter index', winki.lib.error)
				continue

			item = [start, end]
			chapters.append(item)

		if not (len(chapters) == nr_chunks):
			winki.lib.debug('PageSize.submit_chapters()', 'cannot find chapters for every chunk requested', winki.lib.error)
			return False
			
		self.app.project.set_chunks(chapters)

		return True


