# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module winki.lib...'
	import winki.lib

	print 'Importing module winki.base_page...'
	import winki.base_page

	print 'Importing module winki.base_page...'
	import winki.base_page

except:
	error_quit(0, 1)

class PageSource(winki.base_page.BasePage):
	def __init__(self, xml, app):
		winki.base_page.BasePage.__init__(self, xml, app)
		return
		
	def check_widgets(self):
		widgets = ['button_input_file', 'entry_input_file',
		'radio_dvd', 'radio_file', 'radio_vcd']

		winki.lib.check_widgets(self.xml, widgets)
		return
		

	def check_tools(self):
		has_mencoder = self.app.has_program('mencoder')
		has_mplayer = self.app.has_program('mplayer')
		has_lsdvd = self.app.has_program('lsdvd')

		radio_dvd = self.xml.get_widget('radio_dvd')
		radio_vcd = self.xml.get_widget('radio_vcd')
		radio_file = self.xml.get_widget('radio_file')
		entry = self.xml.get_widget('entry_input_file')
		button = self.xml.get_widget('button_input_file')

		if not has_mencoder:
			radio_dvd.set_sensitive(False)
			radio_vcd.set_sensitive(False)
			radio_file.set_sensitive(False)
			winki.lib.debug('PageSource.check_tools()', 'no mencoder', winki.lib.error)
			winki.lib.alert(_('No mencoder installation found'), _('Winki the Ripper cannot work without mencoder. Please install mencoder and try again.'), 'error')
			return

		if not has_mplayer:
			radio_dvd.set_sensitive(False)
			radio_vcd.set_sensitive(False)
			radio_file.set_sensitive(False)
			winki.lib.debug('PageSource.check_tools()', 'no mplayer', winki.lib.error)
			winki.lib.alert(_('No mplayer installation found'), _('Winki the Ripper cannot work without mplayer. Please install mplayer and try again.'), 'error')
			return

		if not has_lsdvd:
			radio_dvd.set_sensitive(False)
			winki.lib.debug('PageSource.check_tools()', 'no lsdvd')

		return

	def on_prepare(self):
		radio_dvd = self.xml.get_widget('radio_dvd')
		radio_vcd = self.xml.get_widget('radio_vcd')
		radio_file = self.xml.get_widget('radio_file')
		entry = self.xml.get_widget('entry_input_file')
		button = self.xml.get_widget('button_input_file')

		self.check_tools()

		if not self.app.project.get_source():
			src = self.app.project.get_default_source()
			if src == 'dvd':
				radio_dvd.set_active(True)
				radio_dvd.grab_focus()
			elif src == 'vcd':
				radio_vcd.set_active(True)
				radio_vcd.grab_focus()
			elif src == 'file':
				radio_file.set_active(True)
				radio_file.grab_focus()
			else:
				assert(0)

			self.app.project.set_source(src)
			self.enable_ui()

		return True

	def on_next(self):
		return self.update_project()

	def on_prev(self):
		return self.update_project()

	def can_exit(self):
		return True

	def update_project(self):
		radio_dvd = self.xml.get_widget('radio_dvd')
		radio_vcd = self.xml.get_widget('radio_vcd')
		radio_file = self.xml.get_widget('radio_file')
		entry = self.xml.get_widget('entry_input_file')

		if winki.lib.is_radio_active(radio_dvd) and radio_dvd.get_property('sensitive'):
			self.app.project.set_source('dvd')
			winki.lib.debug('PageSource.update_project()', 'dvd source selected')

		elif winki.lib.is_radio_active(radio_vcd) and radio_vcd.get_property('sensitive'):
			self.app.project.set_source('vcd')
			winki.lib.debug('PageSource.update_project()', 'vcd source selected')

		elif winki.lib.is_radio_active(radio_file) and radio_file.get_property('sensitive'):
			if not self.check_filename():
				winki.lib.debug('PageSource.update_project()', 'invalid input file selected', winki.lib.error)
				entry.grab_focus()
				return False

			path = winki.lib.normalize_path(entry.get_text())
			self.app.project.set_source_filename(path)
			self.app.project.set_source('file')
			self.app.project.set_number_chunks(1)
			winki.lib.debug('PageSource.update_project()', 'file source selected')

		else:
			return False

		return True

	def show_help(self):
		return winki.base_page.BasePage.show_help(self, 'source.html')

	def enable_ui(self):
		entry = self.xml.get_widget('entry_input_file')
		button = self.xml.get_widget('button_input_file')

		source = self.app.project.get_source()

		if source == 'dvd' or source == 'vcd':
			entry.set_sensitive(False)
			button.set_sensitive(False)

		elif source == 'file':
			entry.set_sensitive(True)
			button.set_sensitive(True)

		else:
			assert(0)

		return

	def set_file_name(self, file_name):
		entry = self.xml.get_widget('entry_input_file')
		entry.set_text(file_name)
		return

	def get_file_name(self):
		entry = self.xml.get_widget('entry_input_file')
		return entry.get_text()

	def check_filename(self):
		entry = self.xml.get_widget('entry_input_file')
		path = winki.lib.normalize_path(entry.get_text())

		primary = _('Could not access file')
		if not path or not len(path):
			winki.lib.debug('PageSource.check_filename()', 'no file selected')
			secondary = _('You have to select a multimedia file to be used as a source. Please type the full path to the file into the edit field or use the "Open" button to locate it.')
			winki.lib.alert(primary, secondary, 'error')
			return False

		if not winki.lib.is_file(path):
			winki.lib.debug('PageSource.check_filename()', 'not a file')
			secondary = _('%s is not a file. You have to select a file which contains at least one video stream.') % path
			winki.lib.alert(primary, secondary, 'error')
			return False

		if not winki.lib.is_readable(path):
			winki.lib.debug('PageSource.check_filename()', 'file not readable by user')
			secondary = _('%s is not readable by you. You should select a file for which you have sufficient access permissions.') % path
			winki.lib.alert(primary, secondary, 'error')
			return False

		return True
