# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module winki.lib...'
	import winki.lib

	print 'Importing module winki.base_page...'
	import winki.base_page

except:
	error_quit(0, 1)

class PageSubtitles(winki.base_page.BasePage):
	def __init__(self, xml, app):
		winki.base_page.BasePage.__init__(self, xml, app)
		self.no_subtitles = [_('No subtitles in this chapter')]
		self.initialized = False
		return
		
	def check_widgets(self):
		widgets = ['button_sub1', 'combo_sub1', 'combo_sub2',
		'combo_sub3', 'entry_sub1', 'label_sub1', 'label_sub2',
		'label_sub3', 'checkbutton_mix_subitles',
		'checkbutton_consider_external']

		winki.lib.check_widgets(self.xml, widgets)
		return

	def on_prepare(self):
		combo_sub1 = self.xml.get_widget('combo_sub1')
		combo_sub2 = self.xml.get_widget('combo_sub2')
		combo_sub3 = self.xml.get_widget('combo_sub3')
		label_sub1 = self.xml.get_widget('label_sub1')
		label_sub2 = self.xml.get_widget('label_sub2')
		label_sub3 = self.xml.get_widget('label_sub3')

		current = self.app.project.get_current_title()

		subtitles = self.app.project.get_subtitles(current)
		default_sub = []
		if not self.initialized:
			default_sub = self.app.project.get_preferred_subtitles()
			self.initialized = True
		else:
			default_sub = self.app.project.get_current_subtitles()

		sub1 = 0
		if len(default_sub) > 0:
			sub1 = default_sub[0]
		sub2 = 0
		if len(default_sub) > 1:
			sub2 = default_sub[1]
		sub3 = 0
		if len(default_sub) > 2:
			sub3 = default_sub[2]

		winki.lib.fill_combo(combo_sub1, subtitles, sub1, self.no_subtitles)
		winki.lib.fill_combo(combo_sub2, subtitles, sub2, self.no_subtitles)
		winki.lib.fill_combo(combo_sub3, subtitles, sub3, self.no_subtitles)

		if self.app.project.get_source() == 'file':
			label_sub1.set_sensitive(True)
			label_sub2.set_sensitive(False)
			label_sub3.set_sensitive(False)
			combo_sub1.set_sensitive(False)
			combo_sub2.set_sensitive(False)
			combo_sub3.set_sensitive(False)
		else:
			label_sub1.set_sensitive(True)
			label_sub2.set_sensitive(True)
			label_sub3.set_sensitive(True)
			combo_sub1.entry.grab_focus() 

		button_sub1 = self.xml.get_widget('button_sub1')
		entry_sub1 = self.xml.get_widget('entry_sub1')
		label_external = self.xml.get_widget('label_external_sub')
		button_consider = self.xml.get_widget('checkbutton_consider_external')

		if self.app.project.get_user_mode() == 'Advanced':
			button_sub1.show()
			entry_sub1.show()
			label_external.show()
			if self.app.project.get_encoding() != 'copy':
				button_consider.show()
			else:
				button_consider.show()
		else:
			button_sub1.hide()
			entry_sub1.hide()
			label_external.hide()
			button_consider.hide()

		self.on_mix_subtitles_toggled()
		return True

	def on_next(self):
		return self.update_project()

	def on_prev(self):
		return self.update_project()

	def can_exit(self):
		return True

	def show_help(self):
		return winki.base_page.BasePage.show_help(self, 'subtitles.html')

	def get_subtitles_filename(self, index):
		if index < 0 or index > 3:
			winki.lib.debug('PageSubtitles.get_subtitles_filename()', 'invalid index', winki.lib.error)
			return None

		entry_name = ('entry_sub%d' % index)
		entry = self.xml.get_widget(entry_name)
		return entry.get_text()

	def set_subtitles_filename(self, file_name, index):
		if index < 0 or index > 3:
			winki.lib.debug('PageSubtitles.get_subtitles_filename()', 'invalid index', winki.lib.error)
			return

		entry_name = ('entry_sub%d' % index)
		entry = self.xml.get_widget(entry_name)
		entry.set_text(file_name)
		return

	def update_project(self):
		result = True

		result &= self.submit_dvd_subtitles()
		result &= self.submit_subtitle_files()

		button_mix = self.xml.get_widget('checkbutton_mix_subitles')
		self.app.project.set_mix_subtitles(button_mix.get_active())

		button_ext = self.xml.get_widget('checkbutton_consider_external')
		self.app.project.set_consider_ext_sub(button_ext.get_active())
		if self.app.project.get_consider_ext_sub():
			entry = self.xml.get_widget('entry_sub1')
			fname = entry.get_text()
			self.app.project.set_external_subtitle(fname)

		return result

	def on_mix_subtitles_toggled(self):
		button_mix = self.xml.get_widget('checkbutton_mix_subitles')
		combo_sub2 = self.xml.get_widget('combo_sub2')
		combo_sub3 = self.xml.get_widget('combo_sub3')

		activate = button_mix.get_active()

		if self.app.project.get_source() == 'file':
			activate = True

		combo_sub2.set_sensitive(not activate)
		combo_sub3.set_sensitive(not activate)

		self.on_enable_mix_ui()
		return

	def on_enable_mix_ui(self):
		button_mix = self.xml.get_widget('checkbutton_mix_subitles')
		button_ext = self.xml.get_widget('checkbutton_consider_external')
		combo_sub1 = self.xml.get_widget('combo_sub1')
		button_sub1 = self.xml.get_widget('button_sub1')
		entry_sub1 = self.xml.get_widget('entry_sub1')
		label_sub = self.xml.get_widget('label_external_sub')

		if self.app.project.get_source() == 'file':
			if self.app.project.get_user_mode() == 'Advanced' and self.app.project.get_encoding() != 'copy':
				button_ext.set_sensitive(True)
			else:
				button_ext.set_sensitive(False)
				
			if button_mix.get_active() or button_ext.get_active():
				entry_sub1.set_sensitive(True)
				button_sub1.set_sensitive(True)
				label_sub.set_sensitive(True)
				combo_sub1.set_sensitive(False)
			else:
				button_sub1.set_sensitive(False)
				label_sub.set_sensitive(False)
				combo_sub1.set_sensitive(False)
				entry_sub1.set_sensitive(False)
			return

		cur_title = self.app.project.get_current_title()
		if cur_title < 0:
			return

		cur_sub = self.app.project.get_subtitles(cur_title)
		i = winki.lib.get_index(cur_sub, combo_sub1.entry.get_text())

		fname = entry_sub1.get_text()

		if button_mix.get_active():
			# i == 0 means no DVD subtitles
			if i > 0:
				combo_sub1.set_sensitive(True)
				entry_sub1.set_sensitive(False)
				button_sub1.set_sensitive(False)

			# no DVD subtitles but external file
			elif fname and len(fname):
				combo_sub1.set_sensitive(False)
				entry_sub1.set_sensitive(True)
				button_sub1.set_sensitive(True)

			# no subtitles at all
			else:
				combo_sub1.set_sensitive(True)
				entry_sub1.set_sensitive(True)
				button_sub1.set_sensitive(True)
		else:
			entry_sub1.set_sensitive(True)
			button_sub1.set_sensitive(True)
			combo_sub1.set_sensitive(True)

		if self.app.project.get_user_mode() == 'Advanced' and self.app.project.get_encoding() != 'copy':
			button_ext.show()
		else:
			button_ext.hide()
		return

	def submit_dvd_subtitles(self):
		combo_sub1 = self.xml.get_widget('combo_sub1')
		combo_sub2 = self.xml.get_widget('combo_sub2')
		combo_sub3 = self.xml.get_widget('combo_sub3')

		result = [0, 0, 0]
		if self.app.project.get_source() == 'file':
			return self.app.project.set_current_subtitles(result)

		cur_title = self.app.project.get_current_title()
		if cur_title < 0:
			return False
		cur_sub = self.app.project.get_subtitles(cur_title)
		if winki.lib.is_sensitive(combo_sub1):
			result[0] = winki.lib.get_index(cur_sub, combo_sub1.entry.get_text())

		if winki.lib.is_sensitive(combo_sub2):
			result[1] = winki.lib.get_index(cur_sub, combo_sub2.entry.get_text())

		if winki.lib.is_sensitive(combo_sub3):
			result[2] = winki.lib.get_index(cur_sub, combo_sub3.entry.get_text())

		return self.app.project.set_current_subtitles(result)

	def submit_subtitle_files(self):
		result = True

		primary = _('Could not access file')

		for i in range(1):
			entry_name = ('entry_sub%d' % (i + 1))
			entry = self.xml.get_widget(entry_name)
			if not winki.lib.is_sensitive(entry):
				continue
			fname = entry.get_text()
			if fname and len(fname):
				if winki.lib.is_file(fname):
					if winki.lib.is_readable(fname):
						self.app.project.set_external_subtitle(fname)
					else:
						secondary = (_('%s is not readable by you') % fname)
						winki.lib.alert(primary, secondary, 'error')
						result = False
				else:
					secondary = (_('%s is not regular file') % fname)
					error.show(primary, secondary)
					result = False
						

		return result

