# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module string...'
	import string

	print 'Importing module winki.lib...'
	import winki.lib

	print 'Importing module winki.build...'
	import winki.build

	print 'Importing module winki.base_page...'
	import winki.base_page

	print 'Importing module winki.base_player_page...'
	import winki.base_player_page

except:
	error_quit(0, 1)

class PageTest(winki.base_page.BasePage, winki.base_player_page.BasePlayerPage):
	def __init__(self, xml, app):
		winki.base_page.BasePage.__init__(self, xml, app)
		winki.base_player_page.BasePlayerPage.__init__(self)
		return
		
	def check_widgets(self):
		widgets = [ 'button_10_backward_test',
		'button_10_forward_test', 'button_1_backward_test',
		'button_1_forward_test', 'button_30_backward_test',
		'button_30_forward_test', 'button_test_settings',
		'drawingarea_test', 'radio_test_audio1', 'radio_test_audio2',
		'radio_test_audio3', 'radio_test_sub1', 'radio_test_sub2',
		'radio_test_sub3']

		winki.lib.check_widgets(self.xml, widgets)
		return


	def on_prepare(self):
		self.enable_ui(True)

		radio_audio1 = self.xml.get_widget('radio_test_audio1')
		radio_audio2 = self.xml.get_widget('radio_test_audio2')
		radio_audio3 = self.xml.get_widget('radio_test_audio3')
		radio_sub1 = self.xml.get_widget('radio_test_sub1')
		radio_sub2 = self.xml.get_widget('radio_test_sub2')
		radio_sub3 = self.xml.get_widget('radio_test_sub3')

		audio = self.app.project.get_current_audio()
		assert(len(audio) == 3)
		if audio[0] != 0:
			radio_audio1.set_active(True)
			radio_audio1.grab_focus()
		elif audio[1] != 0:
			radio_audio2.set_active(True)
			radio_audio2.grab_focus()
		elif audio[2] != 0:
			radio_audio3.set_active(True)
			radio_audio3.grab_focus()

		subs = self.app.project.get_current_subtitles()
		assert(len(subs) == 3)
		if subs[0] != 0:
			radio_sub1.set_active(True)
			radio_sub1.grab_focus()
		elif subs[1] != 0:
			radio_sub2.set_active(True)
			radio_sub2.grab_focus()
		elif subs[2] != 0:
			radio_sub3.set_active(True)
			radio_sub3.grab_focus()

		return True

	def on_next(self):
		if not winki.base_player_page.BasePlayerPage.on_next(self):
			return False

		return True

	def on_prev(self):
		if not winki.base_player_page.BasePlayerPage.on_prev(self):
			return False

		return True

	def can_exit(self):
		return not winki.base_player_page.BasePlayerPage.has_player(self)

	def show_help(self):
		return winki.base_page.BasePage.show_help(self, 'test.html')

	def on_test_settings(self):
		if winki.base_player_page.BasePlayerPage.has_player(self):
			self.stop_test()
		else:
			self.start_test()
		return

	def start_test(self):
		if self.app.project.get_source() == 'dvd' and self.app.project.get_current_title() < 0:
			winki.lib.debug('PageTest.start_crop_detect()', 'no current title set', winki.lib.error)
			return

		self.enable_ui(False)

		aid = self.get_audio_index()
		sid = self.get_subtitle_index()

		builder = winki.build.Builder(self.app.project)
		cmd = builder.build_test_command(aid, sid)

		area = self.xml.get_widget('drawingarea_test')

		winki.base_player_page.BasePlayerPage.start_player(self, area, cmd, self.dummy)
		return

	def stop_test(self):
		winki.base_player_page.BasePlayerPage.stop_player(self)

		self.enable_ui(True)
		return

	def enable_ui(self, enable):
		button_test = self.xml.get_widget('button_test_settings')
		area = self.xml.get_widget('drawingarea_test')
		forward1 = self.xml.get_widget('button_1_forward_test')
		forward10 = self.xml.get_widget('button_10_forward_test')
		forward30 = self.xml.get_widget('button_30_forward_test')
		backward1 = self.xml.get_widget('button_1_backward_test')
		backward10 = self.xml.get_widget('button_10_backward_test')
		backward30 = self.xml.get_widget('button_30_backward_test')
		radio_audio1 = self.xml.get_widget('radio_test_audio1')
		radio_audio2 = self.xml.get_widget('radio_test_audio2')
		radio_audio3 = self.xml.get_widget('radio_test_audio3')
		radio_sub1 = self.xml.get_widget('radio_test_sub1')
		radio_sub2 = self.xml.get_widget('radio_test_sub2')
		radio_sub3 = self.xml.get_widget('radio_test_sub3')

		if not enable:
			winki.lib.set_button_label(button_test, _('_Stop testing'))

		else:
			winki.lib.set_button_label(button_test, _('T_est settings'))

		area.set_sensitive(enable)
		forward1.set_sensitive(not enable)
		forward10.set_sensitive(not enable)
		forward30.set_sensitive(not enable)
		backward1.set_sensitive(not enable)
		backward10.set_sensitive(not enable)
		backward30.set_sensitive(not enable)

		audio = self.app.project.get_current_audio()
		assert(len(audio) == 3)
		radio_audio1.set_sensitive(enable and audio[0] != 0)
		radio_audio2.set_sensitive(enable and audio[1] != 0)
		radio_audio3.set_sensitive(enable and audio[2] != 0)

		subs = self.app.project.get_current_subtitles()
		assert(len(subs) == 3)
		radio_sub1.set_sensitive(enable and subs[0] != 0)
		radio_sub2.set_sensitive(enable and subs[1] != 0)
		radio_sub3.set_sensitive(enable and subs[2] != 0)
		return

	def get_audio_index(self):
		radio_first = self.xml.get_widget('radio_test_audio1')
		radio_second = self.xml.get_widget('radio_test_audio2')
		radio_third = self.xml.get_widget('radio_test_audio3')

		if winki.lib.is_radio_active(radio_first):
			return 0
		elif winki.lib.is_radio_active(radio_second):
			return 1
		elif winki.lib.is_radio_active(radio_third):
			return 2
		else:
			# more radio buttons in this group?
			assert(0)

		return 0

	def get_subtitle_index(self):
		radio_first = self.xml.get_widget('radio_test_sub1')
		radio_second = self.xml.get_widget('radio_test_sub2')
		radio_third = self.xml.get_widget('radio_test_sub3')

		if winki.lib.is_radio_active(radio_first):
			return 0
		elif winki.lib.is_radio_active(radio_second):
			return 1
		elif winki.lib.is_radio_active(radio_third):
			return 2
		else:
			# more radio buttons in this group?
			assert(0)

		return 0

	def dummy(self, line):
		return

