# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	"""Print error message to the console and exit"""

	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module gtk...'
	import gtk

	print 'Importing module winki.lib...'
	import winki.lib

	print 'Importing module string...'
	import string

	print 'Importing module os...'
	import os

	print 'Importing module math...'
	import math

	print 'Importing module glob...'
	import glob

	print 'Importing module build...'
	import build

	print 'Importing module winki.config...'
	import winki.config

except:
	error_quit(0, 1)

class Project:
	"""
	This class contains all the choices the user has made during
	program execution.
	"""

	def __init__(self):
		"""Initialize every variable which will be used later"""

		# -- source --

		# which source media should be used
		# type: string (it should be one of the elements of self.__sources)
		self.__source = None

		# the source file name (if the source is a file)
		self.__source_filename = None

		self.__sources = ['file', 'dvd', 'vcd']

		# -- encoding --

		# which encoding method should be used
		# type: string (it should be one of the elements of self.__encodings)
		self.__encoding = None

		self.__encodings = ['copy', 'multi pass', 'vcd', 'svcd']

		self.__mix_streams = False

		# -- info --

		# all of these will be used for setting meta-information in the output stream
		# type: string
		self.__comment_name = None
		self.__comment_artist = None
		self.__comment_genre = None
		self.__comment_subject = None
		self.__comment_copyright = None
		self.__comment_srcform = None
		self.__comment_comment = None

		# directory where the output will be placed
		self.__dest_dirname = None

		# -- size --

		# in how many chunks should we split the output
		# type: integer
		self.__number_chunks = -1

		# how big should be each ouput file
		# type: integer index between 0 and len(chunk_sizes) - 1
		self.__chunk_size = -1

		self.__chunk_sizes = [650, 700, 800, -1]

		# which chapters should be encoded in every output file
		# type: list of index pairs [start_chapter, end_chapter]
		self.__chunks = []

		# -- dvd --

		# all titles found on the dvd
		# type: list of strings
		self.__dvd_titles = None

		# all chapters found on the dvd
		# type: list of lists of strings (each list represents
		# the chapters in a dvd title)
		self.__dvd_chapters = None

		# lenghts in seconds for every chapter
		# type: list of lists of integers (each list represents
		# the lengths in seconds of the chapters in a dvd title)
		self.__dvd_chapter_times = None
		
		# meaningful only if self.__encoding == 'dvd'
		# all audio streams found on the dvd
		# type: list of lists of strings (each list represents
		# the audio streams in a dvd title)
		self.__dvd_audio = None

		# meaningful only if self.__encoding == 'file'
		# all audio streams found in the file
		# type: lists of strings
		self.__file_audio = None

		# the indices of the current audio streams in self.__dvd_audio
		# or self.__file_audio
		# type: list of 3 integers (0 means none)
		self.__current_audio = [0, 0, 0]
		
		# all subtitle streams found on the dvd
		# type: list of lists of strings (each list represents
		# the subtitle streams in a dvd title)
		self.__dvd_subtitles = None

		# the indices of the current subtitles in self.__dvd_subtitles
		# type: list of 3 integers (0 means none)
		self.__current_subtitles = [0, 0, 0]

		# the current dvd title
		# type: integer
		self.__current_title = -1

		# external audio files to be used
		# type: list of strings
		self.__external_audio = [None, None, None]

		# lenght in seconds for a media file
		# type: integer
		self.__file_time = -1
		
		# if the subtitles should be rendered into the movie
		# type: bool
		self.__mix_subtitles = False

		# the crop numbers
		# type: list of integers
		self.__crop = [0, 0, 0, 0]

		# the bitrate to be used
		# type: integer
		self.__bitrate = -1

		# the constructed commands
		self.__commands = []

		# user mode
		# possible values: Default or Advanced
		# type: string
		self.__user_mode = None

		# if external subtitles should be considered by
		# calculating the vbitrate
		self.__consider_ext_sub = False

		# which external subtitle file should be used
		# type: string
		self.__extrenal_subtitle = None

		self.__scale_detected = []

		self.__aspect_ratio = None
			
		self.__configuration = winki.config.Config()
		self.__configuration.load()
		return

	def get_source(self):
		return self.__source

	def set_source(self, src):
		index = winki.lib.get_index(self.__sources, src)
		if index == -1:
			winki.lib.debug('Project.set_source()', 'entry not found', winki.lib.error)
			return False

		self.__source = src
		if src == 'file':
			self.__number_chunks = 1
		return True

	def set_source_filename(self, filename):
		"""Set method for self.__source_filename"""
		self.__source_filename = filename
		return True

	def get_source_filename(self, basename_only = False):
		"""Get method for self.__source_filename"""
		if not self.__source_filename:
			return None

		if not basename_only:
			return self.__source_filename

		return os.path.basename(self.__source_filename)

	def set_encoding(self, enc):
		"""Set method for self.__encoding"""
		index = winki.lib.get_index(self.__encodings, enc)
		if index == -1:
			winki.lib.debug('Project.set_encoding()', 'entry not found', winki.lib.error)
			return False

		self.__encoding = enc
		return True

	def get_encoding(self):
		"""Get method for self.__encoding"""
		return self.__encoding

	def set_mix_streams(self, value):
		self.__mix_streams = value
		return

	def get_mix_streams(self):
		return self.__mix_streams

	def set_name(self, name):
		"""Set method for self.__comment_name (movie name)"""
		self.__comment_name = name
		return True

	def get_name(self, formatted = False):
		"""Get method for self.__comment_name (movie name)"""
		if not formatted:
			return self.__comment_name

		return winki.lib.escape_name(self.__comment_name)

	def set_artist(self, artist):
		"""Set method for self.__comment_artist"""
		self.__comment_artist = artist
		return True

	def get_artist(self, formatted = False):
		"""Get method for self.__comment_artist"""
		if not formatted:
			return self.__comment_artist

		return winki.lib.escape_name(self.__comment_artist)

	def set_genre(self, genre):
		self.__comment_genre = genre
		return True

	def get_genre(self, formatted = False):
		if not formatted:
			return self.__comment_genre

		return winki.lib.escape_name(self.__comment_genre)

	def set_subject(self, subject):
		self.__comment_subject = subject
		return True

	def get_subject(self, formatted = False):
		if not formatted:
			return self.__comment_subject

		return winki.lib.escape_name(self.__comment_subject)

	def set_copyright(self, copyright):
		self.__comment_copyright = copyright
		return True

	def get_copyright(self, formatted = False):
		if not formatted:
			return self.__comment_copyright

		return winki.lib.escape_name(self.__comment_copyright)

	def set_srcform(self, srcform):
		self.__comment_srcform = srcform
		return True

	def get_srcform(self, formatted = False):
		if not formatted:
			return self.__comment_srcform

		return winki.lib.escape_name(self.__comment_srcform)

	def set_comment(self, comment):
		self.__comment_comment = comment
		return True

	def get_comment(self, formatted = False):
		if not formatted:
			return self.__comment_comment

		return winki.lib.escape_name(self.__comment_comment)

	def get_commands(self):
		if not self.__commands or not len(self.__commands):
			builder = build.Builder(self)
			self.__commands = builder.build_action_commands()

		return self.__commands

	def set_commands(self, commands):
		self.__commands = commands
		return

	def reset_commands(self):
		self.__commands = None
		return

	def cd_wd(self):
		if not self.__dest_dirname:
			winki.lib.debug('Project.cd_wd()', 'no dest_dirname', winki.lib.error)
			return False

		try:
			os.chdir(self.__dest_dirname)
		except:
			winki.lib.debug('Project.cd_wd()', 'cannot change the current working folder to %s' % self.__dest_dirname, winki.lib.error)
			return False

		return True


	def get_default_output_dir(self):
		return self.__configuration.get_output_dir()

	def get_default_size(self):
		return self.__configuration.get_size()

	def get_default_source(self):
		return self.__configuration.get_source()

	def get_default_encoding(self):
		return self.__configuration.get_encoding()

	def set_outputdir(self, dest_dirname):
		self.__dest_dirname = dest_dirname
		return True

	def get_outputdir(self, basename_only = False):
		if not self.__dest_dirname:
			winki.lib.debug('Project.get_outputdir()', 'self.__dest_dirname not set', winki.lib.error)
			return None

		if not basename_only:
			return self.__dest_dirname

		return os.path.basename(self.__dest_dirname)

	def get_file_output_name(self, chunk_nr):
		if not self.__dest_dirname:
			winki.lib.debug('Project.get_file_output_name()', 'self.__dest_dirname not set', winki.lib.error)
			return ''

		if self.__number_chunks < 0:
			winki.lib.debug('Project.get_file_output_name()', 'self.__number_chunks not set', winki.lib.error)
			return ''

		# the file name is the same as the directory name
		path, fname = os.path.split(self.__dest_dirname)
		if not fname or not(len(fname)):
			path, fname = os.path.split(path)

		if not fname or not(len(fname)):
			winki.lib.debug('Project.get_file_output_name()', 'cannot extract name', winki.lib.error)
			return ''

		if self.__number_chunks > 1:
			fname = '%s_%d' % (fname, chunk_nr + 1)

		return os.path.join(self.__dest_dirname, fname)

	def build_directories(self):
		if not self.__dest_dirname:
			return False

		path = self.__dest_dirname

		if winki.lib.is_dir(path):
			if winki.lib.is_writable(path):
				winki.lib.debug('Project.build_directories()', 'nothing to do')
				return True
			else:
				winki.lib.debug('Project.build_directories()', 'folder exist but is not writable', winki.lib.error)
				return False
		else:
			if winki.lib.does_path_exist(path):
				winki.lib.debug('Project.build_directories()', 'path exist but is not a folder', winki.lib.error)
				return False
			else:
				if not winki.lib.mkdir(path):
					winki.lib.debug('Project.build_directories()', 'cannot create destination folder', winki.lib.error)
					return False

		return True

	def estimate_nr_chunks(self, title):
		if self.__encoding == 'multi pass':
			return self.estimate_nr_chunks_multi_pass(title)

		if self.__encoding == 'copy':
			return 1

		elif self.__encoding == 'vcd':
			return self.estimate_nr_chunks_vcd(title)

		elif self.__encoding == 'svcd':
			return self.estimate_nr_chunks_svcd(title)

		else:
			print self.__encoding
			assert(0)

		return 1

	def estimate_nr_chunks_vcd(self, title):
		coef = self.__configuration.get_vcd_bytes_per_second()
		seconds = self.get_title_time(title)
		size = self.__chunk_sizes[self.__chunk_size]
		if size < 0:
			size = self.get_default_size()

		target = size * 1024 * 1024
		result_size = seconds * coef
		
		if target == 0:
			return 1

		nr_chunks = int(math.ceil(result_size / target))

		if nr_chunks < 1: 
			nr_chunks = 1

		return nr_chunks

	def estimate_nr_chunks_svcd(self, title):
		abits = self.__configuration.get_mp2_bytes_per_second()
		#sbits =  self.__configuration.get_sub_bytes_per_second()
		penalty_ratio = self.__configuration.get_svcd_container_penalty()
		pref_bitrate = self.__configuration.get_preferred_bitrate()
		min_bitrate = self.__configuration.get_minimum_bitrate()

		seconds = self.get_title_time(title)
		size = self.__chunk_sizes[self.__chunk_size]
		if size < 0:
			size = self.get_default_size()

		target = size * 1024 * 1024
		penalty = target * penalty_ratio

		asize = seconds * abits * self.get_audio_count()
		#ssize = seconds * sbits * self.get_sub_count()
		occupied = asize# + ssize

		if (target - penalty) == 0:
			return 1

		# nr chunks
		chunks_estimated = (125 * pref_bitrate * seconds + occupied) / (target - penalty)

		# test the result
		target *= int(chunks_estimated)
		penalty = target * penalty_ratio

		if seconds == 0:
			return 1

		vbitrate = (target - occupied - penalty) / seconds / 125

		if vbitrate > min_bitrate:
			chunks = math.floor(chunks_estimated)
		else:
			chunks = math.ceil(chunks_estimated)

		self.set_number_chunks(chunks)
		return chunks

	def estimate_nr_chunks_multi_pass(self, title):
		abits = self.__configuration.get_audio_bytes_per_second()
		sbits =  self.__configuration.get_sub_bytes_per_second()
		penalty_ratio = self.__configuration.get_container_penalty()
		pref_bitrate = self.__configuration.get_preferred_bitrate()
		min_bitrate = self.__configuration.get_minimum_bitrate()

		seconds = self.get_title_time(title)
		size = self.__chunk_sizes[self.__chunk_size]
		if size < 0:
			size = self.get_default_size()

		target = size * 1024 * 1024
		penalty = target * penalty_ratio

		asize = seconds * abits * self.get_audio_count()
		ssize = seconds * sbits * self.get_sub_count()
		occupied = asize + ssize

		if (target - penalty) == 0:
			return 1

		# nr chunks
		chunks_estimated = (125 * pref_bitrate * seconds + occupied) / (target - penalty)

		# test the result
		target *= int(chunks_estimated)
		penalty = target * penalty_ratio

		if seconds == 0:
			return 1

		vbitrate = (target - occupied - penalty) / seconds / 125

		if vbitrate > min_bitrate:
			chunks = math.floor(chunks_estimated)
		else:
			chunks = math.ceil(chunks_estimated)

		self.set_number_chunks(chunks)
		return chunks

	def get_number_chunks(self):
		return self.__number_chunks

	def set_number_chunks(self, number):
		msg_info = '%d chunks requested' % number

		if number < 1:
			winki.lib.debug('Project.set_number_chunks()', msg_info)
			number = 1

		if number > 5:
			winki.lib.debug('Project.set_number_chunks()', msg_info)
			number = 5

		self.__number_chunks = int(number)
		return  self.__number_chunks

	def get_chunk_size(self):
		if self.__chunk_size < 0 or self.__chunk_size >= len(self.__chunk_sizes):
			return -1

		return self.__chunk_sizes[self.__chunk_size]

	def get_chunk_sizes(self):
		return self.__chunk_sizes

	def get_str_chunk_sizes(self):
		result = []
		for i in range(len(self.__chunk_sizes) - 1):
			s = '%d' % self.__chunk_sizes[i]
			result.append(s)

		return result

	def set_chunk_size(self, text):
		if not text or not len(text):
			return True

		number = winki.lib.to_int(text)
		if number < 0:
			winki.lib.debug('Project.set_chunk_size()', 'not a number %s' % text, winki.lib.error)
			return False

		index = -1
		for i in range(len(self.__chunk_sizes)):
			size = '%d' % self.__chunk_sizes[i]
			if text == size:
				index = i
				break

		if index < 0:
			index = 3
			self.__chunk_sizes[index] = number

		self.__chunk_size = index
		return True

	def get_chunks(self):
		return self.__chunks

	def set_chunks(self, chunks):
		self.__chunks = chunks
		return

	def get_titles(self):
		return self.__dvd_titles

	def set_titles(self, titles):
		self.__dvd_titles = titles
		return

	def set_title(self, title):
		index = winki.lib.get_index(self.__dvd_titles, title)
		if index == -1:
			winki.lib.debug('Project.set_title()', 'invalid index requested', winki.lib.error)
			return False

		self.set_current_title(index)
		return True

	def get_chapters(self, title):
		if not self.__dvd_chapters:
			winki.lib.debug('Project.get_chapters()', 'dvd chapters not initialized', winki.lib.error)
			return None

		if title < 0 or title >= len(self.__dvd_chapters):
			winki.lib.debug('Project.get_chapters()', 'invalid index requested', winki.lib.error)
			return None
			
		return self.__dvd_chapters[title]

	def set_chapters(self, chapters):
		self.__dvd_chapters = chapters
		return

	def split_chapters(self, chunks):
		result = []

		if chunks == 0 or chunks > 5:
			winki.lib.debug('Project.split_chapters()', 'invalid number chunks (%d) requested' % chunks, winki.lib.error)
			return result

		if self.__current_title == -1:
			winki.lib.debug('Project.split_chapters()', 'current_title not set', winki.lib.error)
			return result

		if not self.__dvd_chapters or len(self.__dvd_chapters) <= self.__current_title:
			winki.lib.debug('Project.split_chapters()', 'dvd_chapters not set', winki.lib.error)
			return result

		if not self.__dvd_chapter_times or len(self.__dvd_chapter_times) <= self.__current_title:
			winki.lib.debug('Project.split_chapters()', 'dvd_chapter_times not set', winki.lib.error)
			return result

		chapter_times = self.__dvd_chapter_times[self.__current_title]
		dvd_chapters = self.__dvd_chapters[self.__current_title]

		# more chunks are requested than chapters in this title are available
		if len(dvd_chapters) < chunks or len(chapter_times) < chunks:
			# todo: pack every chapter in a chunk instead of giving up
			winki.lib.debug('Project.split_chapters()', 'more chunks are requested than chapters available', winki.lib.error)
			return result

		if chunks == 1:
			chunk_pair = [0]
			chunk_pair.append(len(chapter_times) - 1)
			result.append(chunk_pair)
			return result

		title_time = self.get_title_time(self.__current_title)
		chunk_size = title_time / chunks

		seconds = 0
		chunk_pair = [0]
		i = 0
		while i < len(chapter_times):
			# pack all remaining chapters into the last chunk
			if len(result) == (chunks - 1):
				break

			if (seconds + chapter_times[i]) < chunk_size:
				seconds += chapter_times[i]
				i += 1
			else:
				# try to add one more chapter
				if i < (len(chapter_times) - 1):
					below = seconds
					above = seconds + chapter_times[i]
					if math.fabs(chunk_size - below) > math.fabs(above - chunk_size):
						chunk_pair.append(i)
						result.append(chunk_pair)

						# reset seconds and set chunk_pair to the next chapter
						seconds = 0
						if i < (len(chapter_times) - 1):
							i += 1
						chunk_pair = [i]
					else:
						chunk_pair.append(i - 1)
						result.append(chunk_pair)

						# reset seconds and set chunk_pair to the next chapter
						seconds = 0
						chunk_pair = [i]
				else:
					assert(0)


		chunk_pair.append(len(chapter_times) - 1)
		assert(len(chunk_pair) == 2)
		result.append(chunk_pair)
		assert(len(result) == chunks)

		return result

	def get_title_time(self, title):
		if not self.__dvd_chapter_times:
			winki.lib.debug('Project.get_title_time()', 'chapter_times not initialized', winki.lib.error)
			return 0

		if len(self.__dvd_chapter_times) <= title:
			winki.lib.debug('Project.get_title_time()', 'invalid title index passed', winki.lib.error)
			return 0

		seconds = 0
		for chapter in self.__dvd_chapter_times[title]:
			seconds += chapter
		return seconds

	def get_chapter_times(self, title):
		if not self.__dvd_chapter_times:
			winki.lib.debug('Project.get_chapter_times()', 'dvd_chapter_times not initialized', winki.lib.error)
			return None

		if title < 0 or title >= len(self.__dvd_chapter_times):
			winki.lib.debug('Project.get_chapter_times()', 'invalid title index', winki.lib.error)
			return None
			
		return self.__dvd_chapter_times[title]

	def set_chapter_times(self, times):
		self.__dvd_chapter_times = times
		return

	def get_audio(self, title = -1):
		if not self.__source:
			winki.lib.debug('Project.get_audio()', 'self.__source not initialized', winki.lib.error)
			return None

		if self.__source == 'dvd':
			return self.get_dvd_audio(title)

		elif self.__source == 'file' or self.__source == 'vcd':
			return self.get_file_audio()

		else:
			# new source?
			assert(0)

		return None

	# do not call directly; use get_audio() instead
	def get_dvd_audio(self, title):
		if not self.__dvd_audio:
			winki.lib.debug('Project.get_dvd_audio()', 'dvd_audio not initialized', winki.lib.error)
			return None

		if title < 0 or title >= len(self.__dvd_audio):
			winki.lib.debug('Project.get_dvd_audio()', 'invalid title index', winki.lib.error)
			return None
			
		return self.__dvd_audio[title]

	# do not call directly; use get_audio() instead
	def get_file_audio(self):
		if not self.__file_audio:
			winki.lib.debug('Project.get_file_audio()', 'file_audio not initialized', winki.lib.error)
			return None

		return self.__file_audio

	def set_audio(self, audio):
		if not self.__source:
			winki.lib.debug('Project.set_audio()', 'self.__source not set', winki.lib.error)
			return

		if self.__source == 'dvd':
			self.__dvd_audio = audio

		elif self.__source == 'file' or self.__source == 'vcd':
			self.__file_audio = audio

		else:
			# new source?
			assert(0)
		return

	def set_current_audio(self, audio):
		self.__current_audio = [0, 0, 0]

		if self.__source == 'dvd':
			return self.set_current_dvd_audio(audio)

		elif self.__source == 'file' or self.__source == 'vcd':
			return self.set_current_file_audio(audio)

		else:
			assert(0)

		return False

	# do not call directly; use set_current_audio() instead
	def set_current_file_audio(self, audio):
		if not audio or len(audio) != 3:
			winki.lib.debug('Project.set_current_file_audio()', 'invalid parameter passed', winki.lib.error)
			return False

		for i in range(len(audio)):
			index = winki.lib.get_index(self.__file_audio, audio[i])
			if index < 0:
				self.__current_audio[i] = 0
			else:
				self.__current_audio[i] = index

		return True

	# do not call directly; use set_current_audio() instead
	def set_current_dvd_audio(self, audio):
		if not audio or len(audio) != 3:
			winki.lib.debug('Project.set_current_dvd_audio()', 'invalid parameter passed', winki.lib.error)
			return False

		if self.__current_title < 0:
			winki.lib.debug('Project.set_current_dvd_audio()', 'current_title < 0', winki.lib.error)
			return False

		if not self.__dvd_audio or len(self.__dvd_audio) <= self.__current_title:
			winki.lib.debug('Project.set_current_dvd_audio()', 'dvd_audio not initialized', winki.lib.error)
			return False

		for i in range(len(audio)):
			index = winki.lib.get_index(self.__dvd_audio[self.__current_title], audio[i])
			if index < 0:
				self.__current_audio[i] = 0
			else:
				self.__current_audio[i] = index

		return True

	def get_current_audio(self):
		return self.__current_audio

	def get_audio_count(self):
		count = 0
		for audio in self.__current_audio:
			if audio > 0:
				count += 1
		return count

	def get_preferred_audio(self):
		result = [0, 0, 0]

		audio = self.get_audio(self.__current_title)
		if not audio or not len(audio):
			return result

		langs = self.get_str_preferred_audio()
		languages = string.split(langs)

		idx = 0
		for lang in languages:
			if idx == len(result):
				break

			for i in range(len(audio)):
				if string.find(audio[i], lang) > -1:
					result[idx] = i
					idx += 1
					break
		return result

	def get_subtitles(self, title):
		if not self.__dvd_subtitles:
			winki.lib.debug('Project.get_subtitles()', 'dvd_subtitles not initialized', winki.lib.error)
			return None

		if title < 0 or title >= len(self.__dvd_subtitles):
			winki.lib.debug('Project.get_subtitles()', 'invalid title index', winki.lib.error)
			return None
			
		return self.__dvd_subtitles[title]

	def set_subtitles(self, subtitles):
		self.__dvd_subtitles = subtitles
		return

	def set_current_subtitles(self, sub_list):
		self.__current_subtitles = [0, 0, 0]
		if self.__source == 'file':
			return True

		if len(sub_list) != len(self.__current_subtitles):
			winki.lib.debug('Project.set_current_subtitles()', 'invalid parameter passed', winki.lib.error)
			return False

		if self.__current_title < 0:
			winki.lib.debug('Project.set_current_subtitles()', 'current_title not set', winki.lib.error)
			return False

		if not self.__dvd_subtitles or len(self.__dvd_subtitles) <= self.__current_title:
			winki.lib.debug('Project.set_current_subtitles()', 'dvd_subtitles not set', winki.lib.error)
			return False

		for i in range(len(sub_list)):
			# todo: check min-max range
			self.__current_subtitles[i] = sub_list[i]

		return True

	def get_current_subtitles(self):
		return self.__current_subtitles

	def get_sub_count(self):
		count = 0
		for sub in self.__current_subtitles:
			if sub > 0:
				count += 1
		return count

	def get_preferred_subtitles(self):
		result = [0, 0, 0]

		subtitles = self.get_subtitles(self.__current_title)
		if not subtitles or not len(subtitles):
			return result

		langs = self.get_str_preferred_subtitles()
		languages = string.split(langs)

		idx = 0
		for lang in languages:
			if idx == len(result):
				break

			for i in range(len(subtitles)):
				if string.find(subtitles[i], lang) > -1:
					result[idx] = i
					idx += 1
					break
		return result

	def get_final_subtitles(self):
		if not self.__current_subtitles or len(self.__current_subtitles) != 3:
			winki.lib.debug('Project.get_final_subtitles()', 'current_subtitles not set', winki.lib.error)
			return [0, 0, 0]

		sub = []
		sub.extend(self.__current_subtitles)
		
		# index 0 in the subtitles combo is the localized string 'none'
		# if the user has supplied the third but not the second subtitles language
		if sub[1] == 0 and sub[2] > 0:
			sub[1] = sub[2]
			sub[2] = 0

		# if the user has supplied the second but not the first subtitles language
		if sub[0] == 0 and sub[1] > 0:
			sub[0] = sub[1]
			sub[1] = 0

		# if we happen to have the first and the third but not the second language (this may happen if the user specified only the second and the third languages)
		if sub[0] > 0 and sub[1] == 0 and sub[2] > 0:
			sub[1] = sub[2]
			sub[2] = 0

		return sub

	def get_current_title(self):
		return self.__current_title

	def set_current_title(self, current_title):
		if not self.__dvd_titles:
			winki.lib.debug('Project.set_current_title()', 'dvd_titles not initialized', winki.lib.error)
			return self.__current_title

		if current_title < 0 or current_title >= len(self.__dvd_titles):
			winki.lib.debug('Project.set_current_title()', 'invalid title index', winki.lib.error)
			return self.__current_title

		self.__current_title = current_title
		return self.__current_title

	def set_external_audio(self, index, fname):
		if index < 0 or index > 2:
			return

		if not fname or not len(fname):
			return

		self.__external_audio[index] = fname
		return

	def get_external_audio(self):
		return self.__external_audio

	def get_external_audio_count(self):
		count = 0
		for audio in self.__external_audio:
			if audio:
				count += 1
		return count

	def set_file_time (self, ftime):
		if ftime < 0:
			return

		self.__file_time = ftime
		return

	def get_chunk_lenghts(self):
		result = []

		if self.__source == 'file' or self.__source == 'vcd':
			result.append(self.__file_time)
		elif self.__source == 'dvd':
			chapter_times = self.__dvd_chapter_times[self.__current_title]
			for chapters in self.__chunks:
				start, end = chapters
				i = start
				seconds = 0
				while i <= end:
					seconds += chapter_times[i]
					i += 1
				result.append(seconds)
		else:
			assert(0)
		return result

	def get_file_time (self):
		return self.__file_time

	def set_mix_subtitles(self, mix):
		self.__mix_subtitles = mix
		return

	def get_mix_subtitles(self):
		return self.__mix_subtitles

	def set_consider_ext_sub(self, consider):
		self.__consider_ext_sub = consider
		return

	def get_consider_ext_sub(self):
		return self.__consider_ext_sub

	def set_external_subtitle(self, sub):
		self.__extrenal_subtitle = None

		if not winki.lib.is_file(sub):
			return

		if not winki.lib.is_readable(sub):
			return

		self.__extrenal_subtitle = sub
		return

	def get_external_subtitle(self):
		return self.__extrenal_subtitle

	def set_aspect_ratio(self, aspect_ratio):
		self.__aspect_ratio = aspect_ratio
		return

	def get_aspect_ratio(self):
		return self.__aspect_ratio

	def set_scale_detected(self, x, y):
		nx = winki.lib.to_int(x)
		ny = winki.lib.to_int(y)
		if nx < 0 or ny < 0:
			winki.lib.debug('Project.set_scale_detected()', 'not a number %s %s' % (x, y), winki.lib.error)
			return

		self.__scale_detected = [nx, ny]
		return

	def get_scale_y(self):
		if not len(self.__scale_detected):
			winki.lib.debug('Project.get_scale_y()', 'scale_detected not set', winki.lib.error)
			return -1

		x, y = self.__scale_detected
		scale = int(y / 16)
		return scale * 16

	def get_scale_detected(self):
		return self.__scale_detected

	def set_crop(self, crop):
		self.__crop = [0, 0, 0, 0]

		if not crop or len(crop) != 4:
			winki.lib.debug('Project.set_crop()', 'invalid parameter passed', winki.lib.error)
			return False

		for item in crop:
			if item < 0:
				winki.lib.debug('Project.set_crop()', 'negative value', winki.lib.error)
				return False

		self.__crop = crop
		self.__crop = self.fix_crop()
		return True

	def get_crop(self):
		return self.__crop

	def has_crop(self):
		if not self.__crop or len(self.__crop) != 4:
			return False

		if self.__crop[0] < 1 and self.__crop[1] < 1 and self.__crop[2] < 1 and self.__crop[3] < 1:
			return False

		return True

	def fix_crop(self):
		if not self.__crop or len(self.__crop) != 4:
			winki.lib.debug('Project.fix_crop()', 'no crop values set')
			return self.__crop
			
		fixed_crop = []
		fixed_crop.extend(self.__crop)

		# if the width is not multiple of 16
		if self.__crop[0] % 16 != 0:
			# decrease the width to the previous multiple of 16
			fixed_crop[0] = int(self.__crop[0] / 16) * 16
			diff = self.__crop[0] - fixed_crop[0]
			sign = (float(diff) / 2.0) % 2
			# shift by even number of pixels in horizontal direction
			if math.fabs(sign - 0.0) < 1e-30 or math.fabs(sign - 0.5) < 1e-30:
				fixed_crop[2] += int(diff / 2)
			else:
				fixed_crop[2] += int(diff / 2) - 1

		# if the height is not multiple of 16
		if self.__crop[1] % 16 != 0:
			# decrease the height to the previous multiple of 16
			fixed_crop[1] = int(self.__crop[1] / 16) * 16
			diff = self.__crop[1] - fixed_crop[1]
			sign = (float(diff) / 2.0) % 2
			# shift by even number of pixels in vertical direction
			if math.fabs(sign - 0.0) < 1e-30 or math.fabs(sign - 0.5) < 1e-30:
				fixed_crop[3] += int(diff / 2)
			else:
				fixed_crop[3] += int(diff / 2) - 1
		
		return fixed_crop

	def compute_bitrate(self, fnames, len_seconds):
		if len_seconds == 0:
			return -1

		if self.__chunk_size < 0:
			return -1

		asize = self.sum_sizes(fnames, len_seconds)
		penalty_ratio = 0
		if self.__encoding == 'svcd':
		    penalty_ratio = self.__configuration.get_svcd_container_penalty()
		else:
		    penalty_ratio = self.__configuration.get_container_penalty()

		target = self.__chunk_sizes[self.__chunk_size] * 1024 * 1024
		penalty = target * penalty_ratio
		result = ((target - asize - penalty) / len_seconds / 125)
		result = self.correct_bitrate(result)

		return [result, target, penalty, asize]

	def correct_bitrate(self, vbitrate):
		result = vbitrate
		max_rate = -1

		# vrc_maxrate is set per default to 2600 for svcd
		if self.__encoding == 'svcd':
			opts = self.get_str_svcd_codec_options()
			max_rate = 2600

		# vrc_maxrate is set per default to 1152 for vcd
		elif self.__encoding == 'vcd':
			opts = self.get_str_vcd_codec_options()
			max_rate = 1152

		else:
			# todo: set a global maximum?
			return vbitrate

		pos1 = string.find(opts, 'vrc_maxrate=')
		if pos1 > 0:
			pos2 = string.find(opts, ':', pos1)
			if pos2 < 0 or pos2 < pos1:
				pos2 = len(opts)
			num = winki.lib.to_int(opts[pos1 + len('vrc_maxrate='): pos2])
			if num > 0:
				max_rate = num

		if vbitrate > max_rate:
			result = max_rate

		return result

	def sum_sizes(self, fnames, len_seconds):
		if self.__encoding == 'svcd':
			bps = self.__configuration.get_mp2_bytes_per_second()
			# just one audio for now
			return len_seconds * bps
			#asize = 0
			#for i in range(len(fnames)):
			#	asize += len_seconds * bps
			#return asize

		asize = 0
		for fname in fnames:
			size = winki.lib.file_size(fname)
			if size < 0:
				text = '%s not found' % fname
				winki.lib.debug('Project.sum_sizes()', text, winki.lib.error)
				continue
			asize += size
		return asize

	def get_user_mode(self):
		if self.__user_mode == None:
			self.__user_mode = self.__configuration.get_str_user_mode()
		return self.__user_mode

	def reset_dvd(self):
		self.__dvd_titles = None
		self.__dvd_chapters = None
		self.__dvd_chapter_times = None
		self.__dvd_audio = None
		self.__current_audio = [0, 0, 0]
		self.__dvd_subtitles = None
		self.__current_subtitles = [0, 0, 0]
		self.__current_title = -1
		self.__external_audio = [None, None, None]
		self.__mix_subtitles = False
		self.__file_time = -1
		self.__crop = [0, 0, 0, 0]
		self.__bitrate = -1
		self.__commands = []
		self.__chunk_size = -1
		self.__consider_ext_sub = False
		self.__extrenal_subtitle = None
		self.__scale_detected = []
		self.__aspect_ratio = None
		self.__mix_streams = False
		return

	def unlink_aux(self):
		self.unlink_file('divx2pass.log')
		return

	def unlink_file(self, fname):
		if not self.__dest_dirname:
			winki.lib.debug('Project.unlink_file()', 'working folder not set')
			return

		full_name = winki.lib.build_path([self.__dest_dirname, fname])
		full_name = winki.lib.remove_quotes(full_name)
		fnames = glob.glob(full_name)

		for file_name in fnames:
			if not os.path.exists(file_name):
				return

			try:
				os.unlink(file_name)
			except:
				winki.lib.debug('Project.unlink_file()', ('cannot unlink %s' % file_name), winki.lib.error)
		return

	def has_tool(self, tool):
		loc = self.get_tool_locations([tool])
		if not len(loc):
			return 0
		return len(loc[0])

	def get_tool_locations(self, tool_list):
		result = []

		for tool in tool_list:
			if tool == 'mplayer':
				result.append(self.get_mplayer_name())
			elif tool == 'mencoder':
				result.append(self.get_mencoder_name())
			elif tool == 'lsdvd':
				result.append(self.get_lsdvd_name())
			elif tool == 'vcdimager':
				result.append(self.get_vcdimager_name())
			elif tool == 'Audio compressor':
				result.append(self.get_audio_compressor_name())
			elif tool == 'Stream merger':
				result.append(self.get_stream_merger_name())
			elif tool == 'Multiplexer':
				result.append(self.get_mplexer_name())
			else:
				print 'unknown tool: %s' % tool
				assert(0)

		assert(len(tool_list) == len(result))
		return result

	def get_str_video_codec(self):
		return self.__configuration.get_str_video_codec()

	def get_str_video_codec_options(self):
		return self.__configuration.get_str_video_codec_options()

	def get_str_video_filter(self):
		return self.__configuration.get_str_video_filter()

	def get_str_vcd_codec(self):
		return self.__configuration.get_str_vcd_codec()

	def get_str_vcd_filter(self):
		return self.__configuration.get_str_vcd_filter()

	def get_str_vcd_codec_options(self):
		return self.__configuration.get_str_vcd_codec_options()

	def get_str_svcd_codec(self):
		return self.__configuration.get_str_svcd_codec()

	def get_str_svcd_filter(self):
		return self.__configuration.get_str_svcd_filter()

	def get_str_svcd_codec_options(self):
		return self.__configuration.get_str_svcd_codec_options()

	def get_vcd_bytes_per_second(self):
		return self.__configuration.get_vcd_bytes_per_second()

	def get_vcd_width(self):
		kind = self.__configuration.get_vcd_type()
		if self.__encoding == 'vcd':
			if kind == 'PAL' or kind == 'NTSC':
				return 352

		elif self.__encoding == 'svcd':
			if kind == 'PAL' or kind == 'NTSC':
				return 480

		return -1

	def get_vcd_height(self):
		kind = self.__configuration.get_vcd_type()

		if self.__encoding == 'vcd':
			if kind == 'PAL':
				return 288
			elif kind == 'NTSC':
				return 240

		elif self.__encoding == 'svcd':
			if kind == 'PAL':
				return 576
			elif kind == 'NTSC':
				return 480

		return -1

	def get_vcd_aspect(self):
		return 4.0 / 3.0

	def get_vcd_fps(self):
		kind = self.__configuration.get_vcd_type()

		if self.__encoding == 'svcd':
			if kind == 'PAL':
				return 25
			elif kind == 'NTSC':
				return 29.97

		elif self.__encoding == 'vcd':
			return 25

		return -1

	def get_str_user_defined(self):
		return self.__configuration.get_str_user_defined()

	def get_str_comment(self):
		return self.__configuration.get_str_comment()

	def get_nice_value(self):
		return self.__configuration.get_nice_value()

	def get_str_preferred_audio(self):
		return self.__configuration.get_str_preferred_audio()

	def get_str_preferred_subtitles(self):
		return self.__configuration.get_str_preferred_subtitles()

	def get_mplayer_name(self):
		return self.__configuration.get_str_mplayer_name()

	def get_mencoder_name(self):
		return self.__configuration.get_str_mencoder_name()

	def get_normalize_name(self):
		return self.__configuration.get_str_normalize_name()

	def get_lsdvd_name(self):
		return self.__configuration.get_str_lsdvd_name()

	def get_vcdimager_name(self):
		return self.__configuration.get_str_vcdimager_name()

	def get_mplexer_name(self):
		return self.__configuration.get_str_mplexer_name()

	def get_audio_compressor_name(self):
		return self.__configuration.get_str_audio_compressor()
		
	def get_audio_compressor_options(self):
		return self.__configuration.get_str_auido_compressor_options()

	def get_audio_compressor_ext(self):
		if self.__encoding == 'svcd':
			return 'mp2'

		return self.__configuration.get_str_audio_compressor_ext()

	def get_stream_merger_name(self):
		return self.__configuration.get_str_stream_merger()

	def get_lower_threshold(self):
		return self.__configuration.get_lower_threshold()

	def get_upper_threshold(self):
		return self.__configuration.get_upper_threshold()

	def get_use_warning(self, warning):
		return self.__configuration.get_use_warning(warning)

	def get_dvd_device(self):
		return self.__configuration.get_dvd_device()

	def get_cd_device(self):
		return self.__configuration.get_cd_device()

