# Winki the Ripper - GTK frontend for mencoder, mplayer and lsdvd
# Copyright (C) 2004 Velko Hristov
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

def error_quit(problem_id, do_quit):
	messages = ['Sorry. Cannot load the modules needed by winki the ripper']
	if problem_id < len(messages):
		print messages[problem_id]
	else:
		print 'oops...'
	if do_quit != 0:
		raise SystemExit

try:
	print 'Importing module string...'
	import string

	print 'Importing module winki.lib...'
	import winki.lib

	print 'Importing module winki.config...'
	import winki.config

	print 'Importing module winki.tool_finder...'
	import winki.tool_finder

except:
	error_quit(0, 1)

class WinkiRC:
	def __init__(self):
		self.configuration = winki.config.Config()
		return

	def has_global_rc(self):
		fname = '/etc/winkirc'

		if winki.lib.is_file(fname) and winki.lib.is_readable(fname):
			return 1

		return 0

	def has_local_rc(self):
		fname = winki.lib.normalize_path('~/.winkirc')

		if winki.lib.is_file(fname) and winki.lib.is_readable(fname):
			return 1

		return 0

	def get_version(self, fname):
		raw = ''
		version = None
		try:
			f = open(fname, 'r')
			raw = f.read()
			f.close()
		except:
			return version

		lines = string.split(raw, '\n')
		for line in lines:
			if string.find(line, '# winki version') > -1:
				items = string.split(line)
				version = items[3]
				break

		return version

	def update_local_rc(self, tool_list, obligatory_tools, meta_tools):
		if not self.has_local_rc():
			return self.create_local_rc(tool_list, obligatory_tools, meta_tools)

		local_fname = winki.lib.normalize_path('~/.winkirc')
		global_fname = winki.lib.normalize_path('/etc/winkirc')
		if not local_fname:
			winki.lib.debug('WinkiRC.update_local_rc()', 'cannot build local name', winki.lib.error)
			return 0

		local_version = self.get_version(local_fname)
		global_version = self.get_version(global_fname)
		if not local_version or not global_version:
			winki.lib.debug('WinkiRC.update_local_rc()', 'unknown file version', winki.lib.error)
			return 0

		if local_version == global_version:
			return 1

		print 'Updating your ~/.winkirc...'

		parser, dic = self.configuration.load_version(local_version, local_fname)
		if not parser or not dic:
			winki.lib.debug('WinkiRC.update_local_rc()', 'Cannot parse file %s' % local_fname, winki.lib.error)
			return 0

		if not self.update_local(parser, dic, local_fname, global_fname):
			winki.lib.debug('WinkiRC.update_local_rc()', 'Update failed', winki.lib.error)
			return 0

		if not self.fill_tool_list(tool_list, meta_tools):
			winki.lib.debug('WinkiRC.update_local_rc()', 'Tool update failed', winki.lib.error)
			return 0

		print 'Success'
		return 1


	def update_local(self, parser, dic, local_fname, global_fname):
		lines = winki.lib.read_lines(global_fname)
		if not lines:
			return 0

		cur_section = 'DEFAULT'
		result = []
		for line in lines:
			cur_section = self.find_section(line, cur_section)

			key, value = self.split_pair(line)
			if not key:
				result.append(line)
				continue

			user_setting = value
			if parser.has_option(cur_section, key):
				user_setting = parser.get(cur_section, key)

			line = self.modify_line(line, dic, key, user_setting)

			result.append(line)

		return winki.lib.write_lines(local_fname, result)

	def modify_line(self, line, dic, key, user_setting):
		result = line
		try:
			# if the user changed the default value - preserve it
			if dic[key] != user_setting:
				result = '%s = %s' % (key, user_setting)
		except KeyError:
			pass
		return result

	def fill_tool_list(self, tool_list, meta_tools):
		fname = winki.lib.normalize_path('~/.winkirc')
		lines = winki.lib.read_lines(fname)
		if not lines:
			return 0

		tool_finder = winki.tool_finder.ToolFinder()
		tool_finder.check_tools(tool_list, meta_tools)

		result = self.update_tools(lines, tool_finder, tool_list, meta_tools)
		result = self.update_meta_tools(result, meta_tools)

		return winki.lib.write_lines(fname, result)

	def create_local_rc(self, tool_list, obligatory_tools, meta_tools):
		error = 'Could not find /etc/winkirc. Please check your installation'
		if not self.has_global_rc():
			winki.lib.debug('WinkiRC', error, winki.lib.error)
			return 0

		print 'You seem to run WinkiTheRipper for the first time.'
		print 'Trying to find the needed programs...'

		tool_finder = winki.tool_finder.ToolFinder()
		if not tool_finder.check_tools(obligatory_tools, meta_tools):
			return 0

		infile_name = '/etc/winkirc'
		outfile_name = winki.lib.normalize_path('~/.winkirc')
		if not outfile_name:
			winki.lib.debug('WinkiRC', 'Failed to create ~/.winkirc', winki.lib.error)
			return 0

		lines = winki.lib.read_lines(infile_name)
		if not lines:
			winki.lib.debug('WinkiRC', 'Failed to create ~/.winkirc', winki.lib.error)
			return 0

		result = self.update_tools(lines, tool_finder, tool_list, meta_tools)
		result = self.update_meta_tools(result, meta_tools)

		if not winki.lib.write_lines(outfile_name, result):
			winki.lib.debug('WinkiRC', 'Failed to create ~/.winkirc', winki.lib.error)
			return 0

		print 'Success'
		return 1

	def update_tools(self, lines, tool_finder, tool_list, meta_tools):
		result = []
		for line in lines:
			for tool in tool_list:
				key, value = self.split_pair(line)
				if not key:
					continue

				expanded_tools = []
				for meta_tool in meta_tools:
					if tool == meta_tool:
						expanded_tools = meta_tools[meta_tool]

				if not len(expanded_tools):
					expanded_tools = [tool]
						
				for exp_tool in expanded_tools:
					if key == exp_tool:
						# if the user changed the default value - preserve it
						if value:
							break

						location = tool_finder.get_tool(exp_tool, meta_tools)
						if not location:
							break
						line = '%s = %s' % (key, location)
						break
			result.append(line)
		return result

	def update_meta_tools(self, lines, meta_tools):
		result = []
		for line in lines:
			result.append(line)

		for meta_tool in meta_tools:
			tools = meta_tools[meta_tool]
			for tool in tools:
				if self.has_value(tool, lines):
					self.set_value(meta_tool, tool, result)
					break

		return result

	def has_value(self, search_key, lines):
		for line in lines:
			key, value = self.split_pair(line)
			if not key:
				continue
				
			if key == search_key and len(string.strip(value)):
				return 1
		return 0

	def set_value(self, search_key, value_to_set, lines):
		for i in range(len(lines)):
			key, value = self.split_pair(lines[i])
			if not key:
				continue

			if key == search_key:
				lines[i] = '%s = %s' % (key, value_to_set)
				break
		return

	def split_pair(self, line):
		key = None
		value = None
		if not line or not len(line) or string.find(line, '=') < 0:
			return key, value

		line = string.strip(line)
		if line[0] == '#':
			return key, value

		items = string.split(line, '=')
		if len(items) < 2:
			return key, value

		key = items[0]
		value = ''
		for i in range(len(items)):
			if i == 0:
				continue

			value += items[i]
			if i < (len(items) - 1):
				value += '='

		key = string.strip(key)
		value = string.strip(value)

		return key, value

	def find_section(self, line, current):
		pos1 =string.find(line, '[')
		if pos1 < 0:
			return current

		pos2 = string.find(line, ']')
		if pos2 < 0 or pos2 < pos1:
			return current

		return line[pos1 + 1 : pos2]
